@@ -23,7 +23,7 @@
 #include "lua/luatex-api.h"
 
 static const char _svn_version[] =
-    "$Id: luafont.c 3011 2009-08-17 14:15:46Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/luafont.c $";
+    "$Id: luafont.c 3246 2009-12-10 22:22:21Z hhenkel $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/luafont.c $";
 
 #define noVERBOSE
 
@@ -97,6 +97,8 @@ const char *MATH_param_names[] = {
     "RadicalDegreeBottomRaisePercent",
     "MinConnectorOverlap",
     "SubscriptShiftDownWithSuperscript",
+    "FractionDelimiterSize",
+    "FractionDelimiterDisplayStyleSize",
     NULL,
 };
 
@@ -434,8 +436,13 @@ int font_to_lua(lua_State * L, int f)
     lua_setfield(L, -2, "checksum");
     lua_pushnumber(L, font_slant(f));
     lua_setfield(L, -2, "slant");
-    lua_pushnumber(L, font_extend(f));
+
+    if (font_extend(f) == 1000)
+        lua_pushnumber(L, 0);   /* maybe this special case should be removed,... */
+    else
+        lua_pushnumber(L, font_extend(f));      /* ...better 1000 = nonextended/natural */
     lua_setfield(L, -2, "extend");
+
     lua_pushnumber(L, font_natural_dir(f));
     lua_setfield(L, -2, "direction");
     lua_pushnumber(L, font_encodingbytes(f));
@@ -485,7 +492,9 @@ int font_to_lua(lua_State * L, int f)
         lua_setfield(L, -2, "auto_expand");
     }
     if (pdf_font_attr(f) != 0) {
-        lua_pushstring(L, makecstring(pdf_font_attr(f)));
+        char *s = makecstring(pdf_font_attr(f));
+        lua_pushstring(L, s);
+        free(s);
         lua_setfield(L, -2, "attributes");
     }
 
@@ -1498,8 +1507,8 @@ int font_from_lua(lua_State * L, int f)
         i = -2000;
     if (i > 2000)
         i = 2000;
-    if (i == 1000)
-        i = 0;
+    if (i == 0)
+        i = 1000;
     set_font_extend(f, i);
     i = numeric_field(L, "slant", 0);
     if (i < -1000)
@@ -1603,6 +1612,7 @@ int font_from_lua(lua_State * L, int f)
     lua_getfield(L, -1, "characters");
     if (lua_istable(L, -1)) {
         /* find the array size values */
+        int num = 0;            /* number of charinfo's to add */
         ec = 0;
         bc = -1;
         lua_pushnil(L);         /* first key */
@@ -1611,6 +1621,7 @@ int font_from_lua(lua_State * L, int f)
                 i = lua_tointeger(L, -2);
                 if (i >= 0) {
                     if (lua_istable(L, -1)) {
+                        num++;
                         if (i > ec)
                             ec = i;
                         if (bc < 0)
@@ -1622,9 +1633,9 @@ int font_from_lua(lua_State * L, int f)
             }
             lua_pop(L, 1);
         }
-
         if (bc != -1) {
             /* fprintf(stdout,"defined a font at %d with %d-%d\n",f,bc,ec); */
+            font_malloc_charinfo(f, num);
             set_font_bc(f, bc);
             set_font_ec(f, ec);
             lua_pushnil(L);     /* first key */
