@@ -26,8 +26,8 @@
 
 
 static const char _svn_version[] =
-    "$Id: mapfile.c 2869 2009-07-16 11:31:18Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/mapfile.c $";
+    "$Id: mapfile.c 3247 2009-12-10 23:57:37Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/mapfile.c $";
 
 #define FM_BUF_SIZE     1024
 
@@ -37,8 +37,8 @@ static unsigned char *fm_buffer = NULL;
 static integer fm_size = 0;
 static integer fm_curbyte = 0;
 
-#define fm_open()       \
-    (fm_file = fopen((char *) nameoffile + 1, FOPEN_RBIN_MODE))
+#define fm_open(a)       \
+    (fm_file = fopen((char *)(a), FOPEN_RBIN_MODE))
 #define fm_read_file()  \
     readbinfile(fm_file,&fm_buffer,&fm_size)
 #define fm_close()      xfclose(fm_file, cur_file_name)
@@ -48,7 +48,6 @@ static integer fm_curbyte = 0;
     (c == 10 || c == '*' || c == '#' || c == ';' || c == '%')
 
 typedef enum { FM_DUPIGNORE, FM_REPLACE, FM_DELETE } updatemode;
-typedef enum { MAPFILE, MAPLINE } maptype;
 
 typedef struct mitem {
     updatemode mode;            /* FM_DUPIGNORE or FM_REPLACE or FM_DELETE */
@@ -87,7 +86,7 @@ fm_entry *new_fm_entry(void)
     fm->encname = NULL;
     fm->type = 0;
     fm->slant = 0;
-    fm->extend = 0;
+    fm->extend = 1000;
     fm->links = 0;
     fm->pid = -1;
     fm->eid = -1;
@@ -143,7 +142,7 @@ static int comp_fm_entry_tfm(const void *pa, const void *pb, void *p)
                   ((const fm_entry *) pb)->tfm_name);
 }
 
-/* AVL sort fm_entry into ps_tree by ps_name, slant, and extend */
+/* AVL sort fm_entry into ps_tree by ps_name and extend */
 
 static int comp_fm_entry_ps(const void *pa, const void *pb, void *p)
 {
@@ -153,7 +152,6 @@ static int comp_fm_entry_ps(const void *pa, const void *pb, void *p)
     assert(p1->ps_name != NULL && p2->ps_name != NULL);
     if ((i = strcmp(p1->ps_name, p2->ps_name)))
         return i;
-    cmp_return(p1->slant, p2->slant);
     cmp_return(p1->extend, p2->extend);
     return 0;
 }
@@ -320,12 +318,11 @@ int check_fm_entry(fm_entry * fm, boolean warn)
         a += 2;
     }
 
-    /* SlantFont and ExtendFont can be used only with Type1 fonts */
-    if ((fm->slant != 0 || fm->extend != 0)
-        && !(is_t1fontfile(fm) && is_included(fm))) {
+    /* ExtendFont can be used only with Type1 fonts */
+    if (fm->extend != 1000 && !(is_t1fontfile(fm) && is_included(fm))) {
         if (warn)
             pdftex_warn
-                ("invalid entry for `%s': SlantFont/ExtendFont can be used only with embedded Type1 fonts",
+                ("invalid entry for `%s': ExtendFont can be used only with embedded Type1 fonts",
                  fm->tfm_name);
         a += 4;
     }
@@ -482,8 +479,8 @@ static void fm_scan_line(void)
                     } else if (str_prefix(s, "ExtendFont")) {
                         d *= 1000.0;
                         fm->extend = (integer) (d > 0 ? d + 0.5 : d - 0.5);
-                        if (fm->extend == 1000)
-                            fm->extend = 0;
+                        if (fm->extend == 0)    /* special user case... */
+                            fm->extend = 1000;  /* ...mapped to natural internal representation */
                         r = s + strlen("ExtendFont");
                     } else {    /* unknown name */
                         for (r = s; *r != ' ' && *r != '"' && *r != '\0'; r++); /* jump over name */
@@ -581,7 +578,7 @@ void fm_read_info(void)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
+
     if (tfm_tree == NULL)
         create_avl_trees();
     if (mitem->line == NULL)    /* nothing to do */
@@ -589,75 +586,52 @@ void fm_read_info(void)
     mitem->lineno = 1;
     switch (mitem->type) {
     case MAPFILE:
-        set_cur_file_name(mitem->line);
         if (fm_buffer != NULL) {
             xfree(fm_buffer);
             fm_buffer = NULL;
         }
         fm_curbyte = 0;
         fm_size = 0;
-        callback_id = callback_defined(find_map_file_callback);
-        if (callback_id > 0) {
-            if (run_callback
-                (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-                if (ftemp != NULL && strlen(ftemp)) {
-                    free(nameoffile);
-                    namelength = strlen(ftemp);
-                    nameoffile = xmalloc(namelength + 2);
-                    strcpy((char *) (nameoffile + 1), ftemp);
-                    free(ftemp);
-                }
-            }
-        } else {
-            ftemp =
-                kpse_find_file((char *) (nameoffile + 1), kpse_fontmap_format,
-                               0);
-            if (ftemp != NULL) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
-        callback_id = callback_defined(read_map_file_callback);
-        if (callback_id > 0) {
-            if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
-                             &file_opened, &fm_buffer, &fm_size)) {
-                if (file_opened) {
-                    if (fm_size > 0) {
-                        cur_file_name = (char *) nameoffile + 1;
-                        if (tracefilenames)
-                            tex_printf("{%s", cur_file_name);
-                        while (!fm_eof()) {
-                            fm_scan_line();
-                            mitem->lineno++;
+        cur_file_name = luatex_find_file(mitem->line, find_map_file_callback);
+        if (cur_file_name) {
+            callback_id = callback_defined(read_map_file_callback);
+            if (callback_id > 0) {
+                if (run_callback(callback_id, "S->bSd", cur_file_name,
+                                 &file_opened, &fm_buffer, &fm_size)) {
+                    if (file_opened) {
+                        if (fm_size > 0) {
+                            if (tracefilenames)
+                                tex_printf("{%s", cur_file_name);
+                            while (!fm_eof()) {
+                                fm_scan_line();
+                                mitem->lineno++;
+                            }
+                            if (tracefilenames)
+                                tex_printf("}");
+                            fm_file = NULL;
                         }
-                        if (tracefilenames)
-                            tex_printf("}");
-                        fm_file = NULL;
+                    } else {
+                        pdftex_warn("cannot open font map file");
                     }
                 } else {
                     pdftex_warn("cannot open font map file");
                 }
             } else {
-                pdftex_warn("cannot open font map file");
-            }
-        } else {
-            if (!fm_open()) {
-                pdftex_warn("cannot open font map file");
-            } else {
-                fm_read_file();
-                cur_file_name = (char *) nameoffile + 1;
-                tex_printf("{%s", cur_file_name);
-                while (!fm_eof()) {
-                    fm_scan_line();
-                    mitem->lineno++;
+                if (!fm_open(cur_file_name)) {
+                    pdftex_warn("cannot open font map file");
+                } else {
+                    fm_read_file();
+                    tex_printf("{%s", cur_file_name);
+                    while (!fm_eof()) {
+                        fm_scan_line();
+                        mitem->lineno++;
+                    }
+                    fm_close();
+                    tex_printf("}");
+                    fm_file = NULL;
                 }
-                fm_close();
-                tex_printf("}");
-                fm_file = NULL;
             }
+            cur_file_name = NULL;
         }
         break;
     case MAPLINE:
@@ -682,6 +656,8 @@ static fm_entry_ptr fmlookup(internalfontnumber f)
     if (tfm_tree == NULL)
         fm_read_info();         /* only to read default map file */
     tfm = font_name(f);
+    if (tfm == NULL)            /* wide, lua loaded fonts may not have a name */
+        return (fm_entry_ptr) dummy_fm_entry();
     assert(strcmp(tfm, nontfm));
 
     /* Look up for full <tfmname>[+-]<expand> */
@@ -699,6 +675,8 @@ boolean hasfmentry(internalfontnumber f)
     if (font_map(f) == NULL)
         set_font_map(f, (fm_entry_ptr) fmlookup(f));
     assert(font_map(f) != NULL);
+    font_slant(f) = ((fm_entry *) font_map(f))->slant;
+    font_extend(f) = ((fm_entry *) font_map(f))->extend;
     return font_map(f) != (fm_entry_ptr) dummy_fm_entry();
 }
 
@@ -728,8 +706,8 @@ static boolean fm_valid_for_font_replacement(fm_entry * fm)
 fm_entry *lookup_fontmap(char *ps_name)
 {
     fm_entry *fm, *fm2, tmp;
-    char *a, *b, *c, *d, *e, *s;
-    int i, sl, ex;
+    char *a, *b, *e, *s;
+    int i, ex;
     struct avl_traverser t, t2;
     if (tfm_tree == NULL)
         fm_read_info();         /* only to read default map file */
@@ -746,43 +724,17 @@ fm_entry *lookup_fontmap(char *ps_name)
     }
 
     /*
-     * Scan -Slant_<slant> and -Extend_<extend> font name extensions;
-     * three valid formats:
-     * <fontname>-Slant_<slant>
-     * <fontname>-Slant_<slant>-Extend_<extend>
+     * Scan -Extend_<extend> font name extensions; format:
      * <fontname>-Extend_<extend>
-     * Slant entry must come _before_ Extend entry
      */
 
-    tmp.slant = 0;
-    tmp.extend = 0;
-    if ((a = strstr(s, "-Slant_")) != NULL) {
-        b = a + strlen("-Slant_");
-        sl = (int) strtol(b, &e, 10);
+    tmp.extend = 1000;
+    if ((a = strstr(s, "-Extend_")) != NULL) {
+        b = a + strlen("-Extend_");
+        ex = (int) strtol(b, &e, 10);
         if ((e != b) && (e == strend(b))) {
-            tmp.slant = sl;
-            *a = '\0';          /* ps_name string ends before "-Slant_" */
-        } else {
-            if (e != b) {       /* only if <slant> is valid number */
-                if ((c = strstr(e, "-Extend_")) != NULL) {
-                    d = c + strlen("-Extend_");
-                    ex = (int) strtol(d, &e, 10);
-                    if ((e != d) && (e == strend(d))) {
-                        tmp.slant = sl;
-                        tmp.extend = ex;
-                        *a = '\0';      /* ps_name string ends before "-Slant_" */
-                    }
-                }
-            }
-        }
-    } else {
-        if ((a = strstr(s, "-Extend_")) != NULL) {
-            b = a + strlen("-Extend_");
-            ex = (int) strtol(b, &e, 10);
-            if ((e != b) && (e == strend(b))) {
-                tmp.extend = ex;
-                *a = '\0';      /* ps_name string ends before "-Extend_" */
-            }
+            tmp.extend = ex;
+            *a = '\0';          /* ps_name string ends before "-Extend_" */
         }
     }
     tmp.ps_name = s;
