@@ -38,7 +38,7 @@
 #define noDEBUG
 
 static const char _svn_version[] =
-    "$Id: texfont.c 3011 2009-08-17 14:15:46Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/texfont.c $";
+    "$Id: texfont.c 3218 2009-12-04 08:21:56Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/texfont.c $";
 
 #define proper_char_index(c) (c<=font_ec(f) && c>=font_bc(f))
 #define dxfree(a,b) { xfree(a); a = b ; }
@@ -139,6 +139,15 @@ integer new_font(void)
     return id;
 }
 
+void font_malloc_charinfo(internal_font_number f, int num)
+{
+    int glyph = font_tables[f]->charinfo_size;
+    font_bytes += (num * sizeof(charinfo));
+    do_realloc(font_tables[f]->charinfo, (glyph + num), charinfo);
+    memset(&(font_tables[f]->charinfo[glyph]), 0, (num * sizeof(charinfo)));
+    font_tables[f]->charinfo_size += num;
+}
+
 #define find_charinfo_id(f,c) get_sa_item(font_tables[f]->characters,c)
 
 charinfo *get_charinfo(internal_font_number f, integer c)
@@ -151,11 +160,7 @@ charinfo *get_charinfo(internal_font_number f, integer c)
 
             glyph = ++font_tables[f]->charinfo_count;
             if (glyph >= (unsigned) font_tables[f]->charinfo_size) {
-                font_bytes += (16 * sizeof(charinfo));
-                do_realloc(font_tables[f]->charinfo, (glyph + 16), charinfo);
-                memset(&(font_tables[f]->charinfo[glyph]), 0,
-                       (16 * sizeof(charinfo)));
-                font_tables[f]->charinfo_size += 16;
+                font_malloc_charinfo(f, 256);
             }
             font_tables[f]->charinfo[glyph].ef = 1000;  /* init */
             set_sa_item(font_tables[f]->characters, c, glyph, 1);       /* 1= global */
@@ -260,7 +265,6 @@ charinfo *copy_charinfo(charinfo * ci)
         set_charinfo_hor_variants(co,
                                   copy_variants(get_charinfo_hor_variants(ci)));
     }
-    /* TODO: copy mathkerns */
     x = ci->top_left_math_kerns;
     co->top_left_math_kerns = x;
     if (x > 0) {
@@ -1032,11 +1036,21 @@ void set_font_math_params(internal_font_number f, int b)
 
 integer copy_font(integer f)
 {
-    int i;
+    int i, ci_cnt, ci_size;
     charinfo *ci;
     integer k = new_font();
-    memcpy(font_tables[k], font_tables[f], sizeof(texfont));
 
+    {
+        ci = font_tables[k]->charinfo;
+        ci_cnt = font_tables[k]->charinfo_count;
+        ci_size = font_tables[k]->charinfo_size;
+        memcpy(font_tables[k], font_tables[f], sizeof(texfont));
+        font_tables[k]->charinfo = ci;
+        font_tables[k]->charinfo_count = ci_cnt;
+        font_tables[k]->charinfo_size = ci_size;
+    }
+
+    font_malloc_charinfo(k, font_tables[f]->charinfo_count);
     set_font_cache_id(k, 0);
     set_font_used(k, 0);
     set_font_touched(k, 0);
@@ -1080,11 +1094,7 @@ integer copy_font(integer f)
         memcpy(math_param_base(k), math_param_base(f), i);
     }
 
-    i = sizeof(charinfo) * (font_tables[f]->charinfo_size + 1);
-    font_bytes += i;
-    font_tables[k]->charinfo = xmalloc(i);
-    memset(font_tables[k]->charinfo, 0, i);
-    for (i = 0; i < font_tables[k]->charinfo_size; i++) {
+    for (i = 0; i < font_tables[f]->charinfo_count; i++) {
         ci = copy_charinfo(&font_tables[f]->charinfo[i]);
         font_tables[k]->charinfo[i] = *ci;
     }
@@ -1176,6 +1186,7 @@ boolean cmp_font_name(integer id, str_number t)
         return 1;
     if (tt == NULL || tid == NULL || strcmp(tid, tt) != 0)
         return 0;
+    free(tt);
     return 1;
 }
 
@@ -1194,6 +1205,7 @@ boolean cmp_font_area(integer id, str_number t)
         return 1;
     if (tt == NULL || strcmp(tid, tt) != 0)
         return 0;
+    free(tt);
     return 1;
 }
 
