@@ -22,7 +22,7 @@
 #include "luatexfont.h"
 
 static const char _svn_version[] =
-    "$Id: tfmofm.c 2869 2009-07-16 11:31:18Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/tfmofm.c $";
+    "$Id: tfmofm.c 3187 2009-11-22 22:29:05Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/tfmofm.c $";
 
 /* Here are some macros that help process ligatures and kerns */
 
@@ -381,25 +381,24 @@ int open_tfm_file(char *nom, unsigned char **tfm_buf, integer * tfm_siz)
     boolean opened;             /* was |tfm_file| successfully opened? */
     integer callback_id;
     FILE *tfm_file;
-    /* packfilename(nom,aire,getnullstr()); */
-    if (nameoffile != NULL)
-        xfree(nameoffile);
-    nameoffile = malloc(strlen(nom) + 2);
-    strcpy(stringcast(nameoffile + 1), nom);
-    namelength = strlen(nom);
+    char *fname = luatex_find_file(nom, find_font_file_callback);
+    if (!fname)
+        return -1;
     callback_id = callback_defined(read_font_file_callback);
     if (callback_id > 0) {
-        res = run_callback(callback_id, "S->bSd", stringcast(nameoffile + 1),
-                           &opened, tfm_buf, tfm_siz);
+        res =
+            run_callback(callback_id, "S->bSd", fname, &opened, tfm_buf,
+                         tfm_siz);
         if (res && opened && (*tfm_siz > 0)) {
             return 1;
         }
         if (!opened)
             return -1;
     } else {
-        if (ofm_open_in(tfm_file)) {
+        if (luatex_open_input
+            (&(tfm_file), fname, kpse_ofm_format, FOPEN_RBIN_MODE, true)) {
             res = read_tfm_file(tfm_file, tfm_buf, tfm_siz);
-            b_close(tfm_file);
+            close_file(tfm_file);
             if (res) {
                 return 1;
             }
