@@ -23,8 +23,8 @@
 #include "luatexfont.h"
 
 static const char _svn_version[] =
-    "$Id: vfovf.c 2869 2009-07-16 11:31:18Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/vfovf.c $";
+    "$Id: vfovf.c 3218 2009-12-04 08:21:56Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/vfovf.c $";
 
 /* this is a hack! */
 #define font_max 5000
@@ -254,7 +254,7 @@ vf_def_font(internal_font_number f, unsigned char *vf_buffer, integer * vf_cr)
         tmp_b0--;
         (*vf_cr)++;             /* skip the font path */
     }
-    str_room(tmp_b1);
+    str_room((unsigned)tmp_b1);
     while (tmp_b1 > 0) {
         tmp_b1--;
         junk = vf_buffer[(*vf_cr)];
@@ -287,34 +287,15 @@ int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
 {
     boolean res;                /* was the callback successful? */
     integer callback_id;
-    boolean file_read;          /* was |vf_file| successfully read? */
+    boolean file_read = false;  /* was |vf_file| successfully read? */
     FILE *vf_file;
-    char *fnam = NULL;
+    char *fname = luatex_find_file(fn, find_vf_file_callback);
+    if (!fname)
+        fname = fn;
 
-
-
-
-    namelength = strlen(fn);
-    nameoffile = xmalloc(namelength + 2);
-    strcpy((char *) (nameoffile + 1), fn);
-
-    callback_id = callback_defined(find_vf_file_callback);
-    if (callback_id > 0) {
-        res = run_callback(callback_id, "S->S", fn, &fnam);
-        if (res && (fnam != NULL) && (strlen(fnam) > 0)) {
-            /* @<Fixup |nameoffile| after callback@>; */
-            free(nameoffile);
-            namelength = strlen(fnam);
-            nameoffile = xmalloc(namelength + 2);
-            strcpy((char *) (nameoffile + 1), fnam);
-        } else {
-            return 0;
-        }
-    }
     callback_id = callback_defined(read_vf_file_callback);
     if (callback_id > 0) {
-        file_read = false;
-        res = run_callback(callback_id, "S->bSd", stringcast(nameoffile + 1),
+        res = run_callback(callback_id, "S->bSd", fname,
                            &file_read, vbuffer, vsize);
         if (res && file_read && (*vsize > 0)) {
             return 1;
@@ -322,9 +303,12 @@ int open_vf_file(char *fn, unsigned char **vbuffer, integer * vsize)
         if (!file_read)
             return 0;           /* -1 */
     } else {
-        if (ovf_b_open_in(vf_file) || vf_b_open_in(vf_file)) {
+        if (luatex_open_input
+            (&(vf_file), fname, kpse_ovf_format, FOPEN_RBIN_MODE, false)
+            || luatex_open_input(&(vf_file), fname, kpse_vf_format,
+                                 FOPEN_RBIN_MODE, false)) {
             res = read_vf_file(vf_file, vbuffer, vsize);
-            b_close(vf_file);
+            close_file(vf_file);
             if (res) {
                 return 1;
             }
@@ -1438,7 +1422,6 @@ boolean auto_expand_vf(internal_font_number f)
     return true;
 }
 
-/* this function is not always safe, because it needs makecstring() */
 str_number expand_font_name(internal_font_number f, integer e)
 {
     int old_setting;
