@@ -21,8 +21,8 @@
 #include "ptexlib.h"
 
 static const char _svn_version[] =
-    "$Id: vfpacket.c 3007 2009-08-16 21:18:16Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/vfpacket.c $";
+    "$Id: vfpacket.c 3217 2009-12-03 16:52:01Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/vfpacket.c $";
 
 /*
   The |do_vf_packet| procedure is called in order to interpret the
@@ -208,7 +208,7 @@ void do_vf_packet(PDF pdf, internal_font_number vf_f, integer c)
             break;
         case packet_special_code:
             packet_number(k);
-            str_room(k);
+            str_room((unsigned)k);
             while (k > 0) {
                 k--;
                 append_char(do_packet_byte());
