@@ -23,7 +23,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: writet1.c 2899 2009-07-21 23:03:53Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writet1.c $";
+    "$Id: writet1.c 3250 2009-12-12 14:52:43Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/writet1.c $";
 
 #define t1_log(str)      if(tracefilenames) tex_printf("%s", str)
 #define get_length1()    t1_length1 = t1_offset() - t1_save_offset
@@ -72,8 +72,8 @@ static unsigned char *enc_buffer = NULL;
 static integer enc_size = 0;
 static integer enc_curbyte = 0;
 
-#define enc_open()          \
-    (enc_file = fopen((char *)nameoffile+1, FOPEN_RBIN_MODE))
+#define enc_open(a)          \
+    (enc_file = fopen((char *)(a), FOPEN_RBIN_MODE))
 #define enc_read_file()  \
     readbinfile(enc_file,&enc_buffer,&enc_size)
 #define enc_close()       xfclose(enc_file,cur_file_name)
@@ -266,50 +266,28 @@ char **load_enc_file(char *enc_name)
 {
     int callback_id = 0;
     int file_opened = 0;
-    char *ftemp = NULL;
+
     char buf[ENC_BUF_SIZE], *p, *r;
     int i, names_count;
     char **glyph_names;
-    char *tempname;
-    set_cur_file_name(enc_name);
-    callback_id = callback_defined(find_enc_file_callback);
-    if (callback_id > 0) {
-        if (run_callback
-            (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-            if (ftemp != NULL && strlen(ftemp)) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            } else {
-                pdftex_fail("cannot find encoding file for reading");
-            }
-        }
-    } else {
-        ftemp = kpse_find_file((char *) (nameoffile + 1), kpse_enc_format, 0);
-        if (ftemp == NULL) {
-            pdftex_fail("cannot find encoding file for reading");
-        } else {
-            free(nameoffile);
-            namelength = strlen(ftemp);
-            nameoffile = xmalloc(namelength + 2);
-            strcpy((char *) (nameoffile + 1), ftemp);
-            free(ftemp);
-        }
+
+    cur_file_name = luatex_find_file(enc_name, find_enc_file_callback);
+
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot find encoding file for reading");
     }
     callback_id = callback_defined(read_enc_file_callback);
     enc_curbyte = 0;
     enc_size = 0;
     if (callback_id > 0) {
-        if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
+        if (run_callback(callback_id, "S->bSd", cur_file_name,
                          &file_opened, &enc_buffer, &enc_size)) {
             if ((!file_opened) || enc_size == 0) {
                 pdftex_fail("cannot open encoding file for reading");
             }
         }
     } else {
-        if (!enc_open()) {
+        if (!enc_open(cur_file_name)) {
             pdftex_fail("cannot open encoding file for reading");
         }
         enc_read_file();
@@ -320,9 +298,6 @@ char **load_enc_file(char *enc_name)
         glyph_names[i] = (char *) notdef;
     t1_log("{");
 
-    tempname = xmalloc(namelength + 2);
-    strcpy(tempname, (char *) (nameoffile + 1));
-    set_cur_file_name(tempname);
     t1_log(cur_file_name);
     enc_getline();
     if (*enc_line != '/' || (r = strchr(enc_line, '[')) == NULL) {
@@ -710,9 +685,7 @@ static void t1_modify_fm(void)
         a[i] = t1_scan_num(p, &q);
         p = q;
     }
-    if (fm_slant(fd_cur->fm) != 0)
-        do_slant(a, fm_slant(fd_cur->fm) * 1E-3);
-    if (fm_extend(fd_cur->fm) != 0)
+    if (fm_extend(fd_cur->fm) != 1000)
         do_extend(a, fm_extend(fd_cur->fm) * 1E-3);
     for (i = 0; i < 6; i++) {
         sprintf(r, "%g ", a[i]);
@@ -736,38 +709,16 @@ static void t1_modify_fm(void)
     t1_line_ptr = eol(t1_line_array);
 }
 
-static void t1_modify_italic(void)
-{
-    float a;
-    char *p, *r;
-    if (fm_slant(fd_cur->fm) == 0)
-        return;
-    p = strchr(t1_line_array, ' ');
-    strncpy(t1_buf_array, t1_line_array, (size_t) (p - t1_line_array + 1));
-    a = t1_scan_num(p + 1, &r);
-    a -= atan(fm_slant(fd_cur->fm) * 1E-3) * (180 / M_PI);
-    sprintf(t1_buf_array + (p - t1_line_array + 1), "%g", a);
-    strcpy(strend(t1_buf_array), r);
-    strcpy(t1_line_array, t1_buf_array);
-    t1_line_ptr = eol(t1_line_array);
-    fd_cur->font_dim[ITALIC_ANGLE_CODE].val = round(a);
-    fd_cur->font_dim[ITALIC_ANGLE_CODE].set = true;
-}
-
 static void t1_scan_keys(PDF pdf)
 {
     int i, k;
     char *p, *q, *r;
     key_entry *key;
-    if (fm_extend(fd_cur->fm) != 0 || fm_slant(fd_cur->fm) != 0) {
+    if (fm_extend(fd_cur->fm) != 1000) {
         if (t1_prefix("/FontMatrix")) {
             t1_modify_fm();
             return;
         }
-        if (t1_prefix("/ItalicAngle")) {
-            t1_modify_italic();
-            return;
-        }
     }
     if (t1_prefix("/FontType")) {
         p = t1_line_array + strlen("FontType") + 1;
@@ -792,11 +743,7 @@ static void t1_scan_keys(PDF pdf)
         r = ++p;                /* skip the slash */
         for (q = t1_buf_array; *p != ' ' && *p != 10; *q++ = *p++);
         *q = 0;
-        if (fm_slant(fd_cur->fm) != 0) {
-            sprintf(q, "-Slant_%i", (int) fm_slant(fd_cur->fm));
-            q = strend(q);
-        }
-        if (fm_extend(fd_cur->fm) != 0) {
+        if (fm_extend(fd_cur->fm) != 1000) {
             sprintf(q, "-Extend_%i", (int) fm_extend(fd_cur->fm));
         }
         xfree(fd_cur->fontname);
@@ -992,44 +939,35 @@ static void t1_check_end(PDF pdf)
 
 static boolean t1_open_fontfile(const char *open_name_prefix)
 {
+    ff_entry *ff;
     int callback_id = 0;
     int file_opened = 0;
-    char *ftemp = NULL;
-    ff_entry *ff;
-    ff = check_ff_exist(fd_cur->fm->ff_name, is_truetype(fd_cur->fm));
     t1_curbyte = 0;
     t1_size = 0;
-    if (ff->ff_path != NULL) {
-        callback_id = callback_defined(find_type1_file_callback);
-        if (callback_id > 0) {
-            if (run_callback(callback_id, "S->S", ff->ff_path, &ftemp)) {
-                if (ftemp != NULL && strlen(ftemp)) {
-                    strcpy(ff->ff_path, ftemp);
-                }
-            }
-        }
-        callback_id = callback_defined(read_type1_file_callback);
-        if (callback_id > 0) {
-            if (run_callback(callback_id, "S->bSd", ff->ff_path,
-                             &file_opened, &t1_buffer, &t1_size)
-                && file_opened && t1_size > 0) {
-                cur_file_name = ff->ff_path;
-            } else {
-                set_cur_file_name(fd_cur->fm->ff_name);
-                pdftex_warn("cannot open Type 1 font file for reading");
-                return false;
-            }
-        } else {
-            t1_file = xfopen(cur_file_name = ff->ff_path, FOPEN_RBIN_MODE);
-            t1_read_file();
-            t1_close();
-        }
-        recorder_record_input(ff->ff_path);
-    } else {
-        set_cur_file_name(fd_cur->fm->ff_name);
+    ff = check_ff_exist(fd_cur->fm->ff_name, is_truetype(fd_cur->fm));
+    if (ff->ff_path == NULL) {
         pdftex_fail("cannot open Type 1 font file for reading");
         return false;
     }
+    cur_file_name = luatex_find_file(ff->ff_path, find_type1_file_callback);
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot open Type 1 font file for reading");
+        return false;
+    }
+    callback_id = callback_defined(read_type1_file_callback);
+    if (callback_id > 0) {
+        if (!run_callback(callback_id, "S->bSd", cur_file_name,
+                          &file_opened, &t1_buffer, &t1_size)
+            && file_opened && t1_size > 0) {
+            pdftex_warn("cannot open Type 1 font file for reading");
+            return false;
+        }
+    } else {
+        t1_file = xfopen(cur_file_name, FOPEN_RBIN_MODE);
+        t1_read_file();
+        t1_close();
+    }
+    recorder_record_input(cur_file_name);
     t1_init_params(open_name_prefix);
     return true;
 }
