@@ -25,7 +25,7 @@
 #include "luatexfont.h"
 
 static const char _svn_version[] =
-    "$Id: writet3.c 2684 2009-06-29 13:01:53Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writet3.c $";
+    "$Id: writet3.c 3236 2009-12-05 11:43:29Z hhenkel $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/writet3.c $";
 
 #define T3_BUF_SIZE   1024
 
@@ -186,7 +186,7 @@ static boolean writepk(PDF pdf, internal_font_number f)
         ury = cd.cheight + lly;
         update_bbox(llx, lly, urx, ury, t3_glyph_num == 0);
         t3_glyph_num++;
-        pdf_new_dict(pdf, 0, 0, 0);
+        pdf_new_dict(pdf, obj_type_others, 0, 0);
         t3_char_procs[cd.charcode] = pdf->obj_ptr;
         pdf_begin_stream(pdf);
         pdf_print_real(pdf, t3_char_widths[cd.charcode], 2);
@@ -230,15 +230,15 @@ void writet3(PDF pdf, int objnum, internalfontnumber f)
     integer pk_font_scale;
     boolean is_notdef;
 
-
     t3_glyph_num = 0;
     t3_image_used = false;
     for (i = 0; i < 256; i++) {
         t3_char_procs[i] = 0;
         t3_char_widths[i] = 0;
     }
-    pack_file_name(tex_font_name(f), get_nullstr(), maketexlstring(".pgc", 4));
-    cur_file_name = makecstring(make_name_string());
+    cur_file_name =
+        pack_file_name(tex_font_name(f), get_nullstr(),
+                       maketexlstring(".pgc", 4));
     is_pk_font = false;
 
     if (t3_buffer != NULL) {
