@@ -23,7 +23,7 @@
 #include <string.h>
 
 static const char _svn_version[] =
-    "$Id: writettf.c 2830 2009-07-14 07:09:38Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writettf.c $";
+    "$Id: writettf.c 3187 2009-11-22 22:29:05Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/writettf.c $";
 
 #define DEFAULT_NTABS       14
 #define NEW_CMAP_SIZE       2
@@ -1335,50 +1335,38 @@ void writettf(PDF pdf, fd_entry * fd)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
     fd_cur = fd;                /* fd_cur is global inside writettf.c */
     assert(fd_cur->fm != NULL);
     assert(is_truetype(fd_cur->fm));
     assert(is_included(fd_cur->fm));
 
-    set_cur_file_name(fd_cur->fm->ff_name);
-
     if (is_subsetted(fd_cur->fm) && (fd_cur->fe == NULL)
         && !is_subfont(fd_cur->fm)) {
         pdftex_fail("Subset TrueType must be a reencoded or a subfont");
     }
     ttf_curbyte = 0;
     ttf_size = 0;
-    callback_id = callback_defined(find_truetype_file_callback);
-    if (callback_id > 0) {
-        if (run_callback
-            (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-            if (ftemp != NULL && strlen(ftemp)) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
-    }
 
+    cur_file_name =
+        luatex_find_file(fd_cur->fm->ff_name, find_truetype_file_callback);
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot find TrueType font file for reading");
+    }
     callback_id = callback_defined(read_truetype_file_callback);
     if (callback_id > 0) {
-        if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
+        if (run_callback(callback_id, "S->bSd", cur_file_name,
                          &file_opened, &ttf_buffer, &ttf_size) &&
             file_opened && ttf_size > 0) {
         } else {
             pdftex_fail("cannot open TrueType font file for reading");
         }
     } else {
-        if (!ttf_open()) {
+        if (!ttf_open(cur_file_name)) {
             pdftex_fail("cannot open TrueType font file for reading");
         }
         ttf_read_file();
         ttf_close();
     }
-    cur_file_name = (char *) nameoffile + 1;
     if (tracefilenames) {
         if (is_subsetted(fd_cur->fm))
             tex_printf("<%s", cur_file_name);
@@ -1455,47 +1443,35 @@ void writeotf(PDF pdf, fd_entry * fd)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
-
 
     fd_cur = fd;                /* fd_cur is global inside writettf.c */
     assert(fd_cur->fm != NULL);
     assert(is_opentype(fd_cur->fm));
     assert(is_included(fd_cur->fm));
 
-    set_cur_file_name(fd_cur->fm->ff_name);
     ttf_curbyte = 0;
     ttf_size = 0;
-    callback_id = callback_defined(find_opentype_file_callback);
-    if (callback_id > 0) {
-        if (run_callback
-            (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-            if (ftemp != NULL && strlen(ftemp)) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
+    cur_file_name =
+        luatex_find_file(fd_cur->fm->ff_name, find_opentype_file_callback);
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot find OpenType font file for reading");
     }
     callback_id = callback_defined(read_opentype_file_callback);
     if (callback_id > 0) {
-        if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
+        if (run_callback(callback_id, "S->bSd", cur_file_name,
                          &file_opened, &ttf_buffer, &ttf_size) &&
             file_opened && ttf_size > 0) {
         } else {
             pdftex_fail("cannot open OpenType font file for reading");
         }
     } else {
-        if (!otf_open()) {
+        if (!otf_open(cur_file_name)) {
             pdftex_fail("cannot open OpenType font file for reading");
         }
         ttf_read_file();
         ttf_close();
     }
 
-    cur_file_name = (char *) nameoffile + 1;
     fd_cur->ff_found = true;
     do_writeotf(pdf, fd);
     xfree(ttf_buffer);
