@@ -22,13 +22,12 @@
 #include "writecff.h"
 
 static const char _svn_version[] =
-    "$Id: writetype0.c 3079 2009-10-25 22:53:33Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writetype0.c $";
+    "$Id: writetype0.c 3187 2009-11-22 22:29:05Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/writetype0.c $";
 
 void writetype0(PDF pdf, fd_entry * fd)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
     long i;
     dirtab_entry *tab;
     cff_font *cff;
@@ -41,39 +40,29 @@ void writetype0(PDF pdf, fd_entry * fd)
     assert(is_opentype(fd_cur->fm));
     assert(is_included(fd_cur->fm));
 
-    set_cur_file_name(fd_cur->fm->ff_name);
     ttf_curbyte = 0;
     ttf_size = 0;
-    callback_id = callback_defined(find_opentype_file_callback);
-    if (callback_id > 0) {
-        if (run_callback
-            (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-            if (ftemp != NULL && strlen(ftemp)) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
+    cur_file_name =
+        luatex_find_file(fd_cur->fm->ff_name, find_opentype_file_callback);
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot find OpenType font file for reading");
     }
     callback_id = callback_defined(read_opentype_file_callback);
     if (callback_id > 0) {
-        if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
+        if (run_callback(callback_id, "S->bSd", cur_file_name,
                          &file_opened, &ttf_buffer, &ttf_size) &&
             file_opened && ttf_size > 0) {
         } else {
             pdftex_fail("cannot open OpenType font file for reading");
         }
     } else {
-        if (!otf_open()) {
+        if (!otf_open(cur_file_name)) {
             pdftex_fail("cannot open OpenType font file for reading");
         }
         ttf_read_file();
         ttf_close();
     }
 
-    cur_file_name = (char *) nameoffile + 1;
     fd_cur->ff_found = true;
 
     if (tracefilenames) {
