@@ -25,7 +25,7 @@
 #include "tt_glyf.h"
 
 static const char _svn_version[] =
-    "$Id: writetype2.c 2712 2009-07-01 12:09:02Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writetype2.c $";
+    "$Id: writetype2.c 3187 2009-11-22 22:29:05Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/font/writetype2.c $";
 
 /* forward*/
 void make_tt_subset(PDF pdf, fd_entry * fd, unsigned char *buffer,
@@ -134,54 +134,37 @@ void writetype2(PDF pdf, fd_entry * fd)
 {
     int callback_id;
     int file_opened = 0;
-    char *ftemp = NULL;
 
     glyph_tab = NULL;
 
     fd_cur = fd;                /* fd_cur is global inside writettf.c */
     assert(fd_cur->fm != NULL);
     assert(is_truetype(fd_cur->fm));
-
-    if (!is_subsetted(fd_cur->fm)) {
-        writettf(pdf, fd);
-        return;
-    }
-
     assert(is_included(fd_cur->fm));
 
-    set_cur_file_name(fd_cur->fm->ff_name);
     ttf_curbyte = 0;
     ttf_size = 0;
-    callback_id = callback_defined(find_opentype_file_callback);
-    if (callback_id > 0) {
-        if (run_callback
-            (callback_id, "S->S", (char *) (nameoffile + 1), &ftemp)) {
-            if (ftemp != NULL && strlen(ftemp)) {
-                free(nameoffile);
-                namelength = strlen(ftemp);
-                nameoffile = xmalloc(namelength + 2);
-                strcpy((char *) (nameoffile + 1), ftemp);
-                free(ftemp);
-            }
-        }
+    cur_file_name =
+        luatex_find_file(fd_cur->fm->ff_name, find_opentype_file_callback);
+    if (cur_file_name == NULL) {
+        pdftex_fail("cannot find OpenType font file for reading");
     }
     callback_id = callback_defined(read_opentype_file_callback);
     if (callback_id > 0) {
-        if (run_callback(callback_id, "S->bSd", (char *) (nameoffile + 1),
+        if (run_callback(callback_id, "S->bSd", cur_file_name,
                          &file_opened, &ttf_buffer, &ttf_size) &&
             file_opened && ttf_size > 0) {
         } else {
             pdftex_fail("cannot open OpenType font file for reading");
         }
     } else {
-        if (!otf_open()) {
+        if (!otf_open(cur_file_name)) {
             pdftex_fail("cannot open OpenType font file for reading");
         }
         ttf_read_file();
         ttf_close();
     }
 
-    cur_file_name = (char *) nameoffile + 1;
     fd_cur->ff_found = true;
 
     if (tracefilenames) {
@@ -238,7 +221,7 @@ static struct {
     "head", 1}, {
     "hhea", 1}, {
     "loca", 1}, {
-    "maxp", 1}, {
+    "maxp", 0}, {
     "name", 1}, {
     "glyf", 1}, {
     "hmtx", 1}, {
@@ -263,7 +246,6 @@ unsigned long ttc_read_offset(sfnt * sfont, int ttc_idx)
         fprintf(stderr, "Invalid TTC index number\n");
         uexit(1);
     }
-
     sfnt_seek_set(sfont, 12 + ttc_idx * 4);
     offset = sfnt_get_ulong(sfont);
 
@@ -283,7 +265,7 @@ void make_tt_subset(PDF pdf, fd_entry * fd, unsigned char *buffer,
     unsigned char *cidtogidmap;
     unsigned short num_glyphs, gid;
     struct tt_glyphs *glyphs;
-    char *used_chars;
+    char *used_chars = NULL;
     sfnt *sfont;
     pdf_obj *fontfile;
     int verbose = 0, error = 0;
@@ -294,6 +276,7 @@ void make_tt_subset(PDF pdf, fd_entry * fd, unsigned char *buffer,
 
     if (sfont->type == SFNT_TYPE_TTC) {
         i = ff_get_ttc_index(fd->fm->ff_name, fd->fm->ps_name);
+        tex_printf("(%s:%ld)", (fd->fm->ps_name ? fd->fm->ps_name : ""), i);
         error = sfnt_read_table_directory(sfont, ttc_read_offset(sfont, i));
     } else {
         error = sfnt_read_table_directory(sfont, 0);
@@ -303,72 +286,74 @@ void make_tt_subset(PDF pdf, fd_entry * fd, unsigned char *buffer,
         fprintf(stderr, "Could not parse the ttf directory.\n");
         uexit(1);
     }
+    if (is_subsetted(fd->fm)) {
+        /* rebuild the glyph tables and create a fresh cidmap */
+        glyphs = tt_build_init();
 
-    glyphs = tt_build_init();
+        last_cid = 0;
 
-    last_cid = 0;
-
-    avl_t_init(&t, fd->gl_tree);
-    for (found = (glw_entry *) avl_t_first(&t, fd->gl_tree);
-         found != NULL; found = (glw_entry *) avl_t_next(&t)) {
-        if (found->id > last_cid)
-            last_cid = found->id;
-    }
+        avl_t_init(&t, fd->gl_tree);
+        for (found = (glw_entry *) avl_t_first(&t, fd->gl_tree);
+             found != NULL; found = (glw_entry *) avl_t_next(&t)) {
+            if (found->id > last_cid)
+                last_cid = found->id;
+        }
 
 #ifndef NO_GHOSTSCRIPT_BUG
-    cidtogidmap = NULL;
+        cidtogidmap = NULL;
 #else
-    cidtogidmap = xmalloc(((last_cid + 1) * 2) * sizeof(unsigned char));
-    memset(cidtogidmap, 0, (last_cid + 1) * 2);
+        cidtogidmap = xmalloc(((last_cid + 1) * 2) * sizeof(unsigned char));
+        memset(cidtogidmap, 0, (last_cid + 1) * 2);
 #endif
 
-    /* fill used_chars */
-    used_chars = xmalloc((last_cid + 1) * sizeof(char));
-    memset(used_chars, 0, (last_cid + 1));
-    avl_t_init(&t, fd->gl_tree);
-    for (found = (glw_entry *) avl_t_first(&t, fd->gl_tree);
-         found != NULL; found = (glw_entry *) avl_t_next(&t)) {
-        used_chars[found->id] = 1;
-    }
+        /* fill used_chars */
+        used_chars = xmalloc((last_cid + 1) * sizeof(char));
+        memset(used_chars, 0, (last_cid + 1));
+        avl_t_init(&t, fd->gl_tree);
+        for (found = (glw_entry *) avl_t_first(&t, fd->gl_tree);
+             found != NULL; found = (glw_entry *) avl_t_next(&t)) {
+            used_chars[found->id] = 1;
+        }
 
-    /*
-     * Map CIDs to GIDs.
-     */
+        /*
+         * Map CIDs to GIDs.
+         */
 
-    num_glyphs = 1;             /* .notdef */
-    for (cid = 1; cid <= (long) last_cid; cid++) {
-        if (used_chars[cid] == 0)
-            continue;
-        gid = cid;
+        num_glyphs = 1;         /* .notdef */
+        for (cid = 1; cid <= (long) last_cid; cid++) {
+            if (used_chars[cid] == 0)
+                continue;
+            gid = cid;
 
 
 #ifndef NO_GHOSTSCRIPT_BUG
-        gid = tt_add_glyph(glyphs, gid, cid);
+            gid = tt_add_glyph(glyphs, gid, cid);
 #else
-        gid = tt_add_glyph(glyphs, gid, num_glyphs);
-        cidtogidmap[2 * cid] = gid >> 8;
-        cidtogidmap[2 * cid + 1] = gid & 0xff;
+            gid = tt_add_glyph(glyphs, gid, num_glyphs);
+            cidtogidmap[2 * cid] = gid >> 8;
+            cidtogidmap[2 * cid + 1] = gid & 0xff;
 #endif                          /* !NO_GHOSTSCRIPT_BUG */
 
-        num_glyphs++;
-    }
+            num_glyphs++;
+        }
 
-    if (num_glyphs == 1) {
-        fprintf(stderr, "No glyphs in subset?.\n");
-        uexit(1);
-    }
+        if (num_glyphs == 1) {
+            fprintf(stderr, "No glyphs in subset?.\n");
+            uexit(1);
+        }
 
-    if (tt_build_tables(sfont, glyphs) < 0) {
-        fprintf(stderr, "Could not parse the ttf buffer.\n");
-        uexit(1);
-    }
+        if (tt_build_tables(sfont, glyphs) < 0) {
+            fprintf(stderr, "Could not parse the ttf buffer.\n");
+            uexit(1);
+        }
 
-    if (verbose > 1) {
-        fprintf(stdout, "[%u glyphs (Max CID: %u)]", glyphs->num_glyphs,
-                last_cid);
-    }
+        if (verbose > 1) {
+            fprintf(stdout, "[%u glyphs (Max CID: %u)]", glyphs->num_glyphs,
+                    last_cid);
+        }
 
-    tt_build_finish(glyphs);
+        tt_build_finish(glyphs);
+    }
 
     /* Create font file */
 
