@@ -18,16 +18,16 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: pdftoepdf.h 2579 2009-06-23 14:12:23Z taco $ */
+/* $Id: pdftoepdf.h 3256 2009-12-15 22:39:14Z hhenkel $ */
 
 #ifndef PDFTOEPDF_H
 #  define PDFTOEPDF_H
 
 #  include "image.h"
 
-void read_pdf_info(PDF, image_dict *, integer, integer);
+void read_pdf_info(PDF, image_dict *, integer, integer, img_readtype_e);
 void unrefPdfDocument(char *);
-void write_additional_epdf_objects(PDF);
+void write_additional_epdf_objects(PDF, char *);
 void write_epdf(PDF, image_dict *);
 void epdf_check_mem(void);
 
