@@ -19,8 +19,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: writeimg.c 3043 2009-08-26 20:20:33Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/image/writeimg.c $";
+    "$Id: writeimg.c 3256 2009-12-15 22:39:14Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/image/writeimg.c $";
 
 #include <assert.h>
 #include "ptexlib.h"
@@ -34,12 +34,11 @@ static const char _svn_version[] =
 #include "writejbig2.h"
 #include "pdftoepdf.h"
 
-#include <../lua51/lua.h>
+#include <../lua51/lua.h>       /* for LUA_NOREF */
 #include <../lua51/lauxlib.h>
 
 #define pdf_image_resolution int_par(pdf_image_resolution_code)
-
-integer image_orig_x, image_orig_y;     /* origin of cropped PDF images */
+#define pdf_pagebox int_par(pdf_pagebox_code)
 
 /**********************************************************************/
 /*
@@ -187,15 +186,14 @@ void new_img_pdfstream_struct(image_dict * p)
 
 /**********************************************************************/
 
-void init_image(image * p)
+static void init_image(image * p)
 {
     assert(p != NULL);
-    memset(p, 0, sizeof(image));
     set_wd_running(p);
     set_ht_running(p);
     set_dp_running(p);
-    img_unset_scaled(p);
-    img_arrayidx(p) = -1;       /* -1 = unused, used count from 0 */
+    img_transform(p) = 0;
+    img_dict(p) = NULL;
     img_dictref(p) = LUA_NOREF;
 }
 
@@ -206,15 +204,20 @@ image *new_image(void)
     return p;
 }
 
-void init_image_dict(image_dict * p)
+static void init_image_dict(image_dict * p)
 {
     assert(p != NULL);
     memset(p, 0, sizeof(image_dict));
+    set_wd_running(p);
+    set_ht_running(p);
+    set_dp_running(p);
+    img_transform(p) = 0;
     img_pagenum(p) = 1;
     img_type(p) = IMG_TYPE_NONE;
     img_pagebox(p) = PDF_BOX_SPEC_MEDIA;
     img_unset_bbox(p);
     img_state(p) = DICT_NEW;
+    img_index(p) = -1;          /* -1 = unused, used count from 0 */
 }
 
 image_dict *new_image_dict(void)
@@ -224,20 +227,24 @@ image_dict *new_image_dict(void)
     return p;
 }
 
-void free_dict_strings(image_dict * p)
+static void free_dict_strings(image_dict * p)
 {
-    if (img_filename(p) != NULL)
+    if (img_filename(p) != NULL) {
         xfree(img_filename(p));
-    img_filename(p) = NULL;
-    if (img_filepath(p) != NULL)
+        img_filename(p) = NULL;
+    }
+    if (img_filepath(p) != NULL) {
         xfree(img_filepath(p));
-    img_filepath(p) = NULL;
-    if (img_attr(p) != NULL)
+        img_filepath(p) = NULL;
+    }
+    if (img_attr(p) != NULL) {
         xfree(img_attr(p));
-    img_attr(p) = NULL;
-    if (img_pagename(p) != NULL)
+        img_attr(p) = NULL;
+    }
+    if (img_pagename(p) != NULL) {
         xfree(img_pagename(p));
-    img_pagename(p) = NULL;
+        img_pagename(p) = NULL;
+    }
 }
 
 void free_image_dict(image_dict * p)
@@ -296,9 +303,9 @@ void read_img(PDF pdf,
         } else
             img_filepath(idict) =
                 kpse_find_file(img_filename(idict), kpse_tex_format, true);
+        if (img_filepath(idict) == NULL)
+            pdftex_fail("cannot find image file");
     }
-    if (img_filepath(idict) == NULL)
-        pdftex_fail("cannot find image file");
     recorder_record_input(img_filename(idict));
     /* type checks */
     check_type_by_header(idict);
@@ -307,7 +314,8 @@ void read_img(PDF pdf,
     switch (img_type(idict)) {
     case IMG_TYPE_PDF:
         assert(pdf != NULL);    /* TODO! */
-        read_pdf_info(pdf, idict, minor_version, inclusion_errorlevel);
+        read_pdf_info(pdf, idict, minor_version, inclusion_errorlevel,
+                      IMG_CLOSEINBETWEEN);
         img_group_ref(idict) = epdf_lastGroupObjectNum;
         break;
     case IMG_TYPE_PNG:
@@ -332,35 +340,169 @@ void read_img(PDF pdf,
         img_state(idict) = DICT_FILESCANNED;
 }
 
-integer read_image(PDF pdf,
-                   integer objnum, integer index, char *filename,
-                   integer page_num, char *page_name, char *attr,
-                   integer colorspace, integer page_box,
-                   integer minor_version, integer inclusion_errorlevel)
+static image_dict *read_image(PDF pdf, char *file_name, integer page_num,
+                              char *page_name, integer colorspace,
+                              integer page_box, integer minor_version,
+                              integer inclusion_errorlevel)
 {
-    image_dict *idict;
     image *a = new_image();
-    assert(img_arrayidx(a) == -1);
-    img_arrayidx(a) = img_to_array(a);
-    idict = img_dict(a) = new_image_dict();
-    assert(idict != NULL);
-    img_objnum(idict) = objnum;
-    img_index(idict) = index;
-    /* img_xsize, img_ysize, img_xres, img_yres set by read_img() */
+    image_dict *idict = img_dict(a) = new_image_dict();
+    incr(pdf->ximage_count);
+    pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
+    img_objnum(idict) = pdf->obj_ptr;
+    img_index(idict) = pdf->ximage_count;
+    set_obj_data_ptr(pdf, img_objnum(idict), img_index(idict));
+    idict_to_array(idict);
     img_colorspace(idict) = colorspace;
     img_pagenum(idict) = page_num;
-    /* img_totalpages set by read_img() */
-    if (page_name != 0)
-        img_pagename(idict) = xstrdup(page_name);
-    cur_file_name = filename;
-    assert(cur_file_name != NULL);
-    img_filename(idict) = xstrdup(filename);
-    if (attr != 0)
-        img_attr(idict) = xstrdup(attr);
+    img_pagename(idict) = page_name;
+    assert(file_name != NULL);
+    cur_file_name = file_name;
+    img_filename(idict) = file_name;
     img_pagebox(idict) = page_box;
     read_img(pdf, idict, minor_version, inclusion_errorlevel);
-    img_unset_scaled(a);
-    return img_arrayidx(a);
+    return idict;
+}
+
+/* scans PDF pagebox specification */
+static pdfboxspec_e scan_pdf_box_spec(void)
+{
+    if (scan_keyword("mediabox"))
+        return PDF_BOX_SPEC_MEDIA;
+    else if (scan_keyword("cropbox"))
+        return PDF_BOX_SPEC_CROP;
+    else if (scan_keyword("bleedbox"))
+        return PDF_BOX_SPEC_BLEED;
+    else if (scan_keyword("trimbox"))
+        return PDF_BOX_SPEC_TRIM;
+    else if (scan_keyword("artbox"))
+        return PDF_BOX_SPEC_ART;
+    return PDF_BOX_SPEC_NONE;
+}
+
+void scan_pdfximage(PDF pdf)
+{
+    scaled_whd alt_rule;
+    image_dict *idict;
+    integer transform = 0, page = 1, pagebox, colorspace = 0;
+    char *named = NULL, *attr = NULL, *file_name = NULL;
+    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
+    if (scan_keyword("attr")) {
+        scan_pdf_ext_toks();
+        attr = tokenlist_to_cstring(def_ref, true, NULL);
+        delete_token_ref(def_ref);
+    }
+    if (scan_keyword("named")) {
+        scan_pdf_ext_toks();
+        named = tokenlist_to_cstring(def_ref, true, NULL);
+        delete_token_ref(def_ref);
+        page = 0;
+    } else if (scan_keyword("page")) {
+        scan_int();
+        page = cur_val;
+    }
+    if (scan_keyword("colorspace")) {
+        scan_int();
+        colorspace = cur_val;
+    }
+    pagebox = scan_pdf_box_spec();
+    if (pagebox == PDF_BOX_SPEC_NONE) {
+        pagebox = pdf_pagebox;
+        if (pagebox == PDF_BOX_SPEC_NONE)
+            pagebox = PDF_BOX_SPEC_CROP;
+    }
+    scan_pdf_ext_toks();
+    file_name = tokenlist_to_cstring(def_ref, true, NULL);
+    assert(file_name != NULL);
+    delete_token_ref(def_ref);
+    idict =
+        read_image(pdf, file_name, page, named, colorspace, pagebox,
+                   pdf_minor_version, pdf_inclusion_errorlevel);
+    img_attr(idict) = attr;
+    img_dimen(idict) = alt_rule;
+    img_transform(idict) = transform;
+    pdf_last_ximage = img_objnum(idict);
+    pdf_last_ximage_pages = img_totalpages(idict);
+    pdf_last_ximage_colordepth = img_colordepth(idict);
+}
+
+#define tail          cur_list.tail_field
+
+void scan_pdfrefximage(PDF pdf)
+{
+    integer transform = 0;      /* one could scan transform as well */
+    image_dict *idict;
+    scaled_whd alt_rule, dim;
+    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
+    scan_int();
+    check_obj_exists(pdf, obj_type_ximage, cur_val);
+    new_whatsit(pdf_refximage_node);
+    idict = idict_array[obj_data_ptr(pdf, cur_val)];
+    if (alt_rule.wd != null_flag || alt_rule.ht != null_flag
+        || alt_rule.dp != null_flag)
+        dim = scale_img(idict, alt_rule, transform);
+    else
+        dim = scale_img(idict, img_dimen(idict), img_transform(idict));
+    width(tail) = dim.wd;
+    height(tail) = dim.ht;
+    depth(tail) = dim.dp;
+    pdf_ximage_transform(tail) = transform;
+    pdf_ximage_index(tail) = img_index(idict);
+}
+
+/* tex_scale() sequence of decisions:
+wd ht dp : res = tex;
+wd ht --
+wd -- dp
+wd -- --
+-- ht dp
+-- ht --
+-- -- dp
+-- -- -- : res = nat;
+*/
+
+scaled_whd tex_scale(scaled_whd nat, scaled_whd tex)
+{
+    scaled_whd res;
+    if (!is_running(tex.wd) && !is_running(tex.ht) && !is_running(tex.dp)) {
+        /* width, height, and depth specified */
+        res = tex;
+    } else /* max. 2 dimensions are specified */ if (!is_running(tex.wd)) {
+        res.wd = tex.wd;
+        if (!is_running(tex.ht)) {
+            res.ht = tex.ht;
+            /* width and height specified */
+            res.dp = ext_xn_over_d(tex.ht, nat.dp, nat.ht);
+        } else if (!is_running(tex.dp)) {
+            res.dp = tex.dp;
+            /* width and depth specified */
+            res.ht = ext_xn_over_d(tex.wd, nat.ht + nat.dp, nat.wd) - tex.dp;
+        } else {
+            /* only width specified */
+            res.ht = ext_xn_over_d(tex.wd, nat.ht, nat.wd);
+            res.dp = ext_xn_over_d(tex.wd, nat.dp, nat.wd);
+        }
+    } else if (!is_running(tex.ht)) {
+        res.ht = tex.ht;
+        if (!is_running(tex.dp)) {
+            res.dp = tex.dp;
+            /* height and depth specified */
+            res.wd = ext_xn_over_d(tex.ht + tex.dp, nat.wd, nat.ht + nat.dp);
+        } else {
+            /* only height specified */
+            res.wd = ext_xn_over_d(tex.ht, nat.wd, nat.ht);
+            res.dp = ext_xn_over_d(tex.ht, nat.dp, nat.ht);
+        }
+    } else if (!is_running(tex.dp)) {
+        res.dp = tex.dp;
+        /* only depth specified */
+        res.ht = nat.ht - (tex.dp - nat.dp);
+        res.wd = nat.wd;
+    } else {
+        /* nothing specified */
+        res = nat;
+    }
+    return res;
 }
 
 // Within scale_img() only image width and height matter;
@@ -368,14 +510,11 @@ integer read_image(PDF pdf,
 // But one needs rotation info to swap width and height.
 // img_rotation() comes from the optional /Rotate key in the PDF file.
 
-void scale_img(image * img)
+scaled_whd scale_img(image_dict * idict, scaled_whd alt_rule, integer transform)
 {
     integer x, y, xr, yr, tmp;  /* size and resolution of image */
-    scaled w = 0, h = 0;        /* indeed size corresponds to image resolution */
+    scaled_whd nat;             /* natural size corresponding to image resolution */
     integer default_res;
-    image_dict *idict;
-    assert(img != NULL);
-    idict = img_dict(img);
     assert(idict != NULL);
     if ((img_type(idict) == IMG_TYPE_PDF
          || img_type(idict) == IMG_TYPE_PDFSTREAM) && img_is_bbox(idict)) {
@@ -396,7 +535,7 @@ void scale_img(image * img)
         yr = 0;
         pdftex_warn("ext1: too large image resolution ignored");
     }
-    if (((img_transform(img) - img_rotation(idict)) & 1) == 1) {
+    if (((transform - img_rotation(idict)) & 1) == 1) {
         tmp = x;
         x = y;
         y = tmp;
@@ -404,64 +543,26 @@ void scale_img(image * img)
         xr = yr;
         yr = tmp;
     }
+    nat.dp = 0;                 /* always for images */
     if (img_type(idict) == IMG_TYPE_PDF
         || img_type(idict) == IMG_TYPE_PDFSTREAM) {
-        w = x;
-        h = y;
+        nat.wd = x;
+        nat.ht = y;
     } else {
         default_res = fix_int(pdf_image_resolution, 0, 65535);
         if (default_res > 0 && (xr == 0 || yr == 0)) {
             xr = default_res;
             yr = default_res;
         }
-        if (is_wd_running(img) && is_ht_running(img)) {
-            if (xr > 0 && yr > 0) {
-                w = ext_xn_over_d(one_hundred_inch, x, 100 * xr);
-                h = ext_xn_over_d(one_hundred_inch, y, 100 * yr);
-            } else {
-                w = ext_xn_over_d(one_hundred_inch, x, 7200);
-                h = ext_xn_over_d(one_hundred_inch, y, 7200);
-            }
-        }
-    }
-    if (is_wd_running(img) && is_ht_running(img) && is_dp_running(img)) {
-        img_width(img) = w;
-        img_height(img) = h;
-        img_depth(img) = 0;
-    } else if (is_wd_running(img)) {
-        /* image depth or height is explicitly specified */
-        if (is_ht_running(img)) {
-            /* image depth is explicitly specified */
-            img_width(img) = ext_xn_over_d(h, x, y);
-            img_height(img) = h - img_depth(img);
-        } else if (is_dp_running(img)) {
-            /* image height is explicitly specified */
-            img_width(img) = ext_xn_over_d(img_height(img), x, y);
-            img_depth(img) = 0;
+        if (xr > 0 && yr > 0) {
+            nat.wd = ext_xn_over_d(one_hundred_inch, x, 100 * xr);
+            nat.ht = ext_xn_over_d(one_hundred_inch, y, 100 * yr);
         } else {
-            /* both image depth and height are explicitly specified */
-            img_width(img) =
-                ext_xn_over_d(img_height(img) + img_depth(img), x, y);
+            nat.wd = ext_xn_over_d(one_hundred_inch, x, 7200);
+            nat.ht = ext_xn_over_d(one_hundred_inch, y, 7200);
         }
-    } else {
-        /* image width is explicitly specified */
-        if (is_ht_running(img) && is_dp_running(img)) {
-            /* both image depth and height are not specified */
-            img_height(img) = ext_xn_over_d(img_width(img), y, x);
-            img_depth(img) = 0;
-        }
-        /* image depth is explicitly specified */
-        else if (is_ht_running(img)) {
-            img_height(img) =
-                ext_xn_over_d(img_width(img), y, x) - img_depth(img);
-        }
-        /* image height is explicitly specified */
-        else if (is_dp_running(img)) {
-            img_depth(img) = 0;
-        }
-        /* else both image depth and height are explicitly specified */
     }
-    img_set_scaled(img);
+    return tex_scale(nat, alt_rule);
 }
 
 void write_img(PDF pdf, image_dict * idict)
@@ -493,7 +594,7 @@ void write_img(PDF pdf, image_dict * idict)
         if (tracefilenames)
             tex_printf(">");
         if (img_type(idict) == IMG_TYPE_PDF) {
-            write_additional_epdf_objects(pdf);
+            write_additional_epdf_objects(pdf, img_filepath(idict));
         } else {
             if (img_type(idict) == IMG_TYPE_PNG) {
                 write_additional_png_objects(pdf);
@@ -509,7 +610,7 @@ void pdf_write_image(PDF pdf, integer n)
 {
     pdf_begin_dict(pdf, n, 0);
     if (pdf->draftmode == 0)
-        write_image(pdf, obj_data_ptr(pdf, n));
+        write_img(pdf, idict_array[obj_data_ptr(pdf, n)]);
 }
 
 /**********************************************************************/
@@ -543,17 +644,22 @@ void write_pdfstream(PDF pdf, image_dict * idict)
 
 /**********************************************************************/
 
-/* define img_ptr, img_array, & img_limit */
+/* define idict_ptr, idict_array, & idict_limit */
 /* avoid use of size_t */
-img_entry *img_ptr, *img_array = NULL;
-size_t img_limit;
+idict_entry *idict_ptr, *idict_array = NULL;
+size_t idict_limit;
 
-integer img_to_array(image * img)
+void idict_to_array(image_dict * idict)
 {
-    assert(img != NULL);
-    alloc_array(img, 1, SMALL_BUF_SIZE);
-    *img_ptr = img;
-    return img_ptr++ - img_array;       /* now img is read-only */
+    assert(idict != NULL);
+    if (idict_ptr - idict_array == 0) { /* align to count from 1 */
+        alloc_array(idict, 1, SMALL_BUF_SIZE);  /* /Im0 unused */
+        idict_ptr++;
+    }
+    alloc_array(idict, 1, SMALL_BUF_SIZE);
+    *idict_ptr = idict;
+    assert(img_index(idict) == idict_ptr - idict_array);
+    idict_ptr++;
 }
 
 /**********************************************************************/
@@ -585,43 +691,42 @@ integer img_to_array(image * img)
  * makes the code simpler.
  */
 
-#define dumpcharptr(a)				\
-  do {						\
-    integer x;					\
-    if (a!=NULL) {				\
-      x = strlen(a)+1;				\
-      dumpinteger(x);  dump_things(*a, x);	\
-    } else {					\
-      x = 0; dumpinteger(x);			\
-    }						\
+#define dumpcharptr(a)                          \
+  do {                                          \
+    integer x;                                  \
+    if (a!=NULL) {                              \
+      x = strlen(a)+1;                          \
+      dumpinteger(x);  dump_things(*a, x);      \
+    } else {                                    \
+      x = 0; dumpinteger(x);                    \
+    }                                           \
   } while (0)
 
-#define undumpcharptr(s)			\
-  do {						\
-    integer x;					\
-    char *a;					\
-    undumpinteger (x);				\
-    if (x>0) {					\
-      a = malloc(x);				\
-      undump_things(*a,x);			\
-      s = a ;					\
-    } else { s = NULL; }			\
+#define undumpcharptr(s)                        \
+  do {                                          \
+    integer x;                                  \
+    char *a;                                    \
+    undumpinteger (x);                          \
+    if (x>0) {                                  \
+      a = malloc(x);                            \
+      undump_things(*a,x);                      \
+      s = a ;                                   \
+    } else { s = NULL; }                        \
   } while (0)
 
 void dumpimagemeta(void)
 {
-    int cur_image, i;
-    image *img;
+    int cur_index, i;
     image_dict *idict;
 
-    dumpinteger(img_limit);
-    cur_image = (img_ptr - img_array);
-    dumpinteger(cur_image);
+    i = (int) idict_limit;
+    dumpinteger(i);
+    cur_index = (idict_ptr - idict_array);
+    dumpinteger(cur_index);
 
-    for (i = 0; i < cur_image; i++) {
-        img = img_array[i];
-        assert(img != NULL);
-        idict = img_dict(img);
+    for (i = 1; i < cur_index; i++) {
+        idict = idict_array[i];
+        assert(idict != NULL);
         dumpcharptr(img_filename(idict));
         dumpinteger(img_type(idict));
         dumpinteger(img_color(idict));
@@ -649,23 +754,22 @@ void dumpimagemeta(void)
 void undumpimagemeta(PDF pdf, integer pdfversion,
                      integer pdfinclusionerrorlevel)
 {
-    int cur_image, i;
-    image *img;
+    int cur_index, i;
     image_dict *idict;
 
     assert(pdf != NULL);
-    undumpinteger(img_limit);
+    undumpinteger(i);
+    idict_limit = i;
 
-    img_array = xtalloc(img_limit, img_entry);
-    undumpinteger(cur_image);
-    img_ptr = img_array + cur_image;
+    idict_array = xtalloc(idict_limit, idict_entry);
+    undumpinteger(cur_index);
+    idict_ptr = idict_array + cur_index;
 
-    for (i = 0; i < cur_image; i++) {
-        img = new_image();
-        assert(img_arrayidx(img) == -1);
-        img_arrayidx(img) = img_to_array(img);
-        idict = img_dict(img) = new_image_dict();
+    for (i = 1; i < cur_index; i++) {
+        idict = new_image_dict();
         assert(idict != NULL);
+        assert(img_index(idict) == -1);
+        idict_to_array(idict);
         undumpcharptr(img_filename(idict));
         undumpinteger(img_type(idict));
         undumpinteger(img_color(idict));
@@ -726,11 +830,3 @@ scaled_whd scan_alt_rule(void)
     }
     return alt_rule;
 }
-
-void set_image_dimensions(integer ref, scaled_whd dim)
-{
-    image *a = img_array[ref];
-    img_width(a) = dim.wd;
-    img_height(a) = dim.ht;
-    img_depth(a) = dim.dp;
-}
