@@ -81,8 +81,8 @@ object exists, reference it. Else create fresh one.
 ***********************************************************************/
 
 static const char _svn_version[] =
-    "$Id: writejbig2.c 2862 2009-07-16 06:36:57Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/image/writejbig2.c $";
+    "$Id: writejbig2.c 3237 2009-12-05 11:50:15Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/image/writejbig2.c $";
 
 #undef DEBUG
 
@@ -743,7 +743,7 @@ static void wr_jbig2(PDF pdf, FILEINFO * fip, unsigned long page)
         pdf_puts(pdf, "/Filter [/JBIG2Decode]\n");
         if (fip->page0.last != NULL) {
             if (fip->pdfpage0objnum == 0) {
-                pdf_create_obj(pdf, 0, 0);
+                pdf_create_obj(pdf, obj_type_others, 0);
                 fip->pdfpage0objnum = pdf->obj_ptr;
             }
             pdf_printf(pdf, "/DecodeParms [<< /JBIG2Globals %lu 0 R >>]\n",
