@@ -1,6 +1,6 @@
 /* limglib.c
    
-   Copyright 2006-2008 Taco Hoekwater <taco@luatex.org>
+   Copyright 2006-2009 Taco Hoekwater <taco@luatex.org>
 
    This file is part of LuaTeX.
 
@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: limglib.c 3044 2009-09-13 16:28:28Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/limglib.c $";
+    "$Id: limglib.c 3217 2009-12-03 16:52:01Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/limglib.c $";
 
 #include <stdio.h>
 #include <string.h>
@@ -284,8 +284,6 @@ static void lua_to_image(lua_State * L, image * a)
     lua_pop(L, 1);              /* v k t ... */
     switch (i) {
     case P_WIDTH:
-        if (img_is_refered(a))
-            luaL_error(L, "image.width is now read-only");
         if (lua_isnil(L, -1))
             set_wd_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
@@ -295,11 +293,8 @@ static void lua_to_image(lua_State * L, image * a)
         else
             luaL_error(L,
                        "image.width needs integer or nil value or dimension string");
-        img_unset_scaled(a);
         break;
     case P_HEIGHT:
-        if (img_is_refered(a))
-            luaL_error(L, "image.height is now read-only");
         if (lua_isnil(L, -1))
             set_ht_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
@@ -309,11 +304,8 @@ static void lua_to_image(lua_State * L, image * a)
         else
             luaL_error(L,
                        "image.height needs integer or nil value or dimension string");
-        img_unset_scaled(a);
         break;
     case P_DEPTH:
-        if (img_is_refered(a))
-            luaL_error(L, "image.depth is now read-only");
         if (lua_isnil(L, -1))
             set_dp_running(a);
         else if (lua_type(L, -1) == LUA_TNUMBER)
@@ -323,16 +315,12 @@ static void lua_to_image(lua_State * L, image * a)
         else
             luaL_error(L,
                        "image.depth needs integer or nil value or dimension string");
-        img_unset_scaled(a);
         break;
     case P_TRANSFORM:
-        if (img_is_refered(a))
-            luaL_error(L, "image.transform is now read-only");
         if (lua_isnumber(L, -1))
             img_transform(a) = lua_tointeger(L, -1);
         else
             luaL_error(L, "image.transform needs integer value");
-        img_unset_scaled(a);
         break;
         /* now follow all image_dict entries */
     case P_FILENAME:
@@ -415,7 +403,6 @@ static void lua_to_image(lua_State * L, image * a)
             lua_pop(L, 1);      /* v k t ... */
         }
         img_set_bbox(d);
-        img_unset_scaled(a);
         break;
     case P_STREAM:
         if (img_filename(d) != NULL)
@@ -428,7 +415,6 @@ static void lua_to_image(lua_State * L, image * a)
             xfree(img_pdfstream_stream(d));
         img_pdfstream_stream(d) = xstrdup(lua_tostring(L, -1));
         img_type(d) = IMG_TYPE_PDFSTREAM;
-        img_unset_scaled(a);
         break;
     case P_FILEPATH:
     case P_TOTALPAGES:
@@ -450,16 +436,6 @@ static void lua_to_image(lua_State * L, image * a)
 
 /**********************************************************************/
 
-void fix_image_size(lua_State * L, image * a)
-{
-    if (!img_is_scaled(a) || is_wd_running(a) || is_ht_running(a)
-        || is_dp_running(a)) {
-        if (img_is_refered(a))
-            luaL_error(L, "image is read-only");
-        scale_img(a);
-    }
-}
-
 void copy_image(lua_State * L, lua_Number scale)
 {
     image *a, **aa, *b, **bb;
@@ -529,7 +505,7 @@ static int l_copy_image(lua_State * L)
     return 1;                   /* image */
 }
 
-static void read_scale_img(lua_State * L, image * a)
+static void read_scale_img(image * a)
 {
     image_dict *ad;
     assert(a != NULL);
@@ -542,10 +518,10 @@ static void read_scale_img(lua_State * L, image * a)
             fix_pdf_minorversion(static_pdf);
             read_img(static_pdf,
                      ad, pdf_minor_version, pdf_inclusion_errorlevel);
-            img_unset_scaled(a);
         }
     }
-    fix_image_size(L, a);
+    if (is_wd_running(a) || is_ht_running(a) || is_dp_running(a))
+        img_dimen(a) = scale_img(ad, img_dimen(a), img_transform(a));
 }
 
 static int l_scan_image(lua_State * L)
@@ -558,12 +534,12 @@ static int l_scan_image(lua_State * L)
     aa = (image **) luaL_checkudata(L, 1, TYPE_IMG);    /* image */
     a = *aa;
     check_o_mode(static_pdf, "img.scan()", 1 << OMODE_PDF, false);
-    flush_str(last_tex_string);
-    read_scale_img(L, a);
+    /* flush_str(last_tex_string); */ /* ?? */
+    read_scale_img(a);
     return 1;                   /* image */
 }
 
-static halfword img_to_node(image * a, integer ref)
+static halfword img_to_node(image * a)
 {
     image_dict *ad;
     halfword n;
@@ -572,10 +548,11 @@ static halfword img_to_node(image * a, integer ref)
     assert(ad != NULL);
     assert(img_objnum(ad) != 0);
     n = new_node(whatsit_node, pdf_refximage_node);
-    pdf_ximage_ref(n) = ref;
+    pdf_ximage_index(n) = img_index(ad);
     width(n) = img_width(a);
     height(n) = img_height(a);
     depth(n) = img_depth(a);
+    pdf_ximage_transform(n) = img_transform(a);
     return n;
 }
 
@@ -585,22 +562,21 @@ const char *wrtype_s[] =
 
 extern void lua_nodelib_push_fast(lua_State * L, halfword n);
 
-static void setup_image(PDF pdf, lua_State * L, image * a, wrtype_e writetype)
+static void setup_image(PDF pdf, image * a, wrtype_e writetype)
 {
     image_dict *ad;
     assert(a != NULL);
     ad = img_dict(a);
     check_o_mode(pdf, (char *) wrtype_s[writetype], 1 << OMODE_PDF, false);
-    flush_str(last_tex_string);
-    read_scale_img(L, a);
-    if (img_arrayidx(a) == -1)
-        img_arrayidx(a) = img_to_array(a);      /* now a is read-only */
+    /* flush_str(last_tex_string); */ /* ?? */
+    read_scale_img(a);
     if (img_objnum(ad) == 0) {  /* latest needed just before out_img() */
         pdf->ximage_count++;
         pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
         img_objnum(ad) = pdf->obj_ptr;
         img_index(ad) = pdf->ximage_count;
-        obj_data_ptr(pdf, pdf->obj_ptr) = img_arrayidx(a);
+        idict_to_array(ad);     /* now ad is read-only */
+        obj_data_ptr(pdf, pdf->obj_ptr) = img_index(ad);
     }
 }
 
@@ -616,10 +592,10 @@ static void write_image_or_node(lua_State * L, wrtype_e writetype)
     aa = (image **) luaL_checkudata(L, 1, TYPE_IMG);    /* image */
     a = *aa;
     ad = img_dict(a);
-    setup_image(static_pdf, L, a, writetype);
+    setup_image(static_pdf, a, writetype);
     switch (writetype) {
     case WR_WRITE:
-        n = img_to_node(a, img_arrayidx(a));
+        n = img_to_node(a);
         tail_append(n);
         break;                  /* image */
     case WR_IMMEDIATEWRITE:
@@ -628,7 +604,7 @@ static void write_image_or_node(lua_State * L, wrtype_e writetype)
         break;                  /* image */
     case WR_NODE:              /* image */
         lua_pop(L, 1);          /* - */
-        n = img_to_node(a, img_arrayidx(a));
+        n = img_to_node(a);
         lua_nodelib_push_fast(L, n);
         break;                  /* node */
     default:
@@ -719,13 +695,15 @@ static const struct luaL_Reg imglib[] = {
 void vf_out_image(PDF pdf, unsigned i)
 {
     image *a, **aa;
+    image_dict *ad;
     lua_State *L = Luas;        /* ... */
     lua_rawgeti(L, LUA_GLOBALSINDEX, i);        /* image ... */
     aa = (image **) luaL_checkudata(L, -1, TYPE_IMG);
     a = *aa;
-    setup_image(pdf, L, a, WR_VF_IMG);  /* image ... */
-    assert(img_is_refered(a));
-    pdf_place_image(pdf, img_arrayidx(a));
+    ad = img_dict(a);
+    assert(ad != NULL);
+    setup_image(pdf, a, WR_VF_IMG);     /* image ... */
+    place_img(pdf, ad, img_dimen(a), img_transform(a));
     lua_pop(L, 1);              /* ... */
 }
 
@@ -780,13 +758,15 @@ static int m_img_print(lua_State * L)
 static int m_img_gc(lua_State * L)
 {
     image *a, **aa;
+    image_dict *d;
     aa = (image **) luaL_checkudata(L, 1, TYPE_IMG);
     a = *aa;
+    d = img_dict(*aa);
 #ifdef DEBUG
     printf("\n===== IMG GC ===== a=%d ad=%d\n", a, img_dict(a));
 #endif
     luaL_unref(L, LUA_GLOBALSINDEX, img_dictref(a));
-    if (!img_is_refered(a))
+    if (!img_is_refered(d))
         xfree(a);
     return 0;
 }
