@@ -25,7 +25,7 @@
 #include <kpathsea/readable.h>
 
 static const char _svn_version[] =
-    "$Id: lkpselib.c 2869 2009-07-16 11:31:18Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/lkpselib.c $";
+    "$Id: lkpselib.c 3173 2009-11-19 07:47:02Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/lkpselib.c $";
 
 static const int filetypes[] = {
     kpse_gf_format,
@@ -83,7 +83,8 @@ static const int filetypes[] = {
     kpse_fea_format,
     kpse_cid_format,
     kpse_mlbib_format,
-    kpse_mlbst_format
+    kpse_mlbst_format,
+    kpse_lua_format
 };
 
 static const char *const filetypenames[] = {
@@ -143,6 +144,7 @@ static const char *const filetypenames[] = {
     "cid maps",
     "mlbib",
     "mlbst",
+    "clua",
     NULL
 };
 
