@@ -17,13 +17,13 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char _svn_version[] =
+    "$Id: lnodelib.c 3218 2009-12-04 08:21:56Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/lnodelib.c $";
+
 #include "lua/luatex-api.h"
 #include <ptexlib.h>
 
-static const char _svn_version[] =
-    "$Id: lnodelib.c 3053 2009-10-02 15:14:08Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/lnodelib.c $";
-
 #define init_luaS_index(a) do {                                         \
     lua_pushliteral(L,#a);                                              \
     luaS_##a##_ptr = (char *)lua_tostring(L,-1);                        \
@@ -42,6 +42,8 @@ static const char _svn_version[] =
 
 make_luaS_index(luatex_node);
 
+static int nodelib_getdir(lua_State * L, int n);        /* forward */
+
 halfword *check_isnode(lua_State * L, int ud)
 {
     register halfword *p = lua_touserdata(L, ud);
@@ -476,6 +478,7 @@ static int lua_nodelib_hpack(lua_State * L)
     char *s;
     integer w = 0;
     int m = 1;
+    int d = -1;
     n = *(check_isnode(L, 1));
     if (lua_gettop(L) > 1) {
         w = lua_tointeger(L, 2);
@@ -491,16 +494,21 @@ static int lua_nodelib_hpack(lua_State * L)
                                    "3rd argument should be either additional or exactly");
                     lua_error(L);
                 }
-            }
-
-            else if (lua_type(L, 3) == LUA_TNUMBER) {
+            } else if (lua_type(L, 3) == LUA_TNUMBER) {
                 m = lua_tonumber(L, 3);
             } else {
                 lua_pushstring(L, "incorrect 3rd argument");
             }
+            if (lua_gettop(L) > 3) {
+                if (lua_type(L, 4) == LUA_TSTRING) {
+                    d = nodelib_getdir(L, 4);
+                } else {
+                    lua_pushstring(L, "incorrect 4th argument");
+                }
+            }
         }
     }
-    p = hpack(n, w, m);
+    p = hpack(n, w, m, d);
     lua_nodelib_push_fast(L, p);
     return 1;
 }
@@ -516,7 +524,8 @@ static int lua_nodelib_dimensions(lua_State * L)
         int g_sign = normal;
         int g_order = normal;
         int i = 1;
-        halfword n, p = null;
+        int d = -1;
+        halfword n = null, p = null;
         if (lua_isnumber(L, 1)) {
             if (top < 4) {
                 lua_pushnil(L);
@@ -529,9 +538,16 @@ static int lua_nodelib_dimensions(lua_State * L)
         }
         n = *(check_isnode(L, i));
         if (lua_gettop(L) > i && !lua_isnil(L, (i + 1))) {
-            p = *(check_isnode(L, (i + 1)));
+            if (lua_type(L, (i + 1)) == LUA_TSTRING) {
+                d = nodelib_getdir(L, (i + 1));
+            } else {
+                p = *(check_isnode(L, (i + 1)));
+            }
         }
-        siz = natural_sizes(n, p, g_mult, g_sign, g_order);
+        if (lua_gettop(L) > (i + 1) && lua_type(L, (i + 2)) == LUA_TSTRING) {
+            d = nodelib_getdir(L, (i + 2));
+        }
+        siz = natural_sizes(n, p, g_mult, g_sign, g_order, d);
         lua_pushnumber(L, siz.wd);
         lua_pushnumber(L, siz.ht);
         lua_pushnumber(L, siz.dp);
@@ -552,6 +568,7 @@ static int lua_nodelib_vpack(lua_State * L)
     char *s;
     integer w = 0;
     int m = 1;
+    int d = -1;
     n = *(check_isnode(L, 1));
     if (lua_gettop(L) > 1) {
         w = lua_tointeger(L, 2);
@@ -567,6 +584,13 @@ static int lua_nodelib_vpack(lua_State * L)
                                    "3rd argument should be either additional or exactly");
                     lua_error(L);
                 }
+                if (lua_gettop(L) > 3) {
+                    if (lua_type(L, 4) == LUA_TSTRING) {
+                        d = nodelib_getdir(L, 4);
+                    } else {
+                        lua_pushstring(L, "incorrect 4th argument");
+                    }
+                }
             }
 
             else if (lua_type(L, 3) == LUA_TNUMBER) {
@@ -576,7 +600,7 @@ static int lua_nodelib_vpack(lua_State * L)
             }
         }
     }
-    p = vpackage(n, w, m, max_dimen);
+    p = vpackage(n, w, m, max_dimen, d);
     lua_nodelib_push_fast(L, p);
     return 1;
 }
@@ -1025,21 +1049,30 @@ static int lua_nodelib_count(lua_State * L)
 #define nodelib_pushattr(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
 #define nodelib_pushspec(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
 #define nodelib_pushaction(L,n) { lua_pushnumber(L,n); lua_nodelib_push(L); }
-#define nodelib_pushstring(L,n) { lua_pushstring(L,makecstring(n)); }
+#define nodelib_pushstring(L,n) { char *ss=makecstring(n); lua_pushstring(L,ss); free(ss); }
 
 static void nodelib_pushdir(lua_State * L, int n, boolean dirnode)
 {
-    int f = 0;
-    char dirstring[5] = { 0 };
+    char s[2];
     if (dirnode) {
-        dirstring[f++] = (n < 0 ? '-' : '+');
+        s[0] = (n < 0 ? '-' : '+');
+        s[1] = 0;
+    } else {
+        s[0] = 0;
     }
     if (n < 0)
         n += 64;
-    dirstring[f++] = dir_names[(int) dir_primary[n]];
-    dirstring[f++] = dir_names[(int) dir_secondary[n]];
-    dirstring[f++] = dir_names[(int) dir_tertiary[n]];
-    lua_pushstring(L, dirstring);
+    if (n == dir_TLT) {
+        lua_pushfstring(L, "%sTLT", s);
+    } else if (n == dir_TRT) {
+        lua_pushfstring(L, "%sTRT", s);
+    } else if (n == dir_LTL) {
+        lua_pushfstring(L, "%sLTL", s);
+    } else if (n == dir_RTT) {
+        lua_pushfstring(L, "%sRTT", s);
+    } else {
+        lua_pushstring(L, "???");
+    }
 }
 
 static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
@@ -1196,7 +1229,10 @@ static void lua_nodelib_getfield_whatsit(lua_State * L, int n, int field)
                 lua_pushnumber(L, height(n));
                 break;
             case 7:
-                lua_pushnumber(L, pdf_ximage_objnum(n));
+                lua_pushnumber(L, pdf_ximage_transform(n));
+                break;
+            case 8:
+                lua_pushnumber(L, pdf_ximage_index(n));
                 break;
             default:
                 lua_pushnil(L);
@@ -2027,7 +2063,6 @@ static int nodelib_getdir(lua_State * L, int n)
 {
     char *s = NULL;
     int d = 32;                 /* invalid number */
-    int a = -1, b = -1, c = -1;
     if (lua_type(L, n) == LUA_TSTRING) {
         s = (char *) lua_tostring(L, n);
         if (strlen(s) == 3) {
@@ -2041,36 +2076,19 @@ static int nodelib_getdir(lua_State * L, int n)
             s++;
         }
         if (strlen(s) == 3) {
-            switch (*s) {
-                /*  *INDENT-OFF* */
-                case 'T': a=0; break;
-                case 'L': a=1; break;
-                case 'B': a=2; break;
-                case 'R': a=3; break;
-                /* *INDENT-ON* */
-            }
-            switch (*(s + 1)) {
-                /*  *INDENT-OFF* */
-                case 'T': b=0; break;
-                case 'L': b=1; break;
-                case 'B': b=2; break;
-                case 'R': b=3; break;
-                /* *INDENT-ON* */
-            }
-            switch (*(s + 2)) {
-                /*  *INDENT-OFF* */
-                case 'T': c=0; break;
-                case 'L': c=1; break;
-                case 'B': c=2; break;
-                case 'R': c=3; break;
-                /* *INDENT-ON* */
+            if (strcmp(s, "TLT") == 0) {
+                d += dir_TLT;
+            } else if (strcmp(s, "TRT") == 0) {
+                d += dir_TRT;
+            } else if (strcmp(s, "LTL") == 0) {
+                d += dir_LTL;
+            } else if (strcmp(s, "RTT") == 0) {
+                d += dir_RTT;
             }
         }
-        if (a != -1 && b != -1 && c != -1 && !dir_parallel(a, b)) {
-            d += (a * 8 + dir_rearrange[b] * 4 + c);
-        }
-    } else if (lua_isnumber(L, n)) {
-        d = lua_tonumber(L, n);
+    } else {
+        lua_pushstring(L, "Direction specifiers have to be strings");
+        lua_error(L);
     }
     if ((d > 31) || (d < -64) || (d < 0 && (d + 64) > 31)) {
         d = 0;
@@ -2268,7 +2286,10 @@ static int lua_nodelib_setfield_whatsit(lua_State * L, int n, int field)
             height(n) = lua_tointeger(L, 3);
             break;
         case 7:
-            pdf_ximage_objnum(n) = lua_tointeger(L, 3);
+            pdf_ximage_transform(n) = lua_tointeger(L, 3);
+            break;
+        case 8:
+            pdf_ximage_index(n) = lua_tointeger(L, 3);
             break;
         default:
             return nodelib_cantset(L, field, n);
