@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: lstatslib.c 3078 2009-10-25 22:52:31Z oneiros $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/lstatslib.c $";
+    "$Id: lstatslib.c 3226 2009-12-04 15:50:51Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/lstatslib.c $";
 
 #include "lua/luatex-api.h"
 #include <ptexlib.h>
@@ -42,6 +42,11 @@ char *getbanner(void)
     return ptexbanner;
 }
 
+char *getlogname(void)
+{
+    return (char *)texmf_log_name;
+}
+
 char *get_pdftex_banner(void)
 {
     return pdftex_banner;
@@ -61,14 +66,9 @@ char *getfilename(void)
     while ((level > 0)) {
         t = input_stack[level--].name_field;
         if (t >= STRING_OFFSET)
-            break;
+            return (char *)str_string(t);
     }
-    /* the outermost level could be terminal input, 
-       so this test is still needed */
-    if (t >= STRING_OFFSET)
-        return makecstring(t);
-    else
-        return xstrdup("");
+    return xstrdup("");
 }
 
 char *getlasterror(void)
@@ -78,7 +78,7 @@ char *getlasterror(void)
 
 char *luatexrevision(void)
 {
-    return makecstring(get_luatexrevision());
+    return (strrchr(luatex_version_string,'.')+1);
 }
 
 static lua_Number get_pdf_gone(void)
@@ -164,6 +164,9 @@ extern int luastate_bytes;
 extern int callback_count;
 extern int saved_callback_count;
 
+/* temp, for backward compat */
+static int init_pool_ptr = 0;
+
 static struct statistic stats[] = {
     {"pdf_gone", 'N', &get_pdf_gone},
     {"pdf_ptr", 'N', &get_pdf_ptr},
@@ -171,7 +174,7 @@ static struct statistic stats[] = {
     {"dvi_ptr", 'g', &dvi_ptr},
     {"total_pages", 'g', &total_pages},
     {"output_file_name", 'S', &get_output_file_name},
-    {"log_name", 's', &texmf_log_name}, /* weird */
+    {"log_name", 'S', &getlogname},
     {"banner", 'S', &getbanner},
     {"pdftex_banner", 'S', &get_pdftex_banner},
     {"luatex_version", 'G', &get_luatexversion},
@@ -188,7 +191,7 @@ static struct statistic stats[] = {
     {"str_ptr", 'g', &str_ptr},
     {"init_str_ptr", 'g', &init_str_ptr},
     {"max_strings", 'g', &max_strings},
-    {"pool_ptr", 'g', &pool_ptr},
+    {"pool_ptr", 'g', &pool_size},
     {"init_pool_ptr", 'g', &init_pool_ptr},
     {"pool_size", 'g', &pool_size},
     {"var_mem_max", 'g', &var_mem_max},
@@ -267,7 +270,9 @@ static int do_getstat(lua_State * L, int i)
     case 's':
         str = *(integer *) (stats[i].value);
         if (str) {
-            lua_pushstring(L, makecstring(str));
+            char *ss = makecstring(str);
+            lua_pushstring(L, ss);
+            free(ss);
         } else {
             lua_pushnil(L);
         }
