@@ -21,7 +21,7 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: ltexiolib.c 2869 2009-07-16 11:31:18Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/ltexiolib.c $";
+    "$Id: ltexiolib.c 3217 2009-12-03 16:52:01Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/ltexiolib.c $";
 
 typedef void (*texio_printer) (char *);
 
@@ -110,7 +110,7 @@ static void do_texio_ini_print(lua_State * L, char *extra)
 
 static int texio_print(lua_State * L)
 {
-    if (ready_already != 314159 || pool_ptr == 0 || job_name == 0) {
+    if (ready_already != 314159 || job_name == 0) {
         do_texio_ini_print(L, "");
         return 0;
     }
@@ -119,7 +119,7 @@ static int texio_print(lua_State * L)
 
 static int texio_printnl(lua_State * L)
 {
-    if (ready_already != 314159 || pool_ptr == 0 || job_name == 0) {
+    if (ready_already != 314159 || job_name == 0) {
         do_texio_ini_print(L, "\n");
         return 0;
     }
