@@ -23,7 +23,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: ltexlib.c 3078 2009-10-25 22:52:31Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/ltexlib.c $";
+    "$Id: ltexlib.c 3218 2009-12-04 08:21:56Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/ltexlib.c $";
 
 #define attribute(A) eqtb[attribute_base+(A)].hh.rh
 #define dimen(A) eqtb[scaled_base+(A)].hh.rh
@@ -174,8 +174,7 @@ int luacstring_input(void)
         last = first;
         ret = last;
         check_buffer_overflow(last + t->tsize);
-        /* make sure it fits in the pool as well (for show_token_list c.s) */
-        check_pool_overflow(pool_ptr + t->tsize);
+
         while (t->tsize-- > 0)
             buffer[last++] = *st++;
         if (!t->partial) {
@@ -482,10 +481,9 @@ static int getattribute(lua_State * L)
 
 int vsettoks(lua_State * L, int is_global)
 {
-    int i, j, err;
-    size_t len;
+    int i, err;
     integer k;
-    char *st;
+    lstring str;
     integer save_global_defs = int_par(global_defs_code);
     if (is_global)
         int_par(global_defs_code) = 1;
@@ -494,14 +492,12 @@ int vsettoks(lua_State * L, int is_global)
         lua_pushstring(L, "unsupported value type");
         lua_error(L);
     }
-    st = (char *) lua_tolstring(L, i, &len);
+    str.s = (unsigned char *) lua_tolstring(L, i, &str.l);
     k = get_item_index(L, (i - 1), toks_base);
     check_index_range(k, "settoks");
-    j = maketexlstring(st, len);
-    err = set_tex_toks_register(k, j);
+    err = set_tex_toks_register(k, str);
     int_par(global_defs_code) = save_global_defs;
     if (err) {
-        flush_str(j);
         lua_pushstring(L, "incorrect value");
         lua_error(L);
     }
@@ -524,10 +520,13 @@ static int gettoks(lua_State * L)
 {
     integer k;
     str_number t;
+    char *ss;
     k = get_item_index(L, lua_gettop(L), toks_base);
     check_index_range(k, "gettoks");
     t = get_tex_toks_register(k);
-    lua_pushstring(L, makecstring(t));
+    ss = makecstring(t);
+    lua_pushstring(L, ss);
+    free(ss);
     flush_str(t);
     return 1;
 }
@@ -579,6 +578,13 @@ static int vsetbox(lua_State * L, int is_global)
             return 0;
     } else {
         j = nodelist_from_lua(L);
+        if (j != null && type(j) != hlist_node && type(j) != vlist_node) {
+            lua_pushfstring(L, "setbox: incompatible node type (%s)\n",
+                            get_node_name(type(j), subtype(j)));
+            lua_error(L);
+            return 0;
+        }
+
     }
     err = set_tex_box_register(k, j);
     int_par(global_defs_code) = save_global_defs;
@@ -778,22 +784,6 @@ int settex(lua_State * L)
     return 0;
 }
 
-char *get_something_internal(int cur_cmd, int cur_code)
-{
-    int texstr;
-    char *str;
-    int save_cur_val, save_cur_val_level;
-    save_cur_val = cur_val;
-    save_cur_val_level = cur_val_level;
-    scan_something_simple(cur_cmd, cur_code);
-    texstr = the_scanned_result();
-    cur_val = save_cur_val;
-    cur_val_level = save_cur_val_level;
-    str = makecstring(texstr);
-    flush_str(texstr);
-    return str;
-}
-
 int do_convert(lua_State * L, int cur_code)
 {
     int texstr;
@@ -835,10 +825,12 @@ int do_convert(lua_State * L, int cur_code)
             flush_str(texstr);
         }
     }
-    if (str)
+    if (str) {
         lua_pushstring(L, str);
-    else
+        free(str);
+    } else {
         lua_pushnil(L);
+    }
     return 1;
 }
 
@@ -860,10 +852,12 @@ int do_scan_internal(lua_State * L, int cur_cmd, int cur_code)
     } else {                    /* dir_val_level, mu_val_level, tok_val_level */
         texstr = the_scanned_result();
         str = makecstring(texstr);
-        if (str)
+        if (str) {
             lua_pushstring(L, str);
-        else
+            free(str);
+        } else {
             lua_pushnil(L);
+        }
         flush_str(texstr);
     }
     cur_val = save_cur_val;
@@ -1313,7 +1307,9 @@ static int tex_hashpairs(lua_State * L)
     while (cs < eqtb_size) {
         s = hash_text(cs);
         if (s > 0) {
-            lua_pushstring(L, makecstring(s));
+            char *ss = makecstring(s);
+            lua_pushstring(L, ss);
+            free(ss);
             cmd = eq_type(cs);
             chr = equiv(cs);
             make_token_table(L, cmd, chr, cs);
@@ -1333,7 +1329,9 @@ static int tex_primitives(lua_State * L)
     while (cs < prim_size) {
         s = get_prim_text(cs);
         if (s > 0) {
-            lua_pushstring(L, makecstring(s));
+            char *ss = makecstring(s);
+            lua_pushstring(L, ss);
+            free(ss);
             cmd = get_prim_eq_type(cs);
             chr = get_prim_equiv(cs);
             make_token_table(L, cmd, chr, 0);
@@ -1382,7 +1380,9 @@ static int tex_extraprimitives(lua_State * L)
         s = get_prim_text(cs);
         if (s > 0) {
             if (get_prim_origin(cs) & mask) {
-                lua_pushstring(L, makecstring(s));
+                char *ss = makecstring(s);
+                lua_pushstring(L, ss);
+                free(ss);
                 lua_rawseti(L, -2, i++);
             }
         }
@@ -1473,20 +1473,16 @@ static int tex_run_boot(lua_State * L)
         return 1;
     }
     if (format) {
-        if (nameoffile)
-            xfree(nameoffile);
-        nameoffile = xmallocarray(packed_ASCII_code, strlen(format) + 2);
-        strcpy((char *) (nameoffile + 1), format);
-        if (!w_open_in(fmt_file)) {
+        if (!zopen_w_input(&fmt_file, format, DUMP_FORMAT, FOPEN_RBIN_MODE)) {
             lua_pushboolean(L, 0);      /* false */
             return 1;
         }
-        if (!load_fmt_file()) {
-            w_close(fmt_file);
+        if (!load_fmt_file(format)) {
+            zwclose(fmt_file);
             lua_pushboolean(L, 0);      /* false */
             return 1;
         }
-        w_close(fmt_file);
+        zwclose(fmt_file);
     }
     fix_date_and_time();
     if (format == NULL)
