@@ -21,7 +21,7 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: ltokenlib.c 2899 2009-07-21 23:03:53Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/ltokenlib.c $";
+    "$Id: ltokenlib.c 3218 2009-12-04 08:21:56Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/ltokenlib.c $";
 
 extern int get_command_id(char *);
 
@@ -35,9 +35,9 @@ extern int get_command_id(char *);
 static unsigned char *get_cs_text(integer cs)
 {
     if (cs == null_cs)
-        return (unsigned char *) "\\csname\\endcsname";
+        return (unsigned char *) xstrdup("\\csname\\endcsname");
     else if ((cs_text(cs) < 0) || (cs_text(cs) >= str_ptr))
-        return (unsigned char *) "";
+        return (unsigned char *) xstrdup("");
     else
         return (unsigned char *) makecstring(cs_text(cs));
 }
