@@ -22,7 +22,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: luanode.c 3005 2009-08-16 20:48:36Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/luanode.c $";
+    "$Id: luanode.c 3187 2009-11-22 22:29:05Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/luanode.c $";
 
 static char *group_code_names[] = {
     "",
@@ -158,7 +158,8 @@ lua_linebreak_callback(int is_broken, halfword head_node, halfword * new_head)
 
 
 halfword
-lua_hpack_filter(halfword head_node, scaled size, int pack_type, int extrainfo)
+lua_hpack_filter(halfword head_node, scaled size, int pack_type, int extrainfo,
+                 int pack_direction)
 {
     halfword ret;
     lua_State *L = Luas;
@@ -173,7 +174,11 @@ lua_hpack_filter(halfword head_node, scaled size, int pack_type, int extrainfo)
     lua_pushstring(L, group_code_names[extrainfo]);
     lua_pushnumber(L, size);
     lua_pushstring(L, pack_type_name[pack_type]);
-    if (lua_pcall(L, 4, 1, 0) != 0) {   /* no arg, 1 result */
+    if (pack_direction >= 0)
+        lua_pushstring(L, string_dir(pack_direction));
+    else
+        lua_pushnil(L);
+    if (lua_pcall(L, 5, 1, 0) != 0) {   /* no arg, 1 result */
         fprintf(stdout, "error: %s\n", lua_tostring(L, -1));
         lua_pop(L, 2);
         error();
@@ -197,7 +202,7 @@ lua_hpack_filter(halfword head_node, scaled size, int pack_type, int extrainfo)
 
 halfword
 lua_vpack_filter(halfword head_node, scaled size, int pack_type, scaled maxd,
-                 int extrainfo)
+                 int extrainfo, int pack_direction)
 {
     halfword ret;
     integer callback_id;
@@ -221,7 +226,11 @@ lua_vpack_filter(halfword head_node, scaled size, int pack_type, scaled maxd,
     lua_pushnumber(L, size);
     lua_pushstring(L, pack_type_name[pack_type]);
     lua_pushnumber(L, maxd);
-    if (lua_pcall(L, 5, 1, 0) != 0) {   /* no arg, 1 result */
+    if (pack_direction >= 0)
+        lua_pushstring(L, string_dir(pack_direction));
+    else
+        lua_pushnil(L);
+    if (lua_pcall(L, 6, 1, 0) != 0) {   /* no arg, 1 result */
         fprintf(stdout, "error: %s\n", lua_tostring(L, -1));
         lua_pop(L, 2);
         error();
@@ -275,8 +284,12 @@ void lua_pdf_literal(PDF pdf, int i)
     size_t l = 0;
     lua_rawgeti(Luas, LUA_REGISTRYINDEX, i);
     s = (char *) lua_tolstring(Luas, -1, &l);
-    while (l--) {
-        pdf_out(pdf, *s++);
+    if (l < max_single_pdf_print) {
+        pdf_out_block(pdf, s, l);
+    } else {
+        while (l--) {
+            pdf_out(pdf, *s++);
+        }
     }
     pdf_out(pdf, 10);           /* pdf_print_nl */
     lua_pop(Luas, 1);
