@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char _svn_version[] =
-    "$Id: luastuff.c 3070 2009-10-18 17:57:33Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/luastuff.c $";
+    "$Id: luastuff.c 3217 2009-12-03 16:52:01Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/lua/luastuff.c $";
 
 #include "lua/luatex-api.h"
 #include <ptexlib.h>
@@ -134,11 +134,6 @@ void luainterpreter(void)
 
     open_oslibext(L, safer_option);
 
-    lua_getglobal(L, "package");
-    lua_pushstring(L, "");
-    lua_setfield(L, -2, "cpath");
-    lua_pop(L, 1);              /* pop the table */
-
     /* luasockets */
     /* socket and mime are a bit tricky to open because
      * they use a load-time  dependency that has to be 
@@ -389,15 +384,6 @@ void luatokencall(int p, int nameptr)
     lua_active--;
 }
 
-
-
-void luatex_load_init(int s, LoadS * ls)
-{
-    ls->s = (const char *) &(str_pool[str_start[s]]);
-    ls->size = str_start[s + 1] - str_start[s];
-}
-
-
 lua_State *luatex_error(lua_State * L, int is_fatal)
 {
 
