@@ -12,15 +12,15 @@
 #include "luatex.h"
 
 static const char __svn_version[] =
-    "$Id: luatex.c 3081 2009-10-25 22:57:52Z oneiros $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/luatex.c $";
+    "$Id: luatex.c 3217 2009-12-03 16:52:01Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/luatex.c $";
 
 #define TeX
 
-int luatex_version = 44;        /* \.{\\luatexversion}  */
+int luatex_version = 47;        /* \.{\\luatexversion}  */
 int luatex_revision = '0';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-char *luatex_version_string = "beta-0.44.0";
+char *luatex_version_string = "beta-0.47.0";
 char *engine_name = "luatex";   /* the name of this engine */
 
 
@@ -385,40 +385,6 @@ int runsystem(char *cmd)
     return allow;
 }
 
-/* Like runsystem(), the runpopen() function is called only when
-   shellenabledp == 1.   Unlike runsystem(), here we write errors to
-   stderr, since we have nowhere better to use; and of course we return
-   a file handle (or NULL) instead of a status indicator.  */
-
-static FILE *runpopen(char *cmd, char *mode)
-{
-    FILE *f = NULL;
-    char *safecmd = NULL;
-    char *cmdname = NULL;
-    int allow;
-
-    /* If restrictedshell == 0, any command is allowed. */
-    if (restrictedshell == 0)
-        allow = 1;
-    else
-        allow = shell_cmd_is_allowed(&cmd, &safecmd, &cmdname);
-
-    if (allow == 1)
-        f = popen(cmd, mode);
-    else if (allow == 2)
-        f = popen(safecmd, mode);
-    else if (allow == -1)
-        fprintf(stderr, "\nrunpopen quotation error in command line: %s\n",
-                cmd);
-    else
-        fprintf(stderr, "\nrunpopen command not allowed: %s\n", cmdname);
-
-    if (safecmd)
-        free(safecmd);
-    if (cmdname)
-        free(cmdname);
-    return f;
-}
 #endif
 
 /* The main program, etc.  */
@@ -429,7 +395,7 @@ extern void lua_initialize(int ac, char **av);
 char **argv;
 int argc;
 
-/* The C version of what might wind up in DUMP_VAR.  */
+/* The C version of what might wind up in |TEX_format_default|.  */
 const_string dump_name;
 
 /* The C version of the jobname, if given. */
@@ -722,231 +688,7 @@ string normalize_quotes(const_string name, const_string mesg)
     return ret;
 }
 
-/* Return true if FNAME is acceptable as a name for \openout, \openin, or
-   \input.  */
 
-typedef enum ok_type {
-    ok_reading,
-    ok_writing
-} ok_type;
-
-static const_string ok_type_name[] = {
-    "reading",
-    "writing"
-};
-
-static boolean
-opennameok(const_string fname, const_string check_var,
-           const_string default_choice, ok_type action)
-{
-    /* We distinguish three cases:
-       'a' (any)        allows any file to be opened.
-       'r' (restricted) means disallowing special file names.
-       'p' (paranoid)   means being really paranoid: disallowing special file
-       names and restricting output files to be in or below
-       the working directory or $TEXMFOUTPUT, while input files
-       must be below the current directory, $TEXMFOUTPUT, or
-       (implicitly) in the system areas.
-       We default to "paranoid".  The error messages from TeX will be somewhat
-       puzzling...
-       This function contains several return statements...  */
-
-    const_string open_choice = kpse_var_value(check_var);
-
-    if (!open_choice)
-        open_choice = default_choice;
-
-    if (*open_choice == 'a' || *open_choice == 'y' || *open_choice == '1')
-        return true;
-
-#if defined (unix) && !defined (MSDOS)
-    {
-        const_string base = xbasename(fname);
-        /* Disallow .rhosts, .login, etc.  Allow .tex (for LaTeX).  */
-        if (base[0] == 0 ||
-            (base[0] == '.' && !IS_DIR_SEP(base[1]) && !STREQ(base, ".tex"))) {
-            fprintf(stderr, "%s: Not %s to %s (%s = %s).\n",
-                    program_invocation_name, ok_type_name[action], fname,
-                    check_var, open_choice);
-            return false;
-        }
-    }
-#else
-    /* Other OSs don't have special names? */
-#endif
-
-    if (*open_choice == 'r' || *open_choice == 'n' || *open_choice == '0')
-        return true;
-
-    /* Paranoia supplied by Charles Karney...  */
-    if (kpse_absolute_p(fname, false)) {
-        const_string texmfoutput = kpse_var_value("TEXMFOUTPUT");
-        /* Absolute pathname is only OK if TEXMFOUTPUT is set, it's not empty,
-           fname begins the TEXMFOUTPUT, and is followed by / */
-        if (!texmfoutput || *texmfoutput == '\0'
-            || fname != strstr(fname, texmfoutput)
-            || !IS_DIR_SEP(fname[strlen(texmfoutput)])) {
-            fprintf(stderr, "%s: Not %s to %s (%s = %s).\n",
-                    program_invocation_name, ok_type_name[action], fname,
-                    check_var, open_choice);
-            return false;
-        }
-    }
-    /* For all pathnames, we disallow "../" at the beginning or "/../"
-       anywhere.  */
-    if (fname[0] == '.' && fname[1] == '.' && IS_DIR_SEP(fname[2])) {
-        fprintf(stderr, "%s: Not %s to %s (%s = %s).\n",
-                program_invocation_name, ok_type_name[action], fname,
-                check_var, open_choice);
-        return false;
-    } else {
-        /* Check for "/../".  Since more than one characted can be matched
-           by IS_DIR_SEP, we cannot use "/../" itself. */
-        const_string dotpair = strstr(fname, "..");
-        while (dotpair) {
-            /* If dotpair[2] == DIR_SEP, then dotpair[-1] is well-defined,
-               because the "../" case was handled above. */
-            if (IS_DIR_SEP(dotpair[2]) && IS_DIR_SEP(dotpair[-1])) {
-                fprintf(stderr, "%s: Not %s to %s (%s = %s).\n",
-                        program_invocation_name, ok_type_name[action], fname,
-                        check_var, open_choice);
-                return false;
-            }
-            /* Continue after the dotpair. */
-            dotpair = strstr(dotpair + 2, "..");
-        }
-    }
-
-    /* We passed all tests.  */
-    return true;
-}
-
-boolean openinnameok(const_string fname)
-{
-    /* For input default to all. */
-    return opennameok(fname, "openin_any", "a", ok_reading);
-}
-
-boolean openoutnameok(const_string fname)
-{
-    /* For output, default to paranoid. */
-    return opennameok(fname, "openout_any", "p", ok_writing);
-}
-
-/* 
-  piped I/O
- */
-
-/* The code that implements popen() needs an array for tracking 
-   possible pipe file pointers, because these need to be
-   closed using pclose().
-*/
-
-static FILE *pipes[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
-};
-
-boolean open_in_or_pipe(FILE ** f_ptr, int filefmt, const_string fopen_mode)
-{
-    string fname = NULL;
-    int i;                      /* iterator */
-
-    /* opening a read pipe is straightforward, only have to
-       skip past the pipe symbol in the file name. filename
-       quoting is assumed to happen elsewhere (it does :-)) */
-
-    if (shellenabledp && *(nameoffile + 1) == '|') {
-        /* the user requested a pipe */
-        *f_ptr = NULL;
-        fname = (string) xmalloc(strlen((const_string) (nameoffile + 1)) + 1);
-        strcpy(fname, (const_string) (nameoffile + 1));
-#if !defined(pdfTeX)
-        if (fullnameoffile)
-            free(fullnameoffile);
-        fullnameoffile = xstrdup(fname);
-#endif
-        recorder_record_input(fname + 1);
-        *f_ptr = runpopen(fname + 1, "r");
-        free(fname);
-        for (i = 0; i <= 15; i++) {
-            if (pipes[i] == NULL) {
-                pipes[i] = *f_ptr;
-                break;
-            }
-        }
-        if (*f_ptr)
-            setvbuf(*f_ptr, (char *) NULL, _IOLBF, 0);
-
-        return *f_ptr != NULL;
-    }
-
-    return open_input(f_ptr, filefmt, fopen_mode);
-}
-
-
-boolean open_out_or_pipe(FILE ** f_ptr, const_string fopen_mode)
-{
-    string fname;
-    int i;                      /* iterator */
-
-    /* opening a write pipe takes a little bit more work, because TeX
-       will perhaps have appended ".tex".  To avoid user confusion as
-       much as possible, this extension is stripped only when the command
-       is a bare word.  Some small string trickery is needed to make
-       sure the correct number of bytes is free()-d afterwards */
-
-    if (shellenabledp && *(nameoffile + 1) == '|') {
-        /* the user requested a pipe */
-        fname = (string) xmalloc(strlen((const_string) (nameoffile + 1)) + 1);
-        strcpy(fname, (const_string) (nameoffile + 1));
-        if (strchr(fname, ' ') == NULL && strchr(fname, '>') == NULL) {
-            /* mp and mf currently do not use this code, but it 
-               is better to be prepared */
-            if (STREQ((fname + strlen(fname) - 3), "tex"))
-                *(fname + strlen(fname) - 4) = 0;
-            *f_ptr = runpopen(fname + 1, "w");
-            *(fname + strlen(fname)) = '.';
-        } else {
-            *f_ptr = runpopen(fname + 1, "w");
-        }
-        recorder_record_output(fname + 1);
-        free(fname);
-
-        for (i = 0; i <= 15; i++) {
-            if (pipes[i] == NULL) {
-                pipes[i] = *f_ptr;
-                break;
-            }
-        }
-
-        if (*f_ptr)
-            setvbuf(*f_ptr, (char *) NULL, _IOLBF, 0);
-
-        return *f_ptr != NULL;
-    }
-
-    return open_output(f_ptr, fopen_mode);
-}
-
-
-void close_file_or_pipe(FILE * f)
-{
-    int i;                      /* iterator */
-
-    if (shellenabledp) {
-        /* if this file was a pipe, pclose() it and return */
-        for (i = 0; i <= 15; i++) {
-            if (pipes[i] == f) {
-                if (f)
-                    pclose(f);
-                pipes[i] = NULL;
-                return;
-            }
-        }
-    }
-    close_file(f);
-}
-
 /* All our interrupt handler has to do is set TeX's or Metafont's global
    variable `interrupt'; then they will do everything needed.  */
 #ifdef WIN32
@@ -1339,12 +1081,6 @@ void setupboundvariable(integer * var, const_string var_name, integer dflt)
     }
 }
 
-/* FIXME -- some (most?) of this can/should be moved to the Pascal/WEB side. */
-
-str_number makefullnamestring()
-{
-    return maketexstring(fullnameoffile);
-}
 
 /* Get the job name to be used, which may have been set from the
    command line. */
