@@ -21,9 +21,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: ocp.c 2886 2009-07-17 13:47:29Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/ocp/ocp.c $";
-
-#define flush_string() do { decr(str_ptr); pool_ptr=str_start_macro(str_ptr); } while (0)
+    "$Id: ocp.c 3221 2009-12-04 10:49:35Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/ocp/ocp.c $";
 
 int **ocp_tables;
 
@@ -44,7 +42,6 @@ void new_ocp(small_number a)
     pointer u;                  /* user's ocp identifier */
     internal_ocp_number f;      /* runs through existing ocps */
     str_number t;               /* name for the frozen ocp identifier */
-    str_number flushable_string;        /* string not yet referenced */
     boolean external_ocp = false;       /* external binary file */
     if (job_name == 0)
         open_log_file();
@@ -66,18 +63,23 @@ void new_ocp(small_number a)
        the new name becomes the ocp identifier of record. OCP names `\.{xyz}' and
        `\.{XYZ}' are considered to be different.
      */
-    flushable_string = str_ptr - 1;
     for (f = ocp_base + 1; f <= ocp_ptr; f++) {
         if (str_eq_str(ocp_name(f), cur_name)
             && str_eq_str(ocp_area(f), cur_area)) {
-            if (cur_name == flushable_string) {
-                flush_string();
-                cur_name = ocp_name(f);
-            }
+            flush_str(cur_name);
+            flush_str(cur_area);
+            cur_name = ocp_name(f);
+            cur_area = ocp_area(f);
             goto COMMON_ENDING;
         }
     }
-    f = read_ocp_info(u, cur_name, cur_area, cur_ext, external_ocp);
+    {
+        char *nam = makecstring(cur_name);
+        char *are = makecstring(cur_area);
+        char *ext = makecstring(cur_ext);
+        f = read_ocp_info(u, nam, are, ext, external_ocp);
+        free(nam); free(are); free(ext);
+    }
   COMMON_ENDING:
     equiv(u) = f;
     eqtb[ocp_id_base + f] = eqtb[u];
