@@ -23,7 +23,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: runocp.c 2886 2009-07-17 13:47:29Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/ocp/runocp.c $";
+    "$Id: runocp.c 3218 2009-12-04 08:21:56Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/ocp/runocp.c $";
 
 memory_word active_info[(active_mem_size + 1)];
 active_index active_min_ptr = 0;        /* first unused word of |active_info| */
@@ -773,17 +773,15 @@ void run_ocp(void)
             run_otp();
     } else {
         /* k=0;  ? */
-        char *cmd = xstrdup(makecstring(ocp_external(otp_input_ocp)));
-        char *arg = xstrdup(makecstring(ocp_external_arg(otp_input_ocp)));
-        if (nameoffile)
-            xfree(nameoffile);
-        nameoffile = xmalloc(strlen(cmd) + strlen(arg) + 3);
-        strcpy((char *) (nameoffile + 1), cmd);
-        strcat((char *) (nameoffile + 1), " ");
-        strcat((char *) (nameoffile + 1), arg);
+        char *cmd = makecstring(ocp_external(otp_input_ocp));
+        char *arg = makecstring(ocp_external_arg(otp_input_ocp));
+        char *cmdline = xmalloc(strlen(cmd) + strlen(arg) + 3);
+        strcpy(cmdline, cmd);
+        strcat(cmdline, " ");
+        strcat(cmdline, arg);
         xfree(cmd);
         xfree(arg);
-        run_external_ocp((char *) (nameoffile + 1));
+        run_external_ocp(cmdline);
     }
 
 
