@@ -20,8 +20,8 @@
 #include "ptexlib.h"
 
 static const char __svn_version[] =
-    "$Id: pagetree.c 2777 2009-07-08 12:12:48Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pagetree.c $";
+    "$Id: pagetree.c 3234 2009-12-05 09:56:09Z hhenkel $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pagetree.c $";
 
 /**********************************************************************/
 /* Page diversions */
@@ -69,7 +69,7 @@ static pages_entry *new_pages_entry(PDF pdf)
     for (i = 0; i < PAGES_TREE_KIDSMAX; i++)
         p->kids[i] = 0;
     p->next = NULL;
-    pdf_create_obj(pdf, 0, 0);
+    pdf_create_obj(pdf, obj_type_pages, 0);
     p->objnum = pdf->obj_ptr;
     return p;
 }
@@ -206,15 +206,18 @@ void pdf_do_page_undivert(integer divnum, integer curdivnum)
 
 /* write a /Pages object */
 
+#define pdf_pages_attr equiv(pdf_pages_attr_loc)
+
 static void write_pages(PDF pdf, pages_entry * p, int parent)
 {
     int i;
     assert(p != NULL);
     pdf_begin_dict(pdf, p->objnum, 1);
     pdf_printf(pdf, "/Type /Pages\n");
-    if (parent == 0)            /* it's root */
-        print_pdf_pages_attr(pdf);
-    else
+    if (parent == 0) {          /* it's root */
+        if (pdf_pages_attr != null)
+            pdf_print_toks_ln(pdf, pdf_pages_attr);
+    } else
         pdf_printf(pdf, "/Parent %d 0 R\n", parent);
     pdf_printf(pdf, "/Count %d\n/Kids [", (int) p->number_of_pages);
     for (i = 0; i < p->number_of_kids; i++)
