@@ -22,8 +22,8 @@
 
 
 static const char __svn_version[] =
-    "$Id: pdfdest.c 2953 2009-08-05 18:44:50Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfdest.c $";
+    "$Id: pdfdest.c 3106 2009-11-08 12:04:04Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfdest.c $";
 
 #define pdf_dest_margin          dimen_par(pdf_dest_margin_code)
 
@@ -267,9 +267,9 @@ void scan_pdfdest(PDF pdf)
 
     if (pdf_dest_type(cur_list.tail_field) == pdf_dest_fitr) {
         alt_rule = scan_alt_rule();     /* scans |<rule spec>| to |alt_rule| */
-        set_pdf_width(cur_list.tail_field, alt_rule.wd);
-        set_pdf_height(cur_list.tail_field, alt_rule.ht);
-        set_pdf_depth(cur_list.tail_field, alt_rule.dp);
+        set_width(cur_list.tail_field, alt_rule.wd);
+        set_height(cur_list.tail_field, alt_rule.ht);
+        set_depth(cur_list.tail_field, alt_rule.dp);
     }
     if (pdf_dest_named_id(cur_list.tail_field) != 0) {
         i = tokens_to_string(pdf_dest_id(cur_list.tail_field));
