@@ -22,8 +22,8 @@
 
 
 static const char __svn_version[] =
-    "$Id: pdffont.c 2904 2009-07-23 21:19:48Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdffont.c $";
+    "$Id: pdffont.c 3221 2009-12-04 10:49:35Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdffont.c $";
 
 #define font_id_text(A) cs_text(font_id_base+(A))       /* a frozen font identifier's name */
 
@@ -45,116 +45,19 @@ void output_one_char(PDF pdf, internal_font_number ffi, integer c)
 {
     scaled_whd ci;              /* the real width, height and depth of the character */
     ci = get_charinfo_whd(ffi, c);
-    switch (box_direction(pdf->posstruct->dir)) {
-    case dir_TL_:
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__LL:
-        case dir__LR:
-            pos_down((ci.ht - ci.dp) / 2);
-            break;
-        case dir__LT:
-            break;
-        case dir__LB:
-            pos_down(ci.ht);
-            break;
-        }
+    switch (pdf->posstruct->dir) {
+    case dir_TLT:
         break;
-    case dir_TR_:
+    case dir_TRT:
         pos_left(ci.wd);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__RL:
-        case dir__RR:
-            pos_down((ci.ht - ci.dp) / 2);
-            break;
-        case dir__RT:
-            break;
-        case dir__RB:
-            pos_down(ci.ht);
-            break;
-        }
         break;
-    case dir_BL_:
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__LL:
-        case dir__LR:
-            pos_down((ci.ht - ci.dp) / 2);
-            break;
-        case dir__LT:
-            break;
-        case dir__LB:
-            pos_down(ci.ht);
-            break;
-        }
-        break;
-    case dir_BR_:
+    case dir_LTL:
+        pos_down(ci.ht);
         pos_left(ci.wd);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__RL:
-        case dir__RR:
-            pos_down((ci.ht - ci.dp) / 2);
-            break;
-        case dir__RT:
-            break;
-        case dir__RB:
-            pos_down(ci.ht);
-            break;
-        }
         break;
-    case dir_LT_:
+    case dir_RTT:
         pos_down(ci.ht);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__TL:
-            pos_left(ci.wd);
-            break;
-        case dir__TR:
-            break;
-        case dir__TB:
-        case dir__TT:
-            pos_left(ci.wd / 2);
-            break;
-        }
-        break;
-    case dir_RT_:
-        pos_down(ci.ht);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__TL:
-            pos_left(ci.wd);
-            break;
-        case dir__TR:
-            break;
-        case dir__TB:
-        case dir__TT:
-            pos_left(ci.wd / 2);
-            break;
-        }
-        break;
-    case dir_LB_:
-        pos_up(ci.dp);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__BL:
-            pos_left(ci.wd);
-            break;
-        case dir__BR:
-            break;
-        case dir__BB:
-        case dir__BT:
-            pos_left(ci.wd / 2);
-            break;
-        }
-        break;
-    case dir_RB_:
-        pos_up(ci.dp);
-        switch (font_direction(pdf->posstruct->dir)) {
-        case dir__BL:
-            pos_left(ci.wd);
-            break;
-        case dir__BR:
-            break;
-        case dir__BB:
-        case dir__BT:
-            pos_left(ci.wd / 2);
-            break;
-        }
+        pos_left(ci.wd / 2);
         break;
     }
     if (has_packet(ffi, c)) {
@@ -268,18 +171,13 @@ internal_font_number tfm_lookup(str_number s, scaled fs)
     if (fs != 0) {
         for (k = 1; k <= max_font_id(); k++) {
             if (cmp_font_name(k, s) && (font_size(k) == fs)) {
-                flush_str(s);
                 return k;
             }
         }
     } else {
         for (k = 1; k <= max_font_id(); k++) {
             if (cmp_font_name(k, s)) {
-                flush_string(); /* |font_name| */
-                flush_str(s);
                 return k;
-            } else {
-                flush_string();
             }
         }
     }
@@ -498,7 +396,7 @@ void make_font_copy(small_number a)
 void pdf_include_chars(PDF pdf)
 {
     str_number s;
-    pool_pointer k;             /* running indices */
+    unsigned char *k, *j;   /* running index */
     internal_font_number f;
     scan_font_ident();
     f = cur_val;
@@ -510,10 +408,9 @@ void pdf_include_chars(PDF pdf)
     scan_pdf_ext_toks();
     s = tokens_to_string(def_ref);
     delete_token_ref(def_ref);
-    k = str_start_macro(s);
-    while (k < str_start_macro(s + 1)) {
-        pdf_mark_char(f, str_pool[k]);
-        incr(k);
+    j = str_string(s)+str_length(s);
+    for (k = str_string(s); k < j; k++) {
+        pdf_mark_char(f, *k);
     }
     flush_str(s);
 }
