@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char __svn_version[] =
-    "$Id: pdfgen.c 3077 2009-10-25 22:51:23Z oneiros $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfgen.c $";
+    "$Id: pdfgen.c 3235 2009-12-05 11:36:51Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfgen.c $";
 
 #include "ptexlib.h"
 #include <ctype.h>
@@ -81,7 +81,7 @@ PDF init_pdf_struct(PDF pdf)
     pdf->posstruct = xmalloc(sizeof(posstructure));
     pdf->posstruct->pos.h = 0;
     pdf->posstruct->pos.v = 0;
-    pdf->posstruct->dir = dir_TL_;
+    pdf->posstruct->dir = dir_TLT;
 
     pdf->obj_tab_size = inf_obj_tab_size;       /* allocated size of |obj_tab| array */
     pdf->obj_tab = xmalloc((pdf->obj_tab_size + 1) * sizeof(obj_entry));
@@ -293,7 +293,8 @@ void pdf_os_prepare_obj(PDF pdf, integer i, integer pdf_os_level)
                         && (pdf->objcompresslevel >= pdf_os_level)));
     if (pdf->os_mode) {
         if (pdf->os_cur_objnum == 0) {
-            pdf->os_cur_objnum = pdf_new_objnum(pdf);
+            pdf_create_obj(pdf, obj_type_objstm, pdf->sys_obj_ptr + 1);
+            pdf->os_cur_objnum = pdf->sys_obj_ptr;
             pdf->obj_ptr--;     /* object stream is not accessible to user */
             pdf->os_cntr++;     /* only for statistics */
             pdf->os_idx = 0;
@@ -351,15 +352,15 @@ void pdf_room(PDF pdf, integer n)
  * (40) and  right parenthesis (41)
  */
 
-#define pdf_print_escaped(c)                                            \
-  if ((c)<=32||(c)=='\\'||(c)=='('||(c)==')'||(c)>127) {                \
-    pdf_room(pdf,4);                                                    \
-    pdf_quick_out(pdf,'\\');                                                \
-    pdf_quick_out(pdf,'0' + (((c)>>6) & 0x3));                              \
-    pdf_quick_out(pdf,'0' + (((c)>>3) & 0x7));                              \
-    pdf_quick_out(pdf,'0' + ( (c)     & 0x7));                              \
-  } else {                                                              \
-    pdf_out(pdf,(c));                                                       \
+#define pdf_print_escaped(c)                                  \
+  if ((c)<=32||(c)=='\\'||(c)=='('||(c)==')'||(c)>127) {      \
+    pdf_room(pdf,4);                                          \
+    pdf_quick_out(pdf,'\\');                                  \
+    pdf_quick_out(pdf,'0' + (((c)>>6) & 0x3));                \
+    pdf_quick_out(pdf,'0' + (((c)>>3) & 0x7));                \
+    pdf_quick_out(pdf,'0' + ( (c)     & 0x7));                \
+  } else {                                                    \
+    pdf_out(pdf,(c));                                         \
   }
 
 void pdf_print_char(PDF pdf, int c)
@@ -378,7 +379,9 @@ void pdf_print_wide_char(PDF pdf, int c)
     pdf_quick_out(pdf, hex[1]);
     pdf_quick_out(pdf, hex[2]);
     pdf_quick_out(pdf, hex[3]);
-} void pdf_puts(PDF pdf, const char *s)
+}
+
+void pdf_puts(PDF pdf, const char *s)
 {
 
     size_t l = strlen(s);
@@ -405,7 +408,6 @@ void pdf_printf(PDF pdf, const char *fmt, ...)
     va_end(args);
 }
 
-
 /* print out a string to PDF buffer */
 
 void pdf_print(PDF pdf, str_number s)
@@ -414,9 +416,10 @@ void pdf_print(PDF pdf, str_number s)
         assert(s < 256);
         pdf_out(pdf, s);
     } else {
-        register pool_pointer j = str_start_macro(s);
-        while (j < str_start_macro(s + 1)) {
-            pdf_out(pdf, str_pool[j++]);
+        unsigned char *k, *j;
+        j = str_string(s) + str_length(s);
+        for (k = str_string(s); k < j; k++) {
+            pdf_out(pdf, *k);
         }
     }
 }
@@ -455,8 +458,8 @@ void pdf_print_int(PDF pdf, longinteger n)
     }
 }
 
-
 /* print $m/10^d$ as real */
+
 void pdf_print_real(PDF pdf, integer m, integer d)
 {
     if (m < 0) {
@@ -511,8 +514,8 @@ void pdf_print_str(PDF pdf, char *s)
     pdf_puts(pdf, orig);        /* it was a hex string after all  */
 }
 
-
 /* begin a stream */
+
 void pdf_begin_stream(PDF pdf)
 {
     assert(pdf->os_mode == false);
@@ -535,6 +538,7 @@ void pdf_begin_stream(PDF pdf)
 }
 
 /* end a stream */
+
 void pdf_end_stream(PDF pdf)
 {
     if (pdf->zip_write_state == zip_writing)
@@ -557,7 +561,6 @@ void pdf_remove_last_space(PDF pdf)
         pdf->ptr--;
 }
 
-
 /*
 To print |scaled| value to PDF output we need some subroutines to ensure
 accurary.
@@ -667,12 +670,12 @@ void reset_resource_lists(pdf_resource_struct * p)
     p->image_procset = 0;
 }
 
-void append_object_list(PDF pdf, pdf_obj_type t, integer f)
+void append_object_list(PDF pdf, pdf_obj_type t, integer objnum)
 {
     pdf_object_list *p;
     pdf_object_list *item = xmalloc(sizeof(pdf_object_list));
     item->link = NULL;
-    item->info = f;
+    item->info = objnum;
     switch (t) {
     /* *INDENT-OFF* */
     case obj_type_obj:    set_p_or_return(pdf->resources->obj_list);    break;
@@ -689,6 +692,7 @@ void append_object_list(PDF pdf, pdf_obj_type t, integer f)
     while (p->link != NULL)
         p = p->link;
     p->link = item;
+    set_obj_scheduled(pdf, objnum);
     return;
 }
 
@@ -942,19 +946,19 @@ static void init_pdf_outputparameters(PDF pdf)
 
 /* Checks that we have a name for the generated PDF file and that it's open. */
 
-static void ensure_output_file_open(PDF pdf, char *s)
+static void ensure_output_file_open(PDF pdf, char *ext)
 {
+    char *fn;
     if (pdf->file_name != NULL)
         return;
     if (job_name == 0)
         open_log_file();
-    pack_job_name(s);
+    fn = pack_job_name(ext);
     if (pdf->draftmode == 0 || pdf->o_mode == OMODE_DVI) {
-        while (!lua_b_open_out(pdf->file))
-            prompt_file_name("file name for output", s);
+        while (!lua_b_open_out(&pdf->file, fn))
+            fn = prompt_file_name("file name for output", ext);
     }
-    pdf->file = name_file_pointer;      /* hm ? */
-    pdf->file_name = xstrdup(makecstring(make_name_string()));
+    pdf->file_name = fn;
 }
 
 static void ensure_pdf_header_written(PDF pdf)
@@ -1135,7 +1139,6 @@ void pdf_new_obj(PDF pdf, integer t, integer i, integer pdf_os)
     pdf_begin_obj(pdf, pdf->obj_ptr, pdf_os);
 }
 
-
 /* end a PDF object */
 void pdf_end_obj(PDF pdf)
 {
@@ -1147,11 +1150,10 @@ void pdf_end_obj(PDF pdf)
     }
 }
 
-
 void write_stream_length(PDF pdf, integer length, longinteger offset)
 {
     if (jobname_cstr == NULL)
-        jobname_cstr = xstrdup(makecstring(job_name));
+        jobname_cstr = makecstring(job_name);
     if (pdf->draftmode == 0) {
         xfseeko(pdf->file, (off_t) offset, SEEK_SET, jobname_cstr);
         fprintf(pdf->file, "%li", (long int) length);
@@ -1159,7 +1161,6 @@ void write_stream_length(PDF pdf, integer length, longinteger offset)
     }
 }
 
-
 /* Converts any string given in in in an allowed PDF string which can be
  * handled by printf et.al.: \ is escaped to \\, parenthesis are escaped and
  * control characters are octal encoded.
@@ -1638,8 +1639,8 @@ void set_job_id(PDF pdf, int year, int month, int day, int time)
     if (pdf->job_id_string != NULL)
         return;
 
-    name_string = xstrdup(makecstring(job_name));
-    format_string = xstrdup(makecstring(format_ident));
+    name_string = makecstring(job_name);
+    format_string = makecstring(format_ident);
     make_pdftex_banner();
     slen = SMALL_BUF_SIZE +
         strlen(name_string) + strlen(format_string) + strlen(pdftex_banner);
@@ -1678,6 +1679,9 @@ void set_job_id(PDF pdf, int year, int month, int day, int time)
 
 #define mag int_par(mag_code)
 
+#define pdf_xform_attr equiv(pdf_xform_attr_loc)
+#define pdf_xform_resources equiv(pdf_xform_resources_loc)
+
 void pdf_begin_page(PDF pdf, boolean shipping_page)
 {
     scaled form_margin = one_bp;
@@ -1692,7 +1696,7 @@ void pdf_begin_page(PDF pdf, boolean shipping_page)
     if (shipping_page) {
         pdf->last_page = get_obj(pdf, obj_type_page, total_pages + 1, 0);
         set_obj_aux(pdf, pdf->last_page, 1);    /* mark that this page has been created */
-        pdf_new_dict(pdf, obj_type_others, 0, 0);
+        pdf_new_dict(pdf, obj_type_pagestream, 0, 0);
         pdf->last_stream = pdf->obj_ptr;
         pdf->last_thread = null;
     } else {
@@ -1700,14 +1704,16 @@ void pdf_begin_page(PDF pdf, boolean shipping_page)
         pdf->last_stream = pdf_cur_form;
 
         /* Write out Form stream header */
-        pdf_printf(pdf, "/Type /XObject\n");
-        pdf_printf(pdf, "/Subtype /Form\n");
+        pdf_puts(pdf, "/Type /XObject\n");
+        pdf_puts(pdf, "/Subtype /Form\n");
+        if (pdf_xform_attr != null)
+            pdf_print_toks_ln(pdf, pdf_xform_attr);
         if (obj_xform_attr(pdf, pdf_cur_form) != null) {
             pdf_print_toks_ln(pdf, obj_xform_attr(pdf, pdf_cur_form));
             delete_token_ref(obj_xform_attr(pdf, pdf_cur_form));
             set_obj_xform_attr(pdf, pdf_cur_form, null);
         }
-        pdf_printf(pdf, "/BBox [");
+        pdf_puts(pdf, "/BBox [");
         pdf_print_bp(pdf, -form_margin);
         pdf_out(pdf, ' ');
         pdf_print_bp(pdf, -form_margin);
@@ -1715,9 +1721,9 @@ void pdf_begin_page(PDF pdf, boolean shipping_page)
         pdf_print_bp(pdf, cur_page_size.h + form_margin);
         pdf_out(pdf, ' ');
         pdf_print_bp(pdf, cur_page_size.v + form_margin);
-        pdf_printf(pdf, "]\n");
-        pdf_printf(pdf, "/FormType 1\n");
-        pdf_printf(pdf, "/Matrix [1 0 0 1 0 0]\n");
+        pdf_puts(pdf, "]\n");
+        pdf_puts(pdf, "/FormType 1\n");
+        pdf_puts(pdf, "/Matrix [1 0 0 1 0 0]\n");
         pdf_indirect_ln(pdf, "Resources", pdf->resources->last_resources);
     }
     /* Start stream of page/form contents */
@@ -1726,9 +1732,9 @@ void pdf_begin_page(PDF pdf, boolean shipping_page)
         /* Adjust transformation matrix for the magnification ratio */
         if (mag != 1000) {
             pdf_print_real(pdf, mag, 3);
-            pdf_printf(pdf, " 0 0 ");
+            pdf_puts(pdf, " 0 0 ");
             pdf_print_real(pdf, mag, 3);
-            pdf_printf(pdf, " 0 0 cm\n");
+            pdf_puts(pdf, " 0 0 cm\n");
         }
     }
     pdf_shipout_begin(shipping_page);
@@ -1759,14 +1765,14 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
 
         pdf_begin_dict(pdf, pdf->last_page, 1);
         pdf->last_pages = pdf_do_page_divert(pdf, pdf->last_page, 0);
-        pdf_printf(pdf, "/Type /Page\n");
+        pdf_puts(pdf, "/Type /Page\n");
         pdf_indirect_ln(pdf, "Contents", pdf->last_stream);
         pdf_indirect_ln(pdf, "Resources", res_p->last_resources);
-        pdf_printf(pdf, "/MediaBox [0 0 ");
+        pdf_puts(pdf, "/MediaBox [0 0 ");
         pdf_print_mag_bp(pdf, cur_page_size.h);
         pdf_out(pdf, ' ');
         pdf_print_mag_bp(pdf, cur_page_size.v);
-        pdf_printf(pdf, "]\n");
+        pdf_puts(pdf, "]\n");
         if (pdf_page_attr != null)
             pdf_print_toks_ln(pdf, pdf_page_attr);
         pdf_indirect_ln(pdf, "Parent", pdf->last_pages);
@@ -1777,20 +1783,23 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
         /* Generate array of annotations or beads in page */
         if ((res_p->annot_list != NULL)
             || (res_p->link_list != NULL)) {
-            pdf_printf(pdf, "/Annots [ ");
+            pdf_puts(pdf, "/Annots [ ");
             ol = res_p->annot_list;
             while (ol != NULL) {
-                pdf_print_int(pdf, ol->info);
-                pdf_printf(pdf, " 0 R ");
+                if (ol->info > 0)
+                    pdf_print_int(pdf, ol->info);
+                else
+                    pdf_print_int(pdf, (-ol->info));
+                pdf_puts(pdf, " 0 R ");
                 ol = ol->link;
             }
             ol = res_p->link_list;
             while (ol != NULL) {
                 pdf_print_int(pdf, ol->info);
-                pdf_printf(pdf, " 0 R ");
+                pdf_puts(pdf, " 0 R ");
                 ol = ol->link;
             }
-            pdf_printf(pdf, "]\n");
+            pdf_puts(pdf, "]\n");
         }
         print_beads_list(pdf);
         pdf_end_dict(pdf);
@@ -1841,12 +1850,14 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
         /* Write out PDF annotations */
         if ((ol = res_p->annot_list) != NULL) {
             while (ol != NULL) {
-                j = obj_annot_ptr(pdf, ol->info);       /* |j| points to |pdf_annot_node| */
-                pdf_begin_dict(pdf, ol->info, 1);
-                pdf_printf(pdf, "/Type /Annot\n");
-                pdf_print_toks_ln(pdf, pdf_annot_data(j));
-                pdf_rectangle(pdf, j);
-                pdf_end_dict(pdf);
+                if (ol->info > 0) {
+                    j = obj_annot_ptr(pdf, ol->info);   /* |j| points to |pdf_annot_node| */
+                    pdf_begin_dict(pdf, ol->info, 1);
+                    pdf_puts(pdf, "/Type /Annot\n");
+                    pdf_print_toks_ln(pdf, pdf_annot_data(j));
+                    pdf_rectangle(pdf, j);
+                    pdf_end_dict(pdf);
+                }
                 ol = ol->link;
             }
         }
@@ -1856,14 +1867,14 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
             while (ol != NULL) {
                 j = obj_annot_ptr(pdf, ol->info);
                 pdf_begin_dict(pdf, ol->info, 1);
-                pdf_printf(pdf, "/Type /Annot\n");
+                pdf_puts(pdf, "/Type /Annot\n");
                 if (pdf_action_type(pdf_link_action(j)) != pdf_action_user)
-                    pdf_printf(pdf, "/Subtype /Link\n");
+                    pdf_puts(pdf, "/Subtype /Link\n");
                 if (pdf_link_attr(j) != null)
                     pdf_print_toks_ln(pdf, pdf_link_attr(j));
                 pdf_rectangle(pdf, j);
                 if (pdf_action_type(pdf_link_action(j)) != pdf_action_user)
-                    pdf_printf(pdf, "/A ");
+                    pdf_puts(pdf, "/A ");
                 write_action(pdf, pdf_link_action(j));
                 pdf_end_dict(pdf);
                 ol = ol->link;
@@ -1893,6 +1904,8 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
         if (pdf_page_resources != null)
             pdf_print_toks_ln(pdf, pdf_page_resources);
     } else {
+        if (pdf_xform_resources != null)
+            pdf_print_toks_ln(pdf, pdf_xform_resources);
         if (obj_xform_resources(pdf, pdf_cur_form) != null) {
             pdf_print_toks_ln(pdf, obj_xform_resources(pdf, pdf_cur_form));
             delete_token_ref(obj_xform_resources(pdf, pdf_cur_form));
@@ -1902,25 +1915,25 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
 
     /* Generate font resources */
     if ((ol = res_p->font_list) != NULL) {
-        pdf_printf(pdf, "/Font << ");
+        pdf_puts(pdf, "/Font << ");
         while (ol != NULL) {
-            pdf_printf(pdf, "/F");
+            pdf_puts(pdf, "/F");
             set_ff(ol->info);
             pdf_print_int(pdf, ff);
             pdf_print_resname_prefix(pdf);
             pdf_out(pdf, ' ');
             pdf_print_int(pdf, pdf_font_num(ff));
-            pdf_printf(pdf, " 0 R ");
+            pdf_puts(pdf, " 0 R ");
             ol = ol->link;
         }
-        pdf_printf(pdf, ">>\n");
+        pdf_puts(pdf, ">>\n");
         res_p->text_procset = true;
     }
 
     /* Generate XObject resources */
     if ((res_p->xform_list != NULL)
         || (res_p->ximage_list != NULL)) {
-        pdf_printf(pdf, "/XObject << ");
+        pdf_puts(pdf, "/XObject << ");
         if ((ol = res_p->xform_list) != NULL) {
             while (ol != NULL) {
                 pdf_printf(pdf, "/Fm");
@@ -1928,36 +1941,36 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
                 pdf_print_resname_prefix(pdf);
                 pdf_out(pdf, ' ');
                 pdf_print_int(pdf, ol->info);
-                pdf_printf(pdf, " 0 R ");
+                pdf_puts(pdf, " 0 R ");
                 ol = ol->link;
             }
         }
         if ((ol = res_p->ximage_list) != NULL) {
             while (ol != null) {
-                pdf_printf(pdf, "/Im");
-                pdf_print_int(pdf, image_index(obj_data_ptr(pdf, ol->info)));
+                pdf_puts(pdf, "/Im");
+                pdf_print_int(pdf, obj_data_ptr(pdf, ol->info));
                 pdf_print_resname_prefix(pdf);
                 pdf_out(pdf, ' ');
                 pdf_print_int(pdf, ol->info);
-                pdf_printf(pdf, " 0 R ");
+                pdf_puts(pdf, " 0 R ");
                 update_image_procset(obj_data_ptr(pdf, ol->info));
                 ol = ol->link;
             }
         }
-        pdf_printf(pdf, ">>\n");
+        pdf_puts(pdf, ">>\n");
     }
 
     /* Generate ProcSet */
-    pdf_printf(pdf, "/ProcSet [ /PDF");
+    pdf_puts(pdf, "/ProcSet [ /PDF");
     if (res_p->text_procset)
-        pdf_printf(pdf, " /Text");
+        pdf_puts(pdf, " /Text");
     if (check_image_b(res_p->image_procset))
-        pdf_printf(pdf, " /ImageB");
+        pdf_puts(pdf, " /ImageB");
     if (check_image_c(res_p->image_procset))
-        pdf_printf(pdf, " /ImageC");
+        pdf_puts(pdf, " /ImageC");
     if (check_image_i(res_p->image_procset))
-        pdf_printf(pdf, " /ImageI");
-    pdf_printf(pdf, " ]\n");
+        pdf_puts(pdf, " /ImageI");
+    pdf_puts(pdf, " ]\n");
 
     pdf_end_dict(pdf);
 
@@ -1971,42 +1984,56 @@ void pdf_end_page(PDF pdf, boolean shipping_page)
 
 /* Finishing the PDF output file. */
 
-#define pdf_pages_attr equiv(pdf_pages_attr_loc)
 #define pdf_gen_tounicode int_par(pdf_gen_tounicode_code)
 
 /*
 Destinations that have been referenced but don't exists have
 |obj_dest_ptr=null|. Leaving them undefined might cause troubles for
-PDF browsers, so we need to fix them.
+PDF browsers, so we need to fix them; they point to the last page.
 */
 
-void pdf_fix_dest(PDF pdf, integer k)
+static void check_nonexisting_destinations(PDF pdf)
 {
-    if (obj_dest_ptr(pdf, k) != null)
-        return;
-    pdf_warning("dest", NULL, false, false);
-    if (obj_info(pdf, k) < 0) {
-        tprint("name{");
-        print(-obj_info(pdf, k));
-        tprint("}");
-    } else {
-        tprint("num");
-        print_int(obj_info(pdf, k));
+    integer k;
+    for (k = pdf->head_tab[obj_type_dest]; k != 0; k = obj_link(pdf, k)) {
+        if (obj_dest_ptr(pdf, k) == null) {
+            pdf_warning("dest", NULL, false, false);
+            if (obj_info(pdf, k) < 0) {
+                tprint("name{");
+                print(-obj_info(pdf, k));
+                tprint("}");
+            } else {
+                tprint("num");
+                print_int(obj_info(pdf, k));
+            }
+            tprint
+                (" has been referenced but does not exist, replaced by a fixed one");
+            print_ln();
+            print_ln();
+            pdf_begin_obj(pdf, k, 1);
+            pdf_out(pdf, '[');
+            pdf_print_int(pdf, pdf->last_page);
+            pdf_puts(pdf, " 0 R /Fit]\n");
+            pdf_end_obj(pdf);
+        }
     }
-    tprint(" has been referenced but does not exist, replaced by a fixed one");
-    print_ln();
-    print_ln();
-    pdf_begin_obj(pdf, k, 1);
-    pdf_out(pdf, '[');
-    pdf_print_int(pdf, pdf->head_tab[obj_type_page]);
-    pdf_printf(pdf, " 0 R /Fit]\n");
-    pdf_end_obj(pdf);
 }
 
-void print_pdf_pages_attr(PDF pdf)
+static void check_nonexisting_pages(PDF pdf)
 {
-    if (pdf_pages_attr != null)
-        pdf_print_toks_ln(pdf, pdf_pages_attr);
+    struct avl_traverser t;
+    oentry *p;
+    struct avl_table *page_tree = pdf->obj_tree[obj_type_page];
+    avl_t_init(&t, page_tree);
+    /* search from the end backward until the last real page is found */
+    for (p = avl_t_last(&t, page_tree);
+         p != NULL && obj_aux(pdf, p->objptr) == 0; p = avl_t_prev(&t)) {
+        pdf_warning("dest", "Page ", false, false);
+        print_int(obj_info(pdf, p->objptr));
+        tprint(" has been referenced but does not exist!");
+        print_ln();
+        print_ln();
+    }
 }
 
 /*
@@ -2045,13 +2072,13 @@ void pdf_print_info(PDF pdf, integer luatex_version, str_number luatex_revision)
     }
     if (!producer_given) {
         /* Print the Producer key */
-        pdf_printf(pdf, "/Producer (LuaTeX-");
+        pdf_puts(pdf, "/Producer (LuaTeX-");
         pdf_print_int(pdf, luatex_version / 100);
         pdf_out(pdf, '.');
         pdf_print_int(pdf, luatex_version % 100);
         pdf_out(pdf, '.');
         pdf_print(pdf, luatex_revision);
-        pdf_printf(pdf, ")\n");
+        pdf_puts(pdf, ")\n");
 
     }
     if (pdf_info_toks != null) {
@@ -2072,7 +2099,7 @@ void pdf_print_info(PDF pdf, integer luatex_version, str_number luatex_revision)
         print_mod_date(pdf);
     }
     if (!trapped_given) {
-        pdf_printf(pdf, "/Trapped /False\n");
+        pdf_puts(pdf, "/Trapped /False\n");
     }
     pdf_str_entry_ln(pdf, "PTEX.Fullbanner", pdftex_banner);
     pdf_end_dict(pdf);
@@ -2092,6 +2119,8 @@ void build_free_object_list(PDF pdf)
     set_obj_link(pdf, l, 0);
 }
 
+/**********************************************************************/
+
 /*
 Now the finish of PDF output file. At this moment all Page objects
 are already written completely to PDF output file.
@@ -2118,24 +2147,9 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
         if (pdf->draftmode == 0) {
             pdf_flush(pdf);     /* to make sure that the output file name has been already created */
             flush_jbig2_page0_objects(pdf);     /* flush page 0 objects from JBIG2 images, if any */
-            /* Check for non-existing pages */
-            k = pdf->head_tab[obj_type_page];
-            while (obj_aux(pdf, k) == 0) {
-                pdf_warning("dest", "Page ", false, false);
-                print_int(obj_info(pdf, k));
-                tprint(" has been referenced but does not exist!");
-                print_ln();
-                print_ln();
-                k = obj_link(pdf, k);
-            }
-            pdf->head_tab[obj_type_page] = k;
 
-            /* Check for non-existing destinations */
-            k = pdf->head_tab[obj_type_dest];
-            while (k != 0) {
-                pdf_fix_dest(pdf, k);
-                k = obj_link(pdf, k);
-            }
+            check_nonexisting_pages(pdf);
+            check_nonexisting_destinations(pdf);
 
             /* Output fonts definition */
             for (k = 1; k <= max_font_id(); k++) {
@@ -2199,11 +2213,11 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                 k = pdf->head_tab[obj_type_thread];
                 while (k != 0) {
                     pdf_print_int(pdf, k);
-                    pdf_printf(pdf, " 0 R ");
+                    pdf_puts(pdf, " 0 R ");
                     k = obj_link(pdf, k);
                 }
                 pdf_remove_last_space(pdf);
-                pdf_printf(pdf, "]\n");
+                pdf_puts(pdf, "]\n");
                 pdf_end_obj(pdf);
                 k = pdf->head_tab[obj_type_thread];
                 while (k != 0) {
@@ -2217,7 +2231,7 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
             /* Output the catalog object */
             pdf_new_dict(pdf, obj_type_others, 0, 1);
             root = pdf->obj_ptr;
-            pdf_printf(pdf, "/Type /Catalog\n");
+            pdf_puts(pdf, "/Type /Catalog\n");
             pdf_indirect_ln(pdf, "Pages", pdf->last_pages);
             if (threads != 0)
                 pdf_indirect_ln(pdf, "Threads", threads);
@@ -2253,14 +2267,14 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                     xref_offset_width = 2;
                 /* Build a linked list of free objects */
                 build_free_object_list(pdf);
-                pdf_printf(pdf, "/Type /XRef\n");
-                pdf_printf(pdf, "/Index [0 ");
+                pdf_puts(pdf, "/Type /XRef\n");
+                pdf_puts(pdf, "/Index [0 ");
                 pdf_print_int(pdf, pdf->obj_ptr + 1);
-                pdf_printf(pdf, "]\n");
+                pdf_puts(pdf, "]\n");
                 pdf_int_entry_ln(pdf, "Size", pdf->obj_ptr + 1);
-                pdf_printf(pdf, "/W [1 ");
+                pdf_puts(pdf, "/W [1 ");
                 pdf_print_int(pdf, xref_offset_width);
-                pdf_printf(pdf, " 1]\n");
+                pdf_puts(pdf, " 1]\n");
                 pdf_indirect_ln(pdf, "Root", root);
                 pdf_indirect_ln(pdf, "Info", pdf->obj_ptr - 1);
                 if (pdf_trailer_toks != null) {
@@ -2298,18 +2312,18 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                 build_free_object_list(pdf);
 
                 pdf_save_offset(pdf);
-                pdf_printf(pdf, "xref\n");
-                pdf_printf(pdf, "0 ");
+                pdf_puts(pdf, "xref\n");
+                pdf_puts(pdf, "0 ");
                 pdf_print_int_ln(pdf, pdf->obj_ptr + 1);
                 pdf_print_fw_int(pdf, obj_link(pdf, 0), 10);
-                pdf_printf(pdf, " 65535 f \n");
+                pdf_puts(pdf, " 65535 f \n");
                 for (k = 1; k <= pdf->obj_ptr; k++) {
                     if (!is_obj_written(pdf, k)) {
                         pdf_print_fw_int(pdf, obj_link(pdf, k), 10);
-                        pdf_printf(pdf, " 00000 f \n");
+                        pdf_puts(pdf, " 00000 f \n");
                     } else {
                         pdf_print_fw_int(pdf, obj_offset(pdf, k), 10);
-                        pdf_printf(pdf, " 00000 n \n");
+                        pdf_puts(pdf, " 00000 n \n");
                     }
                 }
 
@@ -2317,8 +2331,8 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
 
             /* Output the trailer */
             if (!pdf->os_enable) {
-                pdf_printf(pdf, "trailer\n");
-                pdf_printf(pdf, "<< ");
+                pdf_puts(pdf, "trailer\n");
+                pdf_puts(pdf, "<< ");
                 pdf_int_entry_ln(pdf, "Size", pdf->sys_obj_ptr + 1);
                 pdf_indirect_ln(pdf, "Root", root);
                 pdf_indirect_ln(pdf, "Info", pdf->sys_obj_ptr);
@@ -2328,14 +2342,14 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                     pdf_trailer_toks = null;
                 }
                 print_ID(pdf, pdf->file_name);
-                pdf_printf(pdf, " >>\n");
+                pdf_puts(pdf, " >>\n");
             }
-            pdf_printf(pdf, "startxref\n");
+            pdf_puts(pdf, "startxref\n");
             if (pdf->os_enable)
                 pdf_print_int_ln(pdf, obj_offset(pdf, pdf->sys_obj_ptr));
             else
                 pdf_print_int_ln(pdf, pdf_saved_offset(pdf));
-            pdf_printf(pdf, "%%%%EOF\n");
+            pdf_puts(pdf, "%%EOF\n");
 
             pdf_flush(pdf);
             if (callback_id == 0) {
@@ -2355,7 +2369,7 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
         }
         libpdffinish(pdf);
         if (pdf->draftmode == 0)
-            b_close(pdf->file);
+            close_file(pdf->file);
         else
             pdf_warning(NULL,
                         "\\pdfdraftmode enabled, not changing output pdf",
