@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: pdfgen.h 3030 2009-08-19 20:08:12Z hhenkel $ */
+/* $Id: pdfgen.h 3187 2009-11-22 22:29:05Z oneiros $ */
 
 #ifndef PDFGEN_H
 #  define PDFGEN_H
@@ -41,6 +41,9 @@ written bytes.
 #  define sup_pdf_op_buf_size 16384     /* size of the PDF output buffer */
 #  define inf_pdf_os_buf_size 1 /* initial value of |pdf_os_buf_size| */
 #  define sup_pdf_os_buf_size 5000000   /* arbitrary upper hard limit of |pdf_os_buf_size| */
+#  define max_single_pdf_print 8192     /* Max size that can be get from pdf_room() at once. 
+                                           the value is on the conservative side, but should be
+                                           large enough to cover most uses */
 #  define pdf_os_max_objs 100   /* maximum number of objects in object stream */
 
 #  define inf_obj_tab_size 1000 /* min size of the cross-reference table for PDF output */
@@ -90,6 +93,12 @@ extern void fix_pdf_minorversion(PDF);
 /* do the same as |pdf_quick_out| and flush the PDF buffer if necessary */
 #  define pdf_out(pdf,A) do { pdf_room(pdf,1); pdf_quick_out(pdf,A); } while (0)
 
+#  define pdf_out_block(pdf,A,n) do {                       \
+        pdf_room(pdf,(n));                                  \
+        (void)memcpy((pdf->buf+pdf->ptr),(A),(n));          \
+        pdf->ptr+=(n);                                      \
+    } while (0)
+
 /*
 Basic printing procedures for PDF output are very similiar to \TeX\ basic
 printing ones but the output is going to PDF buffer. Subroutines with
@@ -215,7 +224,6 @@ extern halfword pdf_catalog_toks;       /* additional keys of Catalog dictionary
 extern halfword pdf_catalog_openaction;
 extern halfword pdf_names_toks; /* additional keys of Names dictionary */
 extern halfword pdf_trailer_toks;       /* additional keys of Trailer dictionary */
-extern void print_pdf_pages_attr(PDF pdf);
 extern void scan_pdfcatalog(PDF pdf);
 extern void finish_pdf_file(PDF pdf, integer luatex_version,
                             str_number luatex_revision);
