@@ -18,14 +18,12 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char __svn_version[] =
-    "$Id: pdfimage.c 3003 2009-08-16 19:55:55Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfimage.c $";
+    "$Id: pdfimage.c 3160 2009-11-16 16:29:11Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfimage.c $";
 
 #include "ptexlib.h"
 
-#define pdf_pagebox int_par(pdf_pagebox_code)
-
-void place_img(PDF pdf, image * img)
+void place_img(PDF pdf, image_dict * idict, scaled_whd dim, int transform)
 {
     float a[6];                 /* transformation matrix */
     float xoff, yoff, tmp;
@@ -33,18 +31,11 @@ void place_img(PDF pdf, image * img)
     scaledpos pos = pdf->posstruct->pos;
     int r;                      /* number of digits after the decimal point */
     int k;
-    scaled wd, ht, dp;
     scaledpos tmppos;
     pdffloat cm[6];
-    image_dict *idict;
     integer groupref;           /* added from web for 1.40.8 */
-    assert(p != NULL);
-    assert(img != 0);
-    idict = img_dict(img);
     assert(idict != 0);
-    wd = img_width(img);
-    ht = img_height(img);
-    dp = img_depth(img);
+    assert(p != NULL);
     a[0] = a[3] = 1.0e6;
     a[1] = a[2] = 0;
     if (img_type(idict) == IMG_TYPE_PDF
@@ -67,11 +58,11 @@ void place_img(PDF pdf, image * img)
         xoff = yoff = 0;
         r = 4;
     }
-    if ((img_transform(img) & 7) > 3) { // mirror cases
+    if ((transform & 7) > 3) {  // mirror cases
         a[0] *= -1;
         xoff *= -1;
     }
-    switch ((img_transform(img) + img_rotation(idict)) & 3) {
+    switch ((transform + img_rotation(idict)) & 3) {
     case 0:                    /* no transform */
         break;
     case 1:                    /* rot. 90 deg. (counterclockwise) */
@@ -98,29 +89,29 @@ void place_img(PDF pdf, image * img)
         break;
     default:;
     }
-    xoff *= wd;
-    yoff *= ht + dp;
-    a[0] *= wd;
-    a[1] *= ht + dp;
-    a[2] *= wd;
-    a[3] *= ht + dp;
+    xoff *= dim.wd;
+    yoff *= dim.ht + dim.dp;
+    a[0] *= dim.wd;
+    a[1] *= dim.ht + dim.dp;
+    a[2] *= dim.wd;
+    a[3] *= dim.ht + dim.dp;
     a[4] = pos.h - xoff;
     a[5] = pos.v - yoff;
-    k = img_transform(img) + img_rotation(idict);
-    if ((img_transform(img) & 7) > 3)
+    k = transform + img_rotation(idict);
+    if ((transform & 7) > 3)
         k++;
     switch (k & 3) {
     case 0:                    /* no transform */
         break;
     case 1:                    /* rot. 90 deg. (counterclockwise) */
-        a[4] += wd;
+        a[4] += dim.wd;
         break;
     case 2:                    /* rot. 180 deg. */
-        a[4] += wd;
-        a[5] += ht + dp;
+        a[4] += dim.wd;
+        a[5] += dim.ht + dim.dp;
         break;
     case 3:                    /* rot. 270 deg. */
-        a[5] += ht + dp;
+        a[5] += dim.ht + dim.dp;
         break;
     default:;
     }
@@ -131,10 +122,10 @@ void place_img(PDF pdf, image * img)
     setpdffloat(cm[3], round(a[3]), r);
     tmppos.h = round(a[4]);
     tmppos.v = round(a[5]);
+    pdf_goto_pagemode(pdf);
     (void) calc_pdfpos(p, tmppos);
     cm[4] = p->cm[4];
     cm[5] = p->cm[5];
-    pdf_goto_pagemode(pdf);
     if (pdf->img_page_group_val == 0)
         pdf->img_page_group_val = img_group_ref(idict); /* added from web for 1.40.8 */
     pdf_printf(pdf, "q\n");
@@ -143,89 +134,21 @@ void place_img(PDF pdf, image * img)
     pdf_print_int(pdf, img_index(idict));
     pdf_print_resname_prefix(pdf);
     pdf_printf(pdf, " Do\nQ\n");
+    if (lookup_object_list(pdf, obj_type_ximage, img_objnum(idict)) == NULL)
+        append_object_list(pdf, obj_type_ximage, img_objnum(idict));
     if (img_state(idict) < DICT_OUTIMG)
         img_state(idict) = DICT_OUTIMG;
 }
 
+/* for normal output, see pdflistout.c */
+
 void pdf_place_image(PDF pdf, halfword p)
 {
-    integer idx = pdf_ximage_idx(p);
-    pdf_goto_pagemode(pdf);
-    place_img(pdf, img_array[idx]);
-    if (lookup_object_list(pdf, obj_type_ximage, image_objnum(idx)) == NULL)
-        append_object_list(pdf, obj_type_ximage, image_objnum(idx));
-}
-
-/* scans PDF pagebox specification */
-static integer scan_pdf_box_spec(void)
-{
-    if (scan_keyword("mediabox"))
-        return pdf_box_spec_media;
-    else if (scan_keyword("cropbox"))
-        return pdf_box_spec_crop;
-    else if (scan_keyword("bleedbox"))
-        return pdf_box_spec_bleed;
-    else if (scan_keyword("trimbox"))
-        return pdf_box_spec_trim;
-    else if (scan_keyword("artbox"))
-        return pdf_box_spec_art;
-    return 0;
-}
-
-void scan_image(PDF pdf)
-{
-    scaled_whd alt_rule;
-    integer k, ref;
-    integer page, pagebox, colorspace;
-    char *named = NULL, *attr = NULL, *s = NULL;
-    incr(pdf->ximage_count);
-    pdf_create_obj(pdf, obj_type_ximage, pdf->ximage_count);
-    k = pdf->obj_ptr;
-    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
-    attr = 0;
-    named = 0;
-    page = 1;
-    colorspace = 0;
-    if (scan_keyword("attr")) {
-        scan_pdf_ext_toks();
-        attr = tokenlist_to_cstring(def_ref, true, NULL);
-        delete_token_ref(def_ref);
-    }
-    if (scan_keyword("named")) {
-        scan_pdf_ext_toks();
-        named = tokenlist_to_cstring(def_ref, true, NULL);
-        delete_token_ref(def_ref);
-        page = 0;
-    } else if (scan_keyword("page")) {
-        scan_int();
-        page = cur_val;
-    }
-    if (scan_keyword("colorspace")) {
-        scan_int();
-        colorspace = cur_val;
-    }
-    pagebox = scan_pdf_box_spec();
-    if (pagebox == 0)
-        pagebox = pdf_pagebox;
-    scan_pdf_ext_toks();
-    s = tokenlist_to_cstring(def_ref, true, NULL);
-    assert(s != NULL);
-    delete_token_ref(def_ref);
-    if (pagebox == 0)           /* no pagebox specification given */
-        pagebox = pdf_box_spec_crop;
-    ref =
-        read_image(pdf,
-                   k, pdf->ximage_count, s, page, named, attr, colorspace,
-                   pagebox, pdf_minor_version, pdf_inclusion_errorlevel);
-    xfree(s);
-    set_obj_data_ptr(pdf, k, ref);
-    if (named != NULL)
-        xfree(named);
-    set_image_dimensions(ref, alt_rule);
-    if (attr != NULL)
-        xfree(attr);
-    scale_image(ref);
-    pdf_last_ximage = k;
-    pdf_last_ximage_pages = image_pages(ref);
-    pdf_last_ximage_colordepth = image_colordepth(ref);
+    scaled_whd dim;
+    image_dict *idict = idict_array[pdf_ximage_index(p)];
+    assert(idict != NULL);
+    dim.wd = width(p);
+    dim.ht = height(p);
+    dim.dp = depth(p);
+    place_img(pdf, idict, dim, pdf_ximage_transform(p));
 }
