@@ -22,8 +22,8 @@
 
 
 static const char __svn_version[] =
-    "$Id: pdflink.c 2978 2009-08-09 09:03:39Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdflink.c $";
+    "$Id: pdflink.c 3233 2009-12-05 07:19:29Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdflink.c $";
 
 #define pdf_link_margin          dimen_par(pdf_link_margin_code)
 
@@ -68,7 +68,6 @@ void do_link(PDF pdf, halfword p, halfword parent_box, scaledpos cur)
     set_rect_dimens(pdf, p, parent_box, cur, alt_rule, pdf_link_margin);
     obj_annot_ptr(pdf, pdf_link_objnum(p)) = p; /* the reference for the pdf annot object must be set here */
     append_object_list(pdf, obj_type_link, pdf_link_objnum(p));
-    set_obj_scheduled(pdf, pdf_link_objnum(p));
 }
 
 void end_link(PDF pdf, halfword p)
@@ -96,23 +95,17 @@ void end_link(PDF pdf, halfword p)
             pdf_ann_right(q) = geturx() + pdf_link_margin;
             pdf_ann_bottom(q) = cur_page_size.v - getlly() + pdf_link_margin;
         } else {
-            switch (box_direction(pdf->posstruct->dir)) {
-            case dir_TL_:
-            case dir_BL_:
+            switch (pdf->posstruct->dir) {
+            case dir_TLT:
                 pdf_ann_right(q) = pos.h + pdf_link_margin;
                 break;
-            case dir_TR_:
-            case dir_BR_:
+            case dir_TRT:
                 pdf_ann_left(q) = pos.h - pdf_link_margin;
                 break;
-            case dir_LT_:
-            case dir_RT_:
+            case dir_LTL:
+            case dir_RTT:
                 pdf_ann_bottom(q) = pos.v - pdf_link_margin;
                 break;
-            case dir_LB_:
-            case dir_RB_:
-                pdf_ann_top(q) = pos.v + pdf_link_margin;
-                break;
             }
         }
     }
@@ -156,7 +149,7 @@ void scan_startlink(PDF pdf)
     new_annot_whatsit(pdf_start_link_node);
     set_pdf_link_attr(cur_list.tail_field, null);
     if (scan_keyword("attr")) {
-        scan_pdf_ext_toks();
+	scan_pdf_ext_toks();
         set_pdf_link_attr(cur_list.tail_field, def_ref);
     }
     r = scan_action(pdf);
