@@ -20,8 +20,8 @@
 #include "ptexlib.h"
 
 static const char __svn_version[] =
-    "$Id: pdfliteral.c 2972 2009-08-08 07:07:26Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfliteral.c $";
+    "$Id: pdfliteral.c 3217 2009-12-03 16:52:01Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfliteral.c $";
 
 void pdf_special(PDF pdf, halfword p)
 {
@@ -29,7 +29,7 @@ void pdf_special(PDF pdf, halfword p)
     str_number s;
     old_setting = selector;
     selector = new_string;
-    show_token_list(token_link(write_tokens(p)), null, pool_size - pool_ptr);
+    show_token_list(token_link(write_tokens(p)), null, -1);
     selector = old_setting;
     s = make_string();
     pdf_literal(pdf, s, scan_special, true);
@@ -51,8 +51,7 @@ void pdf_out_literal(PDF pdf, halfword p)
     if (pdf_literal_type(p) == normal) {
         old_setting = selector;
         selector = new_string;
-        show_token_list(token_link(pdf_literal_data(p)), null,
-                        pool_size - pool_ptr);
+        show_token_list(token_link(pdf_literal_data(p)), null, -1);
         selector = old_setting;
         s = make_string();
         pdf_literal(pdf, s, pdf_literal_mode(p), false);
@@ -81,17 +80,14 @@ void pdf_out_literal(PDF pdf, halfword p)
 /* test equality of start of strings */
 static boolean str_in_cstr(str_number s, char *r, unsigned i)
 {
-    pool_pointer j;             /* running indices */
-    unsigned char *k;
+    unsigned char *k, *l;
     if ((unsigned) str_length(s) < i + strlen(r))
         return false;
-    j = i + str_start_macro(s);
     k = (unsigned char *) r;
-    while ((j < str_start_macro(s + 1)) && (*k)) {
-        if (str_pool[j] != *k)
+    l = str_string(s) + i;
+    while ((*l) && (*k)) {
+        if (*l++ != *k++)
             return false;
-        j++;
-        k++;
     }
     return true;
 }
@@ -100,7 +96,7 @@ void pdf_literal(PDF pdf, str_number s, integer literal_mode, boolean warn)
 {
     pool_pointer j = 0;         /* current character code position, initialized to make the compiler happy */
     if (s >= STRING_OFFSET) {   /* needed for |out_save| */
-        j = str_start_macro(s);
+        j = 0;
         if (literal_mode == scan_special) {
             if (!(str_in_cstr(s, "PDF:", 0) || str_in_cstr(s, "pdf:", 0))) {
                 if (warn
@@ -138,9 +134,17 @@ void pdf_literal(PDF pdf, str_number s, integer literal_mode, boolean warn)
         break;
     }
     if (s >= STRING_OFFSET) {
-        while (j < str_start_macro(s + 1))
-            pdf_out(pdf, str_pool[j++]);
+        unsigned char *ss = str_string(s);
+        int l = str_length(s) - j;
+        if (l < max_single_pdf_print) {
+            pdf_out_block(pdf, (ss+j), l);
+        } else {
+            
+            while (l--)
+                pdf_out(pdf, *(ss+j++));
+        }
     } else {
+        assert(s < 256);
         pdf_out(pdf, s);
     }
     pdf_print_nl(pdf);
