@@ -25,8 +25,8 @@
 #include "ptexlib.h"
 
 static const char __svn_version[] =
-    "$Id: pdfpage.c 3003 2009-08-16 19:55:55Z hhenkel $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfpage.c $";
+    "$Id: pdfpage.c 3220 2009-12-04 10:45:36Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfpage.c $";
 
 #define lround(a) (long) floor((a) + 0.5)
 
@@ -76,39 +76,23 @@ void init_pdf_pagecalculations(PDF pdf)
 void synch_pos_with_cur(posstructure * pos, posstructure * refpos,
                         scaledpos cur)
 {
-    switch (box_direction(pos->dir)) {
-    case dir_TL_:
+    switch (pos->dir) {
+    case dir_TLT:
         pos->pos.h = refpos->pos.h + cur.h;
         pos->pos.v = refpos->pos.v - cur.v;
         break;
-    case dir_TR_:
+    case dir_TRT:
         pos->pos.h = refpos->pos.h - cur.h;
         pos->pos.v = refpos->pos.v - cur.v;
         break;
-    case dir_BL_:
-        pos->pos.h = refpos->pos.h + cur.h;
-        pos->pos.v = refpos->pos.v + cur.v;
-        break;
-    case dir_BR_:
-        pos->pos.h = refpos->pos.h - cur.h;
-        pos->pos.v = refpos->pos.v + cur.v;
-        break;
-    case dir_LT_:
+    case dir_LTL:
         pos->pos.h = refpos->pos.h + cur.v;
         pos->pos.v = refpos->pos.v - cur.h;
         break;
-    case dir_RT_:
+    case dir_RTT:
         pos->pos.h = refpos->pos.h - cur.v;
         pos->pos.v = refpos->pos.v - cur.h;
         break;
-    case dir_LB_:
-        pos->pos.h = refpos->pos.h + cur.v;
-        pos->pos.v = refpos->pos.v + cur.h;
-        break;
-    case dir_RB_:
-        pos->pos.h = refpos->pos.h - cur.v;
-        pos->pos.v = refpos->pos.v + cur.h;
-        break;
     default:
         assert(0);
     }
@@ -176,14 +160,14 @@ void print_pdffloat(PDF pdf, pdffloat f)
     int e = f.e, i, j;
     long l, m = f.m;
     if (m < 0) {
-        pdf_printf(pdf, "-");
+        pdf_puts(pdf, "-");
         m *= -1;
     }
     l = m / ten_pow[e];
     pdf_print_int(pdf, l);
     l = m % ten_pow[e];
     if (l != 0) {
-        pdf_printf(pdf, ".");
+        pdf_puts(pdf, ".");
         j = snprintf(a, 23, "%ld", l + ten_pow[e]);
         assert(j < 23);
         for (i = e; i > 0; i--) {
@@ -191,7 +175,7 @@ void print_pdffloat(PDF pdf, pdffloat f)
                 break;
             a[i] = '\0';
         }
-        pdf_printf(pdf, "%s", a + 1);
+        pdf_puts(pdf, (a + 1));
     }
 }
 
@@ -200,7 +184,7 @@ void print_pdf_matrix(PDF pdf, pdffloat * tm)
     int i;
     for (i = 0; i < 5; i++) {
         print_pdffloat(pdf, tm[i]);
-        pdf_printf(pdf, " ");
+        pdf_puts(pdf, " ");
     }
     print_pdffloat(pdf, tm[i]);
 }
@@ -208,7 +192,7 @@ void print_pdf_matrix(PDF pdf, pdffloat * tm)
 void pdf_print_cm(PDF pdf, pdffloat * cm)
 {
     print_pdf_matrix(pdf, cm);
-    pdf_printf(pdf, " cm\n");
+    pdf_puts(pdf, " cm\n");
 }
 
 void pdf_set_pos(PDF pdf, scaledpos pos)
@@ -241,7 +225,7 @@ static void begin_text(PDF pdf)
     pdfstructure *p = pdf->pstruct;
     assert(is_pagemode(p));
     p->pdf_bt_pos = p->pdf;
-    pdf_printf(pdf, "BT\n");
+    pdf_puts(pdf, "BT\n");
     p->mode = PMODE_TEXT;
 }
 
@@ -249,7 +233,7 @@ static void end_text(PDF pdf)
 {
     pdfstructure *p = pdf->pstruct;
     assert(is_textmode(p));
-    pdf_printf(pdf, "ET\n");
+    pdf_puts(pdf, "ET\n");
     p->pdf = p->pdf_bt_pos;
     p->mode = PMODE_PAGE;
 }
