@@ -18,31 +18,34 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char __svn_version[] =
-    "$Id: pdftables.c 3077 2009-10-25 22:51:23Z oneiros $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdftables.c $";
+    "$Id: pdftables.c 3247 2009-12-10 23:57:37Z oneiros $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdftables.c $";
 
 #include "ptexlib.h"
 
-integer pdf_box_spec_media = 1;
-integer pdf_box_spec_crop = 2;
-integer pdf_box_spec_bleed = 3;
-integer pdf_box_spec_trim = 4;
-integer pdf_box_spec_art = 5;
-
 /**********************************************************************/
-/* One AVL tree for each obj_type 0...pdf_objtype_max */
+/* One AVL tree each for a few obj_type out of 0...PDF_OBJ_TYPE_MAX */
 
-typedef enum {
-    avl_obj_int_type,
-    avl_obj_string_type,
-} avl_obj_types;
+/* mark which objects should be searchable through AVL tree */
 
-typedef struct oentry_ {
-    integer int0;
-    char *str0;
-    int objtype;
-    integer objptr;
-} oentry;
+static int obj_in_tree[PDF_OBJ_TYPE_MAX + 1] = {
+    0,                          /* obj_type_font = 0,       */
+    0,                          /* obj_type_outline = 1,    */
+    1,                          /* obj_type_dest = 2,       */
+    0,                          /* obj_type_obj = 3,        */
+    0,                          /* obj_type_xform = 4,      */
+    0,                          /* obj_type_ximage = 5,     */
+    1,                          /* obj_type_thread = 6,     */
+    /* the ones below don't go into a linked list */
+    0,                          /* obj_type_pagestream = 7, */
+    1,                          /* obj_type_page = 8,       */
+    0,                          /* obj_type_pages = 9,      */
+    0,                          /* obj_type_link = 10,      */
+    0,                          /* obj_type_bead = 11,      */
+    0,                          /* obj_type_annot = 12,     */
+    0,                          /* obj_type_objstm = 13,    */
+    0                           /* obj_type_others = 14     */
+};
 
 /* AVL sort oentry into avl_table[] */
 
@@ -52,22 +55,22 @@ static int compare_info(const void *pa, const void *pb, void *param)
     (void) param;
     a = (const oentry *) pa;
     b = (const oentry *) pb;
-    if (a->objtype == b->objtype) {
-        if (a->objtype == avl_obj_int_type) {
-            return ((a->int0 < b->int0 ? -1 : (a->int0 > b->int0 ? 1 : 0)));
-        } else {                /* string type */
-            return strcmp(a->str0, b->str0);
-        }
-    } else if (a->objtype == avl_obj_int_type) {
+    if (a->u_type == b->u_type) {
+        if (a->u_type == union_type_int)
+            return ((a->u.int0 <
+                     b->u.int0 ? -1 : (a->u.int0 > b->u.int0 ? 1 : 0)));
+        else                    /* string type */
+            return strcmp(a->u.str0, b->u.str0);
+    } else if (a->u_type == union_type_int)
         return -1;
-    } else {
+    else
         return 1;
-    }
 }
 
-void avl_put_obj(PDF pdf, integer t, oentry * oe)
+static void avl_put_obj(PDF pdf, integer t, oentry * oe)
 {
     void **pp;
+    assert(t >= 0 || t <= PDF_OBJ_TYPE_MAX || obj_in_tree[t] == 1);
     if (pdf->obj_tree[t] == NULL) {
         pdf->obj_tree[t] = avl_create(compare_info, NULL, &avl_xallocator);
         if (pdf->obj_tree[t] == NULL)
@@ -78,33 +81,33 @@ void avl_put_obj(PDF pdf, integer t, oentry * oe)
         pdftex_fail("avlstuff.c: avl_probe() out of memory in insertion");
 }
 
-void avl_put_int_obj(PDF pdf, integer int0, integer objptr, integer t)
+static void avl_put_int_obj(PDF pdf, integer int0, integer objptr, integer t)
 {
     oentry *oe;
     oe = xtalloc(1, oentry);
-    oe->int0 = int0;
-    oe->objtype = avl_obj_int_type;
+    oe->u.int0 = int0;
+    oe->u_type = union_type_int;
     oe->objptr = objptr;
     avl_put_obj(pdf, t, oe);
 }
 
-void avl_put_str_obj(PDF pdf, char *str0, integer objptr, integer t)
+static void avl_put_str_obj(PDF pdf, char *str0, integer objptr, integer t)
 {
     oentry *oe;
     oe = xtalloc(1, oentry);
-    oe->str0 = xstrdup(str0);
-    oe->objtype = avl_obj_string_type;
+    oe->u.str0 = str0;          /* no xstrdup() here */
+    oe->u_type = union_type_cstring;
     oe->objptr = objptr;
     avl_put_obj(pdf, t, oe);
 }
 
-
-integer avl_find_int_obj(PDF pdf, integer t, integer i)
+static integer avl_find_int_obj(PDF pdf, integer t, integer i)
 {
     oentry *p;
     oentry tmp;
-    tmp.int0 = i;
-    tmp.objtype = avl_obj_int_type;
+    assert(t >= 0 || t <= PDF_OBJ_TYPE_MAX || obj_in_tree[t] == 1);
+    tmp.u.int0 = i;
+    tmp.u_type = union_type_int;
     if (pdf->obj_tree[t] == NULL)
         return 0;
     p = (oentry *) avl_find(pdf->obj_tree[t], &tmp);
@@ -113,12 +116,13 @@ integer avl_find_int_obj(PDF pdf, integer t, integer i)
     return p->objptr;
 }
 
-integer avl_find_str_obj(PDF pdf, integer t, char *i)
+static integer avl_find_str_obj(PDF pdf, integer t, char *s)
 {
     oentry *p;
     oentry tmp;
-    tmp.str0 = i;
-    tmp.objtype = avl_obj_string_type;
+    assert(t >= 0 || t <= PDF_OBJ_TYPE_MAX || obj_in_tree[t] == 1);
+    tmp.u.str0 = s;
+    tmp.u_type = union_type_cstring;
     if (pdf->obj_tree[t] == NULL)
         return 0;
     p = (oentry *) avl_find(pdf->obj_tree[t], &tmp);
@@ -128,12 +132,12 @@ integer avl_find_str_obj(PDF pdf, integer t, char *i)
 }
 
 /**********************************************************************/
-
-
 /* create an object with type |t| and identifier |i| */
+
 void pdf_create_obj(PDF pdf, integer t, integer i)
 {
-    integer a, p, q;
+    integer a;
+    char *ss = NULL;
     if (pdf->sys_obj_ptr == sup_obj_tab_size)
         overflow("indirect objects table size", pdf->obj_tab_size);
     if (pdf->sys_obj_ptr == pdf->obj_tab_size) {
@@ -148,30 +152,15 @@ void pdf_create_obj(PDF pdf, integer t, integer i)
     incr(pdf->sys_obj_ptr);
     pdf->obj_ptr = pdf->sys_obj_ptr;
     obj_info(pdf, pdf->obj_ptr) = i;
+    obj_type(pdf, pdf->obj_ptr) = t;
     set_obj_fresh(pdf, pdf->obj_ptr);
     obj_aux(pdf, pdf->obj_ptr) = 0;
     if (i < 0) {
-        avl_put_str_obj(pdf, makecstring(-i), pdf->obj_ptr, t);
-    } else {
+        ss = makecstring(-i);
+        avl_put_str_obj(pdf, ss, pdf->obj_ptr, t);
+    } else
         avl_put_int_obj(pdf, i, pdf->obj_ptr, t);
-    }
-    if (t == obj_type_page) {
-        p = pdf->head_tab[t];
-        /* find the right position to insert newly created object */
-        if ((p == 0) || (obj_info(pdf, p) < i)) {
-            obj_link(pdf, pdf->obj_ptr) = p;
-            pdf->head_tab[t] = pdf->obj_ptr;
-        } else {
-            while (p != 0) {
-                if (obj_info(pdf, p) < i)
-                    break;
-                q = p;
-                p = obj_link(pdf, p);
-            }
-            obj_link(pdf, q) = pdf->obj_ptr;
-            obj_link(pdf, pdf->obj_ptr) = p;
-        }
-    } else if (t != obj_type_others) {
+    if (t <= HEAD_TAB_MAX) {
         obj_link(pdf, pdf->obj_ptr) = pdf->head_tab[t];
         pdf->head_tab[t] = pdf->obj_ptr;
         if ((t == obj_type_dest) && (i < 0))
@@ -180,8 +169,23 @@ void pdf_create_obj(PDF pdf, integer t, integer i)
     }
 }
 
-/* The following function finds object with identifier |i| and type |t|.
-   |i < 0| indicates that |-i| should be treated as a string number. If no
+integer find_obj(PDF pdf, integer t, integer i, boolean byname)
+{
+    char *ss = NULL;
+    integer ret;
+    assert(i >= 0);             /* no tricks */
+    if (byname) {
+        ss = makecstring(i);
+        ret = avl_find_str_obj(pdf, t, ss);
+        free(ss);
+    } else {
+        ret = avl_find_int_obj(pdf, t, i);
+    }
+    return ret;
+}
+
+/* The following function finds an object with identifier |i| and type |t|.
+   Identifier |i| is either an integer or a token list index. If no
    such object exists then it will be created. This function is used mainly to
    find destination for link annotations and outlines; however it is also used
    in |ship_out| (to check whether a Page object already exists) so we need
@@ -189,21 +193,11 @@ void pdf_create_obj(PDF pdf, integer t, integer i)
    |vlist_out|.
 */
 
-integer find_obj(PDF pdf, integer t, integer i, boolean byname)
-{
-
-    if (byname) {
-        if (i < 0)
-            i = abs(i);
-        return avl_find_str_obj(pdf, t, makecstring(i));
-    } else
-        return avl_find_int_obj(pdf, t, i);
-}
-
 integer get_obj(PDF pdf, integer t, integer i, boolean byname)
 {
     integer r;
     str_number s;
+    assert(i >= 0);
     if (byname > 0) {
         s = tokens_to_string(i);
         r = find_obj(pdf, t, s, true);
@@ -234,6 +228,14 @@ integer pdf_new_objnum(PDF pdf)
     return pdf->obj_ptr;
 }
 
+void check_obj_exists(PDF pdf, integer t, integer objnum)
+{
+    if (objnum < 0 || objnum > pdf->obj_ptr)
+        pdf_error("ext1", "cannot find referenced object");
+    if (t != obj_type(pdf, objnum))
+        pdf_error("ext1", "referenced object has wrong type");
+}
+
 void set_rect_dimens(PDF pdf, halfword p, halfword parent_box, scaledpos cur,
                      scaled_whd alt_rule, scaled margin)
 {
@@ -336,6 +338,8 @@ void dump_pdftex_data(PDF pdf)
         dump_int(x);
         x = obj_aux(pdf, k);
         dump_int(x);
+        x = obj_type(pdf, k);
+        dump_int(x);
     }
     print_ln();
     print_int(pdf->sys_obj_ptr);
@@ -409,6 +413,8 @@ void undump_pdftex_data(PDF pdf)
         obj_os_idx(pdf, k) = x;
         undump_int(x);
         obj_aux(pdf, k) = x;
+        undump_int(x);
+        obj_type(pdf, k) = x;
     }
 
     undump_int(x);
