@@ -20,8 +20,8 @@
 #include "ptexlib.h"
 
 static const char __svn_version[] =
-    "$Id: pdfthread.c 2978 2009-08-09 09:03:39Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfthread.c $";
+    "$Id: pdfthread.c 3234 2009-12-05 09:56:09Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfthread.c $";
 
 #define pdf_thread_margin        dimen_par(pdf_thread_margin_code)
 #define page_width dimen_par(page_width_code)
@@ -116,21 +116,15 @@ void end_thread(PDF pdf, halfword p)
         pdf_error("ext4",
                   "\\pdfendthread ended up in different nesting level than \\pdfstartthread");
     if (is_running(pdf->thread.dp) && (pdf->last_thread != null)) {
-        switch (box_direction(pdf->posstruct->dir)) {
-        case dir_TL_:
-        case dir_TR_:
+        switch (pdf->posstruct->dir) {
+        case dir_TLT:
+        case dir_TRT:
             pdf_ann_bottom(pdf->last_thread) = pos.v - pdf_thread_margin;
             break;
-        case dir_BL_:
-        case dir_BR_:
-            pdf_ann_top(pdf->last_thread) = pos.v + pdf_thread_margin;
-            break;
-        case dir_LT_:
-        case dir_LB_:
+        case dir_LTL:
             pdf_ann_right(pdf->last_thread) = pos.h + pdf_thread_margin;
             break;
-        case dir_RT_:
-        case dir_RB_:
+        case dir_RTT:
             pdf_ann_left(pdf->last_thread) = pos.h - pdf_thread_margin;
             break;
         }
@@ -172,7 +166,7 @@ void pdf_fix_thread(PDF pdf, integer t)
     pdf_indirect_ln(pdf, "T", t);
     pdf_indirect_ln(pdf, "V", a);
     pdf_indirect_ln(pdf, "N", a);
-    pdf_indirect_ln(pdf, "P", pdf->head_tab[obj_type_page]);
+    pdf_indirect_ln(pdf, "P", pdf->last_page);
     pdf_printf(pdf, "/R [0 0 ");
     pdf_print_bp(pdf, page_width);
     pdf_out(pdf, ' ');
