@@ -17,23 +17,46 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char __svn_version[] =
+    "$Id: pdfxform.c 3229 2009-12-04 21:09:46Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/pdf/pdfxform.c $";
+
 #include "ptexlib.h"
 #include "pdfpage.h"
 
 #define box(A) eqtb[box_base+(A)].hh.rh
 
-static const char __svn_version[] =
-    "$Id: pdfxform.c 3003 2009-08-16 19:55:55Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfxform.c $";
-
 integer pdf_cur_form;           /* the form being output */
 
 void pdf_place_form(PDF pdf, halfword p)
 {
-    integer objnum = pdf_xform_objnum(p);
+    scaled_whd nat, tex;
+    scaled x, y;
+    pdffloat cm[6];
+    pdfstructure *q = pdf->pstruct;
+    integer r = 6, objnum = pdf_xform_objnum(p);
+    nat.wd = obj_xform_width(pdf, objnum);
+    nat.ht = obj_xform_height(pdf, objnum);
+    nat.dp = obj_xform_depth(pdf, objnum);
+    /* no transform yet */
+    tex.wd = width(p);
+    tex.ht = height(p);
+    tex.dp = depth(p);
+    if (nat.wd != tex.wd || nat.ht != tex.ht || nat.dp != tex.dp) {
+        x = ext_xn_over_d(ten_pow[r], tex.wd, nat.wd);
+        y = ext_xn_over_d(ten_pow[r], tex.dp + tex.ht, nat.dp + nat.ht);
+    } else
+        x = y = ten_pow[r];
+    setpdffloat(cm[0], x, r);
+    setpdffloat(cm[1], 0, r);
+    setpdffloat(cm[2], 0, r);
+    setpdffloat(cm[3], y, r);
     pdf_goto_pagemode(pdf);
+    (void) calc_pdfpos(q, pdf->posstruct->pos);
+    cm[4] = q->cm[4];
+    cm[5] = q->cm[5];
     pdf_printf(pdf, "q\n");
-    pdf_set_pos_temp(pdf, pdf->posstruct->pos);
+    pdf_print_cm(pdf, cm);
     pdf_printf(pdf, "/Fm%d", (int) obj_info(pdf, objnum));
     pdf_print_resname_prefix(pdf);
     pdf_printf(pdf, " Do\nQ\n");
@@ -65,22 +88,36 @@ void scan_pdfxform(PDF pdf)
     p = box(cur_val);
     if (p == null)
         pdf_error("ext1", "\\pdfxform cannot be used with a void box");
+    set_obj_xform_box(pdf, k, p);       /* save pointer to the box */
     set_obj_xform_width(pdf, k, width(p));
     set_obj_xform_height(pdf, k, height(p));
     set_obj_xform_depth(pdf, k, depth(p));
-    set_obj_xform_box(pdf, k, p);       /* save pointer to the box */
     box(cur_val) = null;
     pdf_last_xform = k;
 }
 
+#define tail          cur_list.tail_field
 
 void scan_pdfrefxform(PDF pdf)
 {
+    integer transform = 0;
+    scaled_whd alt_rule, dim, nat;
+    alt_rule = scan_alt_rule(); /* scans |<rule spec>| to |alt_rule| */
     scan_int();
-    pdf_check_obj(pdf, obj_type_xform, cur_val);
+    check_obj_exists(pdf, obj_type_xform, cur_val);
     new_whatsit(pdf_refxform_node);
-    set_pdf_xform_objnum(cur_list.tail_field, cur_val);
-    set_pdf_width(cur_list.tail_field, obj_xform_width(pdf, cur_val));
-    set_pdf_height(cur_list.tail_field, obj_xform_height(pdf, cur_val));
-    set_pdf_depth(cur_list.tail_field, obj_xform_depth(pdf, cur_val));
+    nat.wd = obj_xform_width(pdf, cur_val);
+    nat.ht = obj_xform_height(pdf, cur_val);
+    nat.dp = obj_xform_depth(pdf, cur_val);
+    if (alt_rule.wd != null_flag || alt_rule.ht != null_flag
+        || alt_rule.dp != null_flag) {
+        dim = tex_scale(nat, alt_rule);
+    } else {
+        dim = nat;
+    }
+    width(tail) = dim.wd;
+    height(tail) = dim.ht;
+    depth(tail) = dim.dp;
+    pdf_xform_transform(tail) = transform;      /* not implemented yet */
+    pdf_xform_objnum(tail) = cur_val;
 }
