@@ -18,7 +18,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: ptexlib.h 3039 2009-08-22 06:12:19Z hhenkel $ */
+/* $Id: ptexlib.h 3229 2009-12-04 21:09:46Z hhenkel $ */
 
 #ifndef PTEXLIB_H
 #  define PTEXLIB_H
@@ -143,10 +143,6 @@ size_t          T##_limit
 #  define xtalloc             XTALLOC
 #  define xretalloc           XRETALLOC
 
-#  define set_cur_file_name(s) \
-    cur_file_name = s;         \
-    pack_file_name(maketexstring(cur_file_name), get_nullstr(), get_nullstr())
-
 #  define cmp_return(a, b) \
     if ((a) > (b))         \
         return 1;          \
@@ -158,7 +154,6 @@ size_t          T##_limit
 /* that was ptexmac.h */
 
 #  include "tex/mainbody.h"
-#  include "tex/textoken.h"
 #  include "tex/expand.h"
 #  include "tex/conditional.h"
 #  include "pdf/pdftypes.h"
@@ -218,6 +213,7 @@ size_t          T##_limit
 #  include "tex/errors.h"
 #  include "tex/inputstack.h"
 #  include "tex/stringpool.h"
+#  include "tex/textoken.h"
 #  include "tex/printing.h"
 #  include "tex/texfileio.h"
 #  include "tex/arithmetic.h"
@@ -253,11 +249,11 @@ extern halfword handle_ligaturing(halfword head, halfword tail);
 extern halfword handle_kerning(halfword head, halfword tail);
 
 halfword lua_hpack_filter(halfword head_node, scaled size, int pack_type,
-                          int extrainfo);
+                          int extrainfo, int d);
 void lua_node_filter(int filterid, int extrainfo, halfword head_node,
                      halfword * tail_node);
 halfword lua_vpack_filter(halfword head_node, scaled size, int pack_type,
-                          scaled maxd, int extrainfo);
+                          scaled maxd, int extrainfo, int d);
 void lua_node_filter_s(int filterid, char *extrainfo);
 int lua_linebreak_callback(int is_broken, halfword head_node,
                            halfword * new_head);
@@ -375,18 +371,6 @@ extern char *get_lua_name(int i);
 /* test whether a char in font is marked */
 #  define pdf_char_marked char_used
 
-#  define tex_b_open_in(f) \
-    open_input (&(f), kpse_tex_format, FOPEN_RBIN_MODE)
-#  define ovf_b_open_in(f) \
-    open_input (&(f), kpse_ovf_format, FOPEN_RBIN_MODE)
-#  define vf_b_open_in(f) \
-    open_input (&(f), kpse_vf_format, FOPEN_RBIN_MODE)
-
-extern int open_outfile(FILE ** f, char *name, char *mode);
-
-#  define do_a_open_out(f) open_outfile(&(f),(char *)(nameoffile+1),FOPEN_W_MODE)
-#  define do_b_open_out(f) open_outfile(&(f),(char *)(nameoffile+1),FOPEN_WBIN_MODE)
-
 #  define pdfassert assert
 #  define voidcast(a) (void *)(a)
 #  define varmemcast(a) (memory_word *)(a)
@@ -402,13 +386,6 @@ extern void undump_node_mem(void);
 
 extern void do_vf(internal_font_number tmp_f);
 
-extern int readbinfile(FILE * f, unsigned char **b, integer * s);
-
-#  define read_tfm_file  readbinfile
-#  define read_vf_file   readbinfile
-#  define read_ocp_file  readbinfile
-#  define read_data_file readbinfile
-
 /* This routine has to return four values.  */
 #  define	dateandtime(i,j,k,l) get_date_and_time (&(i), &(j), &(k), &(l))
 extern void get_date_and_time(integer *, integer *, integer *, integer *);
@@ -432,41 +409,6 @@ extern void topenin(void);
 /* Set an array size from texmf.cnf.  */
 extern void setupboundvariable(integer *, const_string, integer);
 
-/* These defines reroute the file i/o calls to the new pipe-enabled 
-   functions in texmfmp.c*/
-
-#  undef aopenin
-#  undef aopenout
-#  undef aclose
-#  define a_open_in(f,p)  open_in_or_pipe(&(f),p,FOPEN_RBIN_MODE)
-#  define a_open_out(f)   open_out_or_pipe(&(f),FOPEN_W_MODE)
-#  define a_close(f)     close_file_or_pipe(f)
-
-/* `bopenin' (and out) is used only for reading (and writing) .tfm
-   files; `wopenin' (and out) only for dump files.  The filenames are
-   passed in as a global variable, `nameoffile'.  */
-#  define b_open_in(f)	open_input (&(f), kpse_tfm_format, FOPEN_RBIN_MODE)
-#  define ocp_open_in(f)	open_input (&(f), kpse_ocp_format, FOPEN_RBIN_MODE)
-#  define ofm_open_in(f)	open_input (&(f), kpse_ofm_format, FOPEN_RBIN_MODE)
-#  define b_open_out(f)	open_output (&(f), FOPEN_WBIN_MODE)
-
-/* Used in tex.ch (section 1338) to get a core dump in debugging mode.  */
-#  ifdef unix
-#    define dumpcore abort
-#  else
-#    define dumpcore uexit (1)
-#  endif
-
-#  define b_close close_file
-/* We define the routines to do the actual work in texmf.c.  */
-#  define w_open_in(f)     zopen_w_input (&(f), DUMP_FORMAT, FOPEN_RBIN_MODE)
-#  define w_open_out(f)    zopen_w_output (&(f), FOPEN_WBIN_MODE)
-#  define w_close         zwclose
-
-extern boolean zopen_w_input(FILE **, int, const_string fopen_mode);
-extern boolean zopen_w_output(FILE **, const_string fopen_mode);
-extern void zwclose(FILE *);
-
 /* here  are a few functions that used to be in coerce.h */
 
 extern str_number getjobname(str_number);
