@@ -20,8 +20,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: commands.c 2977 2009-08-08 12:49:35Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/commands.c $";
+    "$Id: commands.c 3203 2009-12-01 10:18:20Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/commands.c $";
 
 void initialize_commands(void)
 {
@@ -84,6 +84,10 @@ void initialize_commands(void)
                      local_base);
     primitive_pdftex("pdfpageresources", assign_toks_cmd,
                      pdf_page_resources_loc, local_base);
+    primitive_pdftex("pdfxformattr", assign_toks_cmd, pdf_xform_attr_loc,
+                     local_base);
+    primitive_pdftex("pdfxformresources", assign_toks_cmd,
+                     pdf_xform_resources_loc, local_base);
     primitive_pdftex("pdfpkmode", assign_toks_cmd, pdf_pk_mode_loc, local_base);
 
     /* The integer parameter names must be entered into the hash table */
@@ -535,7 +539,11 @@ void initialize_commands(void)
     cs_text(frozen_null_ocp_list) = maketexstring("nullocplist");
     eqtb[frozen_null_ocp_list] = eqtb[cur_val];
 
-    primitive_tex("span", tab_mark_cmd, span_code, span_code);
+    primitive_tex("span", tab_mark_cmd, span_code, tab_mark_cmd_code);
+    primitive_luatex("aligntab", tab_mark_cmd, tab_mark_cmd_code,
+                     tab_mark_cmd_code);
+    primitive_luatex("alignmark", mac_param_cmd, tab_mark_cmd_code,
+                     tab_mark_cmd_code);
     primitive_tex("cr", car_ret_cmd, cr_code, cr_code);
     cs_text(frozen_cr) = maketexstring("cr");
     eqtb[frozen_cr] = eqtb[cur_val];
@@ -1030,8 +1038,6 @@ void initialize_commands(void)
                  int_base + no_local_whatsits_code, int_base);
     primitive_no("nolocaldirs", assign_int_cmd,
                  int_base + no_local_dirs_code, int_base);
-    primitive_no("levellocaldir", assign_int_cmd,
-                 int_base + level_local_dir_code, int_base);
 
 }
 
