@@ -21,48 +21,36 @@
 
 
 static const char __svn_version[] =
-    "$Id: directions.c 2954 2009-08-05 19:06:00Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/directions.c $";
-
-/* |scan_direction| has to be defined here because luatangle will output
-   a character constant when it sees a string literal of length 1 */
-
-#define scan_single_dir(A) do {                     \
-        if (scan_keyword("T")) A=dir_T;             \
-        else if (scan_keyword("L")) A=dir_L;        \
-        else if (scan_keyword("B")) A=dir_B;        \
-        else if (scan_keyword("R")) A=dir_R;        \
-        else {                                      \
-            tex_error("Bad direction", NULL);       \
-            cur_val=0;                              \
-            return;                                 \
-        }                                           \
-    } while (0)
+    "$Id: directions.c 3204 2009-12-01 10:40:28Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/directions.c $";
 
 void scan_direction(void)
 {
-    integer d1, d2, d3;
     int save_cur_cmd = cur_cmd;
     int save_cur_chr = cur_chr;
     get_x_token();
     if (cur_cmd == assign_dir_cmd) {
         cur_val = eqtb[cur_chr].cint;
-        return;
+        goto EXIT;
     } else {
         back_input();
     }
-    scan_single_dir(d1);
-    scan_single_dir(d2);
-    if (dir_parallel(d1, d2)) {
+    if (scan_keyword("TLT")) {
+        cur_val = dir_TLT;
+    } else if (scan_keyword("TRT")) {
+        cur_val = dir_TRT;
+    } else if (scan_keyword("LTL")) {
+        cur_val = dir_LTL;
+    } else if (scan_keyword("RTT")) {
+        cur_val = dir_RTT;
+    } else {
         tex_error("Bad direction", NULL);
         cur_val = 0;
-        return;
     }
-    scan_single_dir(d3);
     get_x_token();
     if (cur_cmd != spacer_cmd)
         back_input();
-    cur_val = d1 * 8 + dir_rearrange[d2] * 4 + d3;
+  EXIT:
     cur_cmd = save_cur_cmd;
     cur_chr = save_cur_chr;
 }
@@ -86,18 +74,14 @@ halfword do_pop_dir_node(halfword p)
 
 halfword dir_ptr;
 
-integer dvi_direction;
 int dir_primary[32];
 int dir_secondary[32];
 int dir_tertiary[32];
-int dir_rearrange[4];
-str_number dir_names[4];
 halfword text_dir_ptr;
 
 void initialize_directions(void)
 {
     int k;
-    pack_direction = -1;
     for (k = 0; k <= 7; k++) {
         dir_primary[k] = dir_T;
         dir_primary[k + 8] = dir_L;
@@ -121,14 +105,6 @@ void initialize_directions(void)
         dir_tertiary[k * 4 + 2] = dir_B;
         dir_tertiary[k * 4 + 3] = dir_R;
     }
-    dir_rearrange[0] = 0;
-    dir_rearrange[1] = 0;
-    dir_rearrange[2] = 1;
-    dir_rearrange[3] = 1;
-    dir_names[0] = 'T';
-    dir_names[1] = 'L';
-    dir_names[2] = 'B';
-    dir_names[3] = 'R';
 }
 
 halfword new_dir(int s)
@@ -141,11 +117,25 @@ halfword new_dir(int s)
     return p;
 }
 
+char *string_dir(int d)
+{
+    if (d == dir_TLT) {
+        return "TLT";
+    } else if (d == dir_TRT) {
+        return "TRT";
+    } else if (d == dir_LTL) {
+        return "LTL";
+    } else if (d == dir_RTT) {
+        return "RTT";
+    } else {
+        return "???";
+    }
+}
+
+
 void print_dir(int d)
 {
-    print_char(dir_names[dir_primary[d]]);
-    print_char(dir_names[dir_secondary[d]]);
-    print_char(dir_names[dir_tertiary[d]]);
+    tprint(string_dir(d));
 }
 
 /**********************************************************************/
@@ -154,13 +144,13 @@ scaled pack_width(int curdir, int pdir, halfword p, boolean isglyph)
 {
     scaled wd = 0;
     if (isglyph) {
-        if (dir_parallel(dir_secondary[curdir], dir_secondary[pdir]) ==
+        if (textdir_parallel(curdir, pdir) ==
             dir_orthogonal(dir_secondary[pdir], dir_tertiary[pdir]))
             wd = glyph_width(p);
         else
             wd = glyph_depth(p) + glyph_height(p);
     } else {                    /* hlist, vlist, image, form, rule */
-        if (dir_parallel(dir_secondary[pdir], dir_secondary[curdir]))
+        if (textdir_parallel(pdir, curdir))
             wd = width(p);
         else
             wd = depth(p) + height(p);
@@ -175,23 +165,21 @@ scaled_whd pack_width_height_depth(int curdir, int pdir, halfword p,
     whd.wd = pack_width(curdir, pdir, p, isglyph);
     if (isglyph) {
         if (is_rotated(curdir)) {
-            if (dir_parallel(dir_secondary[curdir], dir_secondary[pdir]))
+            if (textdir_parallel(curdir, pdir))
                 whd.ht = whd.dp = (glyph_height(p) + glyph_depth(p)) / 2;
             else
                 whd.ht = whd.dp = glyph_width(p) / 2;
         } else {
             if (is_rotated(pdir)) {
-                if (dir_parallel(dir_secondary[curdir], dir_secondary[pdir]))
+                if (textdir_parallel(curdir, pdir))
                     whd.ht = whd.dp = (glyph_height(p) + glyph_depth(p)) / 2;
                 else
                     whd.ht = glyph_width(p);
             } else {
-                if (dir_eq(dir_tertiary[curdir], dir_tertiary[pdir])) {
+                if (glyphdir_eq(curdir, pdir)) {
                     whd.ht = glyph_height(p);
                     whd.dp = glyph_depth(p);
-                } else
-                    if (dir_opposite(dir_tertiary[curdir], dir_tertiary[pdir]))
-                {
+                } else if (glyphdir_opposite(curdir, pdir)) {
                     whd.ht = glyph_depth(p);
                     whd.dp = glyph_height(p);
                 } else
@@ -200,15 +188,15 @@ scaled_whd pack_width_height_depth(int curdir, int pdir, halfword p,
         }
     } else {
         if (is_rotated(curdir)) {
-            if (dir_parallel(dir_secondary[curdir], dir_secondary[pdir]))
+            if (textdir_parallel(curdir, pdir))
                 whd.ht = whd.dp = (height(p) + depth(p)) / 2;
             else
                 whd.ht = whd.dp = width(p) / 2;
         } else {
-            if (dir_eq(dir_primary[curdir], dir_primary[pdir])) {
+            if (pardir_eq(curdir, pdir)) {
                 whd.ht = height(p);
                 whd.dp = depth(p);
-            } else if (dir_opposite(dir_primary[curdir], dir_primary[pdir])) {
+            } else if (pardir_opposite(curdir, pdir)) {
                 whd.ht = depth(p);
                 whd.dp = height(p);
             } else
@@ -217,3 +205,16 @@ scaled_whd pack_width_height_depth(int curdir, int pdir, halfword p,
     }
     return whd;
 }
+
+void update_text_dir_ptr (int val)
+{
+    if (dir_level(text_dir_ptr) == cur_level) {
+        /* update */
+        dir_dir(text_dir_ptr) = val;
+    } else {
+        /* addition */
+        halfword text_dir_tmp = new_dir(val);
+        vlink(text_dir_tmp) = text_dir_ptr;
+        text_dir_ptr = text_dir_tmp;
+    }
+}
