@@ -20,8 +20,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: dumpdata.c 2886 2009-07-17 13:47:29Z taco $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/dumpdata.c $";
+    "$Id: dumpdata.c 3221 2009-12-04 10:49:35Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/dumpdata.c $";
 
 #define font_id_text(A) cs_text(font_id_base+(A))
 #define prev_depth cur_list.aux_field.cint
@@ -61,6 +61,7 @@ void store_fmt_file(void)
     halfword p;                 /* all-purpose pointer */
     integer x;                  /* something to dump */
     char *format_engine;
+    char *fmtname = NULL;
     /* If dumping is not allowed, abort */
     /* The user is not allowed to dump a format file unless |save_ptr=0|.
        This condition implies that |cur_level=level_one|, hence
@@ -91,12 +92,14 @@ void store_fmt_file(void)
         selector = log_only;
     else
         selector = term_and_log;
-    pack_job_name(format_extension);
-    while (!w_open_out(fmt_file))
-        prompt_file_name("format file name", format_extension);
+
+    fmtname = pack_job_name(format_extension);
+    while (!zopen_w_output(&fmt_file, fmtname, FOPEN_WBIN_MODE)) {
+        fmtname = prompt_file_name("format file name", format_extension);
+    }
     tprint_nl("Beginning to dump on file ");
-    slow_print(w_make_name_string(fmt_file));
-    flush_string();
+    tprint(fmtname);
+    free(fmtname);
     tprint_nl("");
     slow_print(format_ident);
 
@@ -122,14 +125,12 @@ void store_fmt_file(void)
     dump_int(hash_prime);
 
     /* Dump the string pool */
-    dump_int(pool_ptr);
-    dump_int((str_ptr - STRING_OFFSET));
-    dump_things(str_start[0], (str_ptr - STRING_OFFSET) + 1);
-    dump_things(str_pool[0], pool_ptr);
+    k = dump_string_pool();
     print_ln();
-    print_int(str_ptr - STRING_OFFSET);
-    tprint(" strings of total length ");
-    print_int(pool_ptr);
+    print_int(k);    
+    tprint(" strings using ");
+    print_int(pool_size); 
+    tprint(" bytes");
 
     /* Dump the dynamic memory */
     /* By sorting the list of available spaces in the variable-size portion of
@@ -264,9 +265,7 @@ void store_fmt_file(void)
         tprint_nl("\\font");
         print_esc(font_id_text(k));
         print_char('=');
-        print_file_name(tex_font_name(k), tex_font_area(k), get_nullstr());
-        flush_string();
-        flush_string();
+        tprint_file_name((unsigned char *)font_name(k), (unsigned char *)font_area(k), NULL);
         if (font_size(k) != font_dsize(k)) {
             tprint(" at ");
             print_scaled(font_size(k));
@@ -304,7 +303,7 @@ void store_fmt_file(void)
     dump_luac_registers();
 
     /* Close the format file */
-    w_close(fmt_file);
+    zwclose(fmt_file);
 }
 
 /*
@@ -348,7 +347,7 @@ read an integer value |x| that is supposed to be in the range |a<=x<=b|.
     } while (0)
 
 
-boolean load_fmt_file(void)
+boolean load_fmt_file(char *fmtname)
 {
     integer j, k;               /* all-purpose indices */
     halfword p;                 /* all-purpose pointer */
@@ -356,8 +355,6 @@ boolean load_fmt_file(void)
     char *format_engine;
     /* Undump constants for consistency check */
     if (ini_version) {
-        libcfree(str_pool);
-        libcfree(str_start);
         libcfree(hash);
         libcfree(eqtb);
         libcfree(fixmem);
@@ -378,8 +375,7 @@ boolean load_fmt_file(void)
     if (strcmp(engine_name, format_engine)) {
         wake_up_terminal();
         wterm_cr();
-        fprintf(term_out, "---! %s was written by %s",
-                (char *) (nameoffile + 1), format_engine);
+        fprintf(term_out, "---! %s was written by %s", fmtname, format_engine);
         xfree(format_engine);
         goto BAD_FMT;
     }
@@ -390,7 +386,7 @@ boolean load_fmt_file(void)
         wake_up_terminal();
         wterm_cr();
         fprintf(term_out, "---! %s was written by a different version",
-                (char *) (nameoffile + 1));
+                fmtname);
         goto BAD_FMT;
     }
     undump_int(x);
@@ -422,21 +418,7 @@ boolean load_fmt_file(void)
         goto BAD_FMT;
 
     /* Undump the string pool */
-    undump_size(0, sup_pool_size - pool_free, "string pool size", pool_ptr);
-    if (pool_size < pool_ptr + pool_free)
-        pool_size = pool_ptr + pool_free;
-    undump_size(0, sup_max_strings - strings_free, "sup strings", str_ptr);
-    if (max_strings < str_ptr + strings_free)
-        max_strings = str_ptr + strings_free;
-    str_start = xmallocarray(pool_pointer, max_strings);
-    str_ptr = str_ptr + STRING_OFFSET;
-    undump_checked_things(0, pool_ptr, str_start[0],
-                          (unsigned) ((str_ptr - STRING_OFFSET) + 1));
-    str_pool = xmallocarray(packed_ASCII_code, pool_size);
-    undump_things(str_pool[0], pool_ptr);
-    init_str_ptr = str_ptr;
-    init_pool_ptr = pool_ptr;
-
+    str_ptr = undump_string_pool();
     /* Undump the dynamic memory */
     undump_node_mem();
     undump_int(temp_token_head);
