@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: equivalents.h 2944 2009-08-02 22:14:29Z oneiros $ */
+/* $Id: equivalents.h 3203 2009-12-01 10:18:20Z taco $ */
 
 #ifndef EQUIVALENTS_H
 #  define EQUIVALENTS_H
@@ -196,8 +196,10 @@ here, and the |number_regs| \.{\\dimen} registers.
 #  define pdf_pages_attr_loc (pdftex_first_loc + 0)     /* points to token list for \.{\\pdfpagesattr} */
 #  define pdf_page_attr_loc (pdftex_first_loc + 1)      /* points to token list for \.{\\pdfpageattr} */
 #  define pdf_page_resources_loc (pdftex_first_loc + 2) /* points to token list for \.{\\pdfpageresources} */
-#  define pdf_pk_mode_loc (pdftex_first_loc + 3)        /* points to token list for \.{\\pdfpkmode} */
-#  define pdf_toks (pdftex_first_loc+4) /* end of \pdfTeX's token list parameters */
+#  define pdf_xform_attr_loc (pdftex_first_loc + 3)     /* points to token list for \.{\\pdfxformattr} */
+#  define pdf_xform_resources_loc (pdftex_first_loc + 4)        /* points to token list for \.{\\pdfxformresources} */
+#  define pdf_pk_mode_loc (pdftex_first_loc + 5)        /* points to token list for \.{\\pdfpkmode} */
+#  define pdf_toks (pdftex_first_loc+6) /* end of \pdfTeX's token list parameters */
 #  define ocp_trace_level_base (pdf_toks)
 #  define ocp_active_number_base (ocp_trace_level_base+1)
 #  define ocp_active_min_ptr_base (ocp_active_number_base+1)
@@ -285,7 +287,7 @@ here, and the |number_regs| \.{\\dimen} registers.
 #  define local_broken_penalty_code 56  /* local \.{\\brokenpenalty} */
 #  define no_local_whatsits_code 57     /* counts local whatsits */
 #  define no_local_dirs_code 58
-#  define level_local_dir_code 59
+
 #  define luastartup_id_code 60
 #  define disable_lig_code 61
 #  define disable_kern_code 62
