@@ -1,4 +1,4 @@
-/* commands.c
+/* extensions.c
 
    Copyright 2009 Taco Hoekwater <taco@luatex.org>
 
@@ -17,11 +17,11 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include <ptexlib.h>
-
 static const char _svn_version[] =
-    "$Id: extensions.c 3044 2009-09-13 16:28:28Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/extensions.c $";
+    "$Id: extensions.c 3229 2009-12-04 21:09:46Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/extensions.c $";
+
+#include <ptexlib.h>
 
 #define mode          cur_list.mode_field
 #define tail          cur_list.tail_field
@@ -131,7 +131,7 @@ the |do_extension| routine is called.
 
 void do_extension(PDF pdf)
 {
-    integer i, j, k;            /* all-purpose integers */
+    integer i, k;               /* all-purpose integers */
     halfword p;                 /* all-purpose pointer */
     switch (cur_chr) {
     case open_node:
@@ -269,7 +269,7 @@ void do_extension(PDF pdf)
         scan_pdf_ext_toks();
         set_pdf_font_attr(k, tokens_to_string(def_ref));
         if (str_length(pdf_font_attr(k)) == 0) {
-            flush_string();
+            flush_str((str_ptr-1)); /* from tokens_to_string */
             set_pdf_font_attr(k, 0);
         }
         break;
@@ -402,10 +402,7 @@ void do_extension(PDF pdf)
     case pdf_refobj_node:
         /* Implement \.{\\pdfrefobj} */
         check_o_mode(pdf, "\\pdfrefobj", 1 << OMODE_PDF, false);
-        scan_int();
-        pdf_check_obj(pdf, obj_type_obj, cur_val);
-        new_whatsit(pdf_refobj_node);
-        set_pdf_obj_objnum(tail, cur_val);
+        scan_refobj(pdf);
         break;
     case pdf_refxform_node:
         /* Implement \.{\\pdfrefxform} */
@@ -415,14 +412,7 @@ void do_extension(PDF pdf)
     case pdf_refximage_node:
         /* Implement \.{\\pdfrefximage} */
         check_o_mode(pdf, "\\pdfrefximage", 1 << OMODE_PDF, false);
-        scan_int();
-        pdf_check_obj(pdf, obj_type_ximage, cur_val);
-        new_whatsit(pdf_refximage_node);
-        j = obj_data_ptr(pdf, cur_val);
-        set_pdf_ximage_idx(tail, j);
-        set_pdf_width(tail, image_width(j));
-        set_pdf_height(tail, image_height(j));
-        set_pdf_depth(tail, image_depth(j));
+        scan_pdfrefximage(pdf);
         break;
     case pdf_save_pos_node:
         /* Implement \.{\\pdfsavepos} */
@@ -461,7 +451,7 @@ void do_extension(PDF pdf)
         check_o_mode(pdf, "\\pdfximage", 1 << OMODE_PDF, false);
         /* png, jpeg, and pdf image handling depends on this done so early: */
         fix_pdf_minorversion(pdf);
-        scan_image(pdf);
+        scan_pdfximage(pdf);
         break;
     case save_cat_code_table_code:
         /* Implement \.{\\savecatcodetable} */
@@ -597,13 +587,6 @@ integer pdf_last_xform;
 integer pdf_last_ximage;
 integer pdf_last_ximage_pages;
 integer pdf_last_ximage_colordepth;
-
-void flush_str(str_number s)
-{                               /* flush a string if possible */
-    if (flushable(s))
-        flush_string();
-}
-
 integer pdf_last_annot;
 
 /* pdflastlink needs an extra global variable */
@@ -1003,16 +986,12 @@ integer get_tex_toks_register(integer j)
     return s;
 }
 
-integer set_tex_toks_register(integer j, str_number s)
+integer set_tex_toks_register(integer j, lstring s)
 {
-    pool_pointer s_pool_ptr;
     halfword ref;
     int a;
-    s_pool_ptr = pool_ptr;
-    pool_ptr = str_start_macro(s + 1);
     ref = get_avail();
-    (void) str_toks(str_start_macro(s));
-    pool_ptr = s_pool_ptr;
+    (void) str_toks(s);
     set_token_ref_count(ref, 0);
     set_token_link(ref, token_link(temp_token_head));
     if (global_defs > 0)
@@ -1020,7 +999,6 @@ integer set_tex_toks_register(integer j, str_number s)
     else
         a = 0;
     define(j + toks_base, call_cmd, ref);
-    flush_str(s);
     return 0;
 }
 
@@ -1231,13 +1209,6 @@ The |edit_name_start| will be set to point into |str_pool| somewhere after
 its beginning if \TeX\ is supposed to switch to an editor on exit.
 */
 
-/*
-These are used when we regenerate the representation of the first 256
-strings.
-*/
-
-str_number save_str_ptr;
-pool_pointer save_pool_ptr;
 int shellenabledp;
 int restrictedshell;
 char *output_comment;
