@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: extensions.h 3037 2009-08-20 20:15:09Z hhenkel $ */
+/* $Id: extensions.h 3221 2009-12-04 10:49:35Z taco $ */
 
 #ifndef EXTENSIONS_H
 #  define EXTENSIONS_H
@@ -43,8 +43,6 @@ typesetting engine itself. Each has an id, a type, and a value. The
 type of the value depends on the |user_node_type| field.
 */
 
-#  define flushable(A)  ((A) == str_ptr - 1)
-
 extern void new_whatsit(int s);
 extern void new_write_whatsit(int w);
 extern void scan_pdf_ext_toks(void);
@@ -53,7 +51,6 @@ extern integer pdf_last_xform;
 extern integer pdf_last_ximage;
 extern integer pdf_last_ximage_pages;
 extern integer pdf_last_ximage_colordepth;
-extern void flush_str(str_number s);
 extern integer pdf_last_annot;
 extern integer pdf_last_link;
 extern scaledpos pdf_last_pos;
@@ -94,7 +91,7 @@ extern integer set_tex_count_register(integer j, scaled v);
 extern integer set_tex_box_register(integer j, scaled v);
 extern integer set_tex_attribute_register(integer j, scaled v);
 extern integer get_tex_toks_register(integer l);
-extern integer set_tex_toks_register(integer j, str_number s);
+extern integer set_tex_toks_register(integer j, lstring s);
 extern scaled get_tex_box_width(integer j);
 extern integer set_tex_box_width(integer j, scaled v);
 extern scaled get_tex_box_height(integer j);
@@ -113,8 +110,6 @@ extern pool_pointer edit_name_start;
 extern integer edit_name_length, edit_line;
 extern int ipcon;
 extern boolean stop_at_space;
-extern str_number save_str_ptr;
-extern pool_pointer save_pool_ptr;
 extern int shellenabledp;
 extern int restrictedshell;
 extern char *output_comment;
