@@ -23,7 +23,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: filename.c 2869 2009-07-16 11:31:18Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/filename.c $";
+    "$Id: filename.c 3258 2009-12-16 10:40:36Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/filename.c $";
 
 /* for use by |scan_file_name|, it comes from fontforge's Unicode library */
 
@@ -100,26 +100,32 @@ static boolean more_name(ASCII_code c)
 
 static void end_name(void)
 {
+    unsigned char *s;
     if (str_ptr + 3 > (max_strings + STRING_OFFSET))
-        overflow("number of strings", max_strings - init_str_ptr);
+        overflow("number of strings", max_strings - init_str_ptr + STRING_OFFSET);
     /* @:TeX capacity exceeded number of strings}{\quad number of strings@> */
-
+    /* at this point, the full string lives in |cur_string| */
     if (area_delimiter == 0) {
         cur_area = get_nullstr();
     } else {
-        cur_area = str_ptr;
-        str_start_macro(str_ptr + 1) =
-            str_start_macro(str_ptr) + area_delimiter;
-        incr(str_ptr);
+        s = (unsigned char *)xstrdup((char *)(cur_string + area_delimiter));
+        cur_string[area_delimiter] = '\0';
+        cur_length = strlen((char *)cur_string);
+        cur_area = make_string();
+        cur_length = strlen((char *)s);
+        cur_string = s;
     }
     if (ext_delimiter == 0) {
-        cur_ext = get_nullstr();
         cur_name = make_string();
+        cur_ext = get_nullstr();
     } else {
-        cur_name = str_ptr;
-        str_start_macro(str_ptr + 1) =
-            str_start_macro(str_ptr) + ext_delimiter - area_delimiter - 1;
-        incr(str_ptr);
+        int l = (ext_delimiter - area_delimiter -1);
+        s = (unsigned char *)xstrdup((char *)(cur_string + l));
+        cur_string[l] = '\0';
+        cur_length = strlen((char *)cur_string);
+        cur_name = make_string();
+        cur_length = strlen((char *)s);
+        cur_string = s;
         cur_ext = make_string();
     }
 }
@@ -169,6 +175,39 @@ void scan_file_name(void)
     name_in_progress = false;
 }
 
+/* This function constructs a the three file name strings from a token list */
+
+void scan_file_name_toks(void)
+{
+    char *a, *n, *e, *s = NULL;
+    int i, l = 0;
+    (void) scan_toks(false, true);
+    s = tokenlist_to_cstring(def_ref, true, &l);
+    a = n = s;
+    e = NULL;
+    for (i = 0; i < l; i++) {
+        if (ISDIRSEP(s[i])) {
+            n = s + i + 1;
+            e = NULL;
+        } else if (s[i] == '.') {
+            e = s + i;
+        }
+    }
+    if (n != s) {               /* explicit area */
+        cur_area = maketexlstring(a, (n - a));
+    } else {
+        cur_area = get_nullstr();
+    }
+    if (e != NULL) {            /* explicit extension */
+        cur_name = maketexlstring(n, (e - n));
+        cur_ext = maketexstring(e);
+    } else {
+        cur_name = maketexstring(n);
+        cur_ext = get_nullstr();
+    }
+}
+
+
 
 /*
   Here is a routine that manufactures the output file names, assuming that
@@ -176,25 +215,24 @@ void scan_file_name(void)
   and |cur_ext|.
 */
 
-#define pack_cur_name() pack_file_name(cur_name,cur_area,cur_ext)
 
-void pack_job_name(char *s)
+char *pack_job_name(char *s)
 {                               /* |s = ".log"|, |".dvi"|, or |format_extension| */
     cur_area = get_nullstr();
     cur_ext = maketexstring(s);
     cur_name = job_name;
-    pack_cur_name();
+    return pack_file_name(cur_name, cur_area, cur_ext);
 }
 
 /* If some trouble arises when \TeX\ tries to open a file, the following
    routine calls upon the user to supply another file name. Parameter~|s|
    is used in the error message to identify the type of file; parameter~|e|
    is the default extension if none is given. Upon exit from the routine,
-   variables |cur_name|, |cur_area|, |cur_ext|, and |nameoffile| are
+   variables |cur_name|, |cur_area|, and |cur_ext| are
    ready for another attempt at file opening.
 */
 
-void prompt_file_name(char *s, char *e)
+char *prompt_file_name(char *s, char *e)
 {
     int k;                      /* index into |buffer| */
     str_number saved_cur_name;  /* to catch empty terminal input */
@@ -204,9 +242,9 @@ void prompt_file_name(char *s, char *e)
     if (interaction == scroll_mode) {
         wake_up_terminal();
     }
-    ar = xstrdup(makecstring(cur_area));
-    na = xstrdup(makecstring(cur_name));
-    ex = xstrdup(makecstring(cur_ext));
+    ar = makecstring(cur_area);
+    na = makecstring(cur_name);
+    ex = makecstring(cur_ext);
     if (strcmp(s, "input file name") == 0) {    /* @.I can't find file x@> */
         snprintf(prompt, 255, "I can't find file `%s%s%s'.", ar, na, ex);
     } else {                    /*@.I can't write on file x@> */
@@ -240,70 +278,33 @@ void prompt_file_name(char *s, char *e)
         cur_ext = maketexstring(e);
     if (str_length(cur_name) == 0)
         cur_name = saved_cur_name;
-    pack_cur_name();
+    return pack_file_name(cur_name, cur_area, cur_ext);
 }
 
-
-str_number make_name_string(void)
-{
-    int k;                      /* index into |nameoffile| */
-    pool_pointer save_area_delimiter, save_ext_delimiter;
-    boolean save_name_in_progress, save_stop_at_space;
-    str_number ret;
-    if ((pool_ptr + namelength > pool_size) ||
-        (str_ptr == max_strings) || (cur_length > 0)) {
-        ret = maketexstring("?");
-    } else {
-        for (k = 1; k <= namelength; k++)
-            append_char(nameoffile[k]);
-        ret = make_string();
-    }
-    /* At this point we also reset |cur_name|, |cur_ext|, and |cur_area| to
-       match the contents of |nameoffile|. */
-    save_area_delimiter = area_delimiter;
-    save_ext_delimiter = ext_delimiter;
-    save_name_in_progress = name_in_progress;
-    save_stop_at_space = stop_at_space;
-    name_in_progress = true;
-    begin_name();
-    stop_at_space = false;
-    k = 1;
-    while ((k <= namelength) && (more_name(nameoffile[k])))
-        k++;
-    stop_at_space = save_stop_at_space;
-    end_name();
-    name_in_progress = save_name_in_progress;
-    area_delimiter = save_area_delimiter;
-    ext_delimiter = save_ext_delimiter;
-    return ret;
-}
-
-
-
-void print_file_name(str_number n, str_number a, str_number e)
+void tprint_file_name(unsigned char *n, unsigned char *a, unsigned char *e)
 {
     boolean must_quote;         /* whether to quote the filename */
-    pool_pointer j;             /* index into |str_pool| */
+    unsigned char *j;             /* index into string */
     must_quote = false;
-    if (a != 0) {
-        j = str_start_macro(a);
-        while ((!must_quote) && (j < str_start_macro(a + 1))) {
-            must_quote = (str_pool[j] == ' ');
-            incr(j);
+    if (a != NULL) {
+        j = a;
+        while ((!must_quote) && (*j)) {
+            must_quote = (*j == ' ');
+            j++;
         }
     }
-    if (n != 0) {
-        j = str_start_macro(n);
-        while ((!must_quote) && (j < str_start_macro(n + 1))) {
-            must_quote = (str_pool[j] == ' ');
-            incr(j);
+    if (n != NULL) {
+        j = n;
+        while ((!must_quote) && (*j)) {
+            must_quote = (*j == ' ');
+            j++;
         }
     }
-    if (e != 0) {
-        j = str_start_macro(e);
-        while ((!must_quote) && (j < str_start_macro(e + 1))) {
-            must_quote = (str_pool[j] == ' ');
-            incr(j);
+    if (e != NULL) {
+        j = e;
+        while ((!must_quote) && (*j)) {
+            must_quote = (*j == ' ');
+            j++;
         }
     }
     /* FIXME: Alternative is to assume that any filename that has to be quoted has
@@ -315,21 +316,32 @@ void print_file_name(str_number n, str_number a, str_number e)
 
     if (must_quote)
         print_char('"');
-    if (a != 0) {
-        for (j = str_start_macro(a); j <= str_start_macro(a + 1) - 1; j++)
-            if (str_pool[j] != '"')
-                print_char(str_pool[j]);
+    if (a != NULL) {
+        for (j = a; *j; j++)
+            if (*j != '"')
+                print_char(*j);
     }
-    if (n != 0) {
-        for (j = str_start_macro(n); j <= str_start_macro(n + 1) - 1; j++)
-            if (str_pool[j] != '"')
-                print_char(str_pool[j]);
+    if (n != NULL) {
+        for (j = n; *j; j++)
+            if (*j != '"')
+                print_char(*j);
     }
-    if (e != 0) {
-        for (j = str_start_macro(e); j <= str_start_macro(e + 1) - 1; j++)
-            if (str_pool[j] != '"')
-                print_char(str_pool[j]);
+    if (e != NULL) {
+        for (j = e; *j; j++)
+            if (*j != '"')
+                print_char(*j);
     }
     if (must_quote)
         print_char('"');
 }
+
+
+void print_file_name(str_number n, str_number a, str_number e)
+{
+    char *nam, *are, *ext;
+    nam = makecstring(n);
+    are = makecstring(a);
+    ext = makecstring(e);
+    tprint_file_name((unsigned char *)nam, (unsigned char *)are, (unsigned char *)ext);
+    free(nam); free(are); free(ext);
+}
