@@ -22,7 +22,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: inputstack.c 2875 2009-07-16 15:29:20Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/inputstack.c $";
+    "$Id: inputstack.c 3221 2009-12-04 10:49:35Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/inputstack.c $";
 
 
 #define end_line_char int_par(end_line_char_code)
@@ -41,7 +41,7 @@ alpha_file *input_file = NULL;
 integer line = 0;               /* current line number in the current source file */
 integer *line_stack = NULL;
 str_number *source_filename_stack = NULL;
-str_number *full_source_filename_stack = NULL;
+char **full_source_filename_stack = NULL;
 
 
 integer scanner_status = 0;     /* can a subfile end now? */
@@ -544,7 +544,7 @@ void begin_file_reading(void)
     push_input();
     iindex = in_open;
     source_filename_stack[iindex] = 0;
-    full_source_filename_stack[iindex] = 0;
+    full_source_filename_stack[iindex] = NULL;
     eof_seen[iindex] = false;
     grp_stack[iindex] = cur_boundary;
     if_stack[iindex] = cond_ptr;
@@ -601,7 +601,7 @@ void initialize_inputstack(void)
     max_in_stack = 0;
     source_filename_stack[0] = 0;
 
-    full_source_filename_stack[0] = 0;
+    full_source_filename_stack[0] = NULL;
     in_open = 0;
     open_parens = 0;
     max_buf_stack = 0;
@@ -649,6 +649,51 @@ all the following words contain ASCII codes.
 
 halfword pseudo_files;          /* stack of pseudo files */
 
+static halfword string_to_pseudo(str_number str, integer nl)
+{
+    halfword i, r, q = null;
+    unsigned l, len;
+    four_quarters w;
+    int sz;
+    halfword h = new_node(pseudo_file_node, 0);
+    unsigned char *s = str_string(str);
+    len =  str_length(str);
+    l = 0;
+    while (l<len) {
+        unsigned m = l; /* start of current line */
+        while ((l < len) && (s[l] != nl))
+            l++;
+        sz = (l - m + 7) / 4;
+        if (sz == 1)
+            sz = 2;
+        r = new_node(pseudo_line_node, sz);
+        i = r;
+        while (--sz > 1) {
+            w.b0 = s[m++];
+            w.b1 = s[m++];
+            w.b2 = s[m++];
+            w.b3 = s[m++];
+            varmem[++i].qqqq = w;
+        }
+        w.b0 = (l > m ? s[m++] : ' ');
+        w.b1 = (l > m ? s[m++] : ' ');
+        w.b2 = (l > m ? s[m++] : ' ');
+        w.b3 = (l > m ? s[m] : ' ');
+        varmem[++i].qqqq = w;
+        if (pseudo_lines(h) == null) {
+            pseudo_lines(h) = r;
+            q = r;
+        } else {
+            couple_nodes(q, r);
+        }
+        q = vlink(q);
+        if (s[l] == nl)
+            l++;
+    }
+    return h;
+}
+
+
 /* The |pseudo_start| procedure initiates reading from a pseudo file. */
 
 void pseudo_from_string(void)
@@ -657,13 +702,10 @@ void pseudo_from_string(void)
     halfword p;                 /* for list construction */
     s = make_string();
     /* Convert string |s| into a new pseudo file */
-    str_pool[pool_ptr] = ' ';
-    p = string_to_pseudo(str_start_macro(s), pool_ptr,
-                         int_par(new_line_char_code));
+    p = string_to_pseudo(s, int_par(new_line_char_code));
     vlink(p) = pseudo_files;
     pseudo_files = p;
-
-    flush_string();
+    flush_str(s);
     /* Initiate input from new pseudo file */
     begin_file_reading();       /* set up |cur_file| and new level of input */
     line = 0;
