@@ -20,8 +20,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: packaging.c 3057 2009-10-08 11:47:45Z taco $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/packaging.c $";
+    "$Id: packaging.c 3203 2009-12-01 10:18:20Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/packaging.c $";
 
 #define scan_normal_dimen() scan_dimen(false,false,false)
 
@@ -64,9 +64,6 @@ box containing the vlist that starts at |p|. In this case |w| represents
 a height instead of a width; the parameter |m| is interpreted as in |hpack|.
 */
 
-integer pack_direction;
-integer spec_direction;
-
 /*
 The parameters to |hpack| and |vpack| correspond to \TeX's primitives
 like `\.{\\hbox} \.{to} \.{300pt}', `\.{\\hbox} \.{spread} \.{10pt}'; note
@@ -80,18 +77,39 @@ $$\vbox{\halign{#\hfil\cr
 |hpack(p,saved_value(0),saved_level(0)).|\cr}}$$
 */
 
-/*
-Special care is necessary to ensure that the special |save_stack| codes
-are placed just below the new group code, because scanning can change
-|save_stack| when \.{\\csname} appears.
+void scan_spec(group_code c)
+{                               /* scans a box specification and left brace */
+    int spec_code;
+    if (scan_keyword("to")) {
+        spec_code = exactly;
+        scan_normal_dimen();
+    } else if (scan_keyword("spread")) {
+        spec_code = additional;
+        scan_normal_dimen();
+    } else {
+        spec_code = additional;
+        cur_val = 0;
+    }
+    set_saved_record(0, saved_boxspec, spec_code, cur_val);
+    save_ptr++;
+    new_save_level(c);
+    scan_left_brace();
+}
 
-This is signaled by the |three_codes| argument, which coincidentally can
-be used as a flag to decide on scanning |dir| and |attr| keywords, as these
-are exaclty the uses of \.{\\hbox}, \.{\\vbox}, and \.{\\vtop} in the input
-stream (the others are \.{\\vcenter}, \.{\\valign}, and \.{\\halign}).
+/*
+
+When scanning, special care is necessary to ensure that the special
+|save_stack| codes are placed just below the new group code, because
+scanning can change |save_stack| when \.{\\csname} appears.  
+
+This coincides with the text on |dir| and |attr| keywords, as these 
+are exaclty the uses of \.{\\hbox}, \.{\\vbox}, and \.{\\vtop} in the
+input stream (the others are \.{\\vcenter}, \.{\\valign}, and
+\.{\\halign}).
 */
 
-void scan_spec(group_code c, boolean three_codes)
+
+void scan_full_spec(group_code c, integer spec_direction)
 {                               /* scans a box specification and left brace */
     integer s;                  /* temporarily saved value */
     integer i;
@@ -102,36 +120,34 @@ void scan_spec(group_code c, boolean three_codes)
     if (attr_list_cache == cache_disabled)
         update_attribute_cache();
     attr_list = attr_list_cache;
-    if (three_codes) {
-        assert(saved_type(0) == saved_boxcontext);
-        s = saved_value(0);     /* the box context */
-      CONTINUE:
-        while (cur_cmd == relax_cmd || cur_cmd == spacer_cmd) {
-            get_x_token();
-            if (cur_cmd != relax_cmd && cur_cmd != spacer_cmd)
-                back_input();
-        }
-        if (scan_keyword("attr")) {
-            scan_register_num();
-            i = cur_val;
-            scan_optional_equals();
-            scan_int();
-            v = cur_val;
-            if ((attr_list != null) && (attr_list == attr_list_cache)) {
-                attr_list = copy_attribute_list(attr_list_cache);
-                add_node_attr_ref(attr_list);   /* will be used once */
-            }
-            attr_list = do_set_attribute(attr_list, i, v);
-            goto CONTINUE;
-        }
-        if (scan_keyword("dir")) {
-            scan_direction();
-            spec_direction = cur_val;
-            goto CONTINUE;
-        }
-        if (attr_list == attr_list_cache) {
-            add_node_attr_ref(attr_list);
+    assert(saved_type(0) == saved_boxcontext);
+    s = saved_value(0);         /* the box context */
+  CONTINUE:
+    while (cur_cmd == relax_cmd || cur_cmd == spacer_cmd) {
+        get_x_token();
+        if (cur_cmd != relax_cmd && cur_cmd != spacer_cmd)
+            back_input();
+    }
+    if (scan_keyword("attr")) {
+        scan_register_num();
+        i = cur_val;
+        scan_optional_equals();
+        scan_int();
+        v = cur_val;
+        if ((attr_list != null) && (attr_list == attr_list_cache)) {
+            attr_list = copy_attribute_list(attr_list_cache);
+            add_node_attr_ref(attr_list);       /* will be used once */
         }
+        attr_list = do_set_attribute(attr_list, i, v);
+        goto CONTINUE;
+    }
+    if (scan_keyword("dir")) {
+        scan_direction();
+        spec_direction = cur_val;
+        goto CONTINUE;
+    }
+    if (attr_list == attr_list_cache) {
+        add_node_attr_ref(attr_list);
     }
     if (scan_keyword("to")) {
         spec_code = exactly;
@@ -144,33 +160,25 @@ void scan_spec(group_code c, boolean three_codes)
     }
     scan_normal_dimen();
   FOUND:
-    if (three_codes) {
-        set_saved_record(0, saved_boxcontext, 0, s);
-        set_saved_record(1, saved_boxspec, spec_code, cur_val);
-        /* DIR: Adjust |text_dir_ptr| for |scan_spec| */
-        if (spec_direction != -1) {
-            set_saved_record(2, saved_boxdir, spec_direction, text_dir_ptr);
-            text_dir_ptr = new_dir(spec_direction);
-        } else {
-            set_saved_record(2, saved_boxdir, spec_direction, null);
-        }
-        set_saved_record(3, saved_boxattr, 0, attr_list);
-        save_ptr += 4;
-        new_save_level(c);
-        scan_left_brace();
-        eq_word_define(int_base + body_direction_code, spec_direction);
-        eq_word_define(int_base + par_direction_code, spec_direction);
-        eq_word_define(int_base + text_direction_code, spec_direction);
-        eq_word_define(int_base + level_local_dir_code, cur_level);
+    set_saved_record(0, saved_boxcontext, 0, s);
+    set_saved_record(1, saved_boxspec, spec_code, cur_val);
+    /* DIR: Adjust |text_dir_ptr| for |scan_spec| */
+    if (spec_direction != -1) {
+        set_saved_record(2, saved_boxdir, spec_direction, text_dir_ptr);
+        text_dir_ptr = new_dir(spec_direction);
     } else {
-        set_saved_record(0, saved_boxspec, spec_code, cur_val);
-        save_ptr++;
-        new_save_level(c);
-        scan_left_brace();
+        set_saved_record(2, saved_boxdir, spec_direction, null);
     }
-    spec_direction = -1;
+    set_saved_record(3, saved_boxattr, 0, attr_list);
+    save_ptr += 4;
+    new_save_level(c);
+    scan_left_brace();
+    eq_word_define(int_base + body_direction_code, spec_direction);
+    eq_word_define(int_base + par_direction_code, spec_direction);
+    eq_word_define(int_base + text_direction_code, spec_direction);
 }
 
+
 /*
 To figure out the glue setting, |hpack| and |vpack| determine how much
 stretchability and shrinkability are present, considering all four orders
@@ -400,7 +408,7 @@ Here is |hpack|, which is place where we do font substituting when
 font expansion is being used.
 */
 
-halfword hpack(halfword p, scaled w, int m)
+halfword hpack(halfword p, scaled w, int m, int pack_direction)
 {
     halfword r;                 /* the box node that will be returned */
     halfword q;                 /* trails behind |p| */
@@ -480,7 +488,7 @@ halfword hpack(halfword p, scaled w, int m)
                 }
             }
             f = font(p);
-            whd = pack_width_height_depth(hpack_dir, glyph_dir, p, true);
+            whd = pack_width_height_depth(hpack_dir, dir_TRT, p, true);
             x += whd.wd;
             if (whd.ht > h)
                 h = whd.ht;
@@ -811,27 +819,28 @@ halfword hpack(halfword p, scaled w, int m)
         q = list_ptr(r);
         list_ptr(r) = null;
         flush_node(r);
-        r = hpack(q, w, subst_ex_font);
+        r = hpack(q, w, subst_ex_font, hpack_dir);
     }
     while (dir_ptr != null)
         pop_dir_node();
     return r;
 }
 
-halfword filtered_hpack(halfword p, halfword qt, scaled w, int m, integer grp)
+halfword filtered_hpack(halfword p, halfword qt, scaled w, int m, integer grp,
+                        int pac)
 {
     halfword q;
     new_hyphenation(p, qt);
     (void) new_ligkern(p, qt);  /* don't care about the tail in this case */
     q = vlink(p);
-    q = lua_hpack_filter(q, w, m, grp);
-    return hpack(q, w, m);
+    q = lua_hpack_filter(q, w, m, grp, pac);
+    return hpack(q, w, m, pac);
 }
 
 /* here is a function to calculate the natural whd of a (horizontal) node list */
 
 scaled_whd natural_sizes(halfword p, halfword pp, glue_ratio g_mult,
-                         integer g_sign, integer g_order)
+                         integer g_sign, integer g_order, int pack_direction)
 {
     scaled s;                   /* shift amount */
     halfword g;                 /* points to a glue specification */
@@ -845,9 +854,9 @@ scaled_whd natural_sizes(halfword p, halfword pp, glue_ratio g_mult,
         hpack_dir = pack_direction;
     }
     while (p != pp && p != null) {
-        while (is_char_node(p)) {
+        while (is_char_node(p) && p != pp) {
             f = font(p);
-            whd = pack_width_height_depth(hpack_dir, glyph_dir, p, true);
+            whd = pack_width_height_depth(hpack_dir, dir_TRT, p, true);
             siz.wd += whd.wd;
             if (whd.ht > siz.ht)
                 siz.ht = whd.ht;
@@ -919,7 +928,8 @@ scaled_whd natural_sizes(halfword p, halfword pp, glue_ratio g_mult,
                 siz.wd += surround(p);
                 break;
             case disc_node:
-                xx = natural_sizes(no_break(p), null, g_mult, g_sign, g_order);
+                xx = natural_sizes(no_break(p), null, g_mult, g_sign, g_order,
+                                   hpack_dir);
                 siz.wd += xx.wd;
                 if (xx.ht > siz.ht)
                     siz.ht = xx.ht;
@@ -955,7 +965,7 @@ computed by the normal rules; if it exceeds this limit, the reference
 point is simply moved down until the limiting depth is attained.
 */
 
-halfword vpackage(halfword p, scaled h, int m, scaled l)
+halfword vpackage(halfword p, scaled h, int m, scaled l, int pack_direction)
 {
     halfword r;                 /* the box node that will be returned */
     scaled w, d, x;             /* width, depth, and natural height */
@@ -969,7 +979,6 @@ halfword vpackage(halfword p, scaled h, int m, scaled l)
         box_dir(r) = body_direction;
     } else {
         box_dir(r) = pack_direction;
-        pack_direction = -1;
     }
     subtype(r) = min_quarterword;
     shift_amount(r) = 0;
@@ -1187,12 +1196,13 @@ halfword vpackage(halfword p, scaled h, int m, scaled l)
     return r;
 }
 
-halfword filtered_vpackage(halfword p, scaled h, int m, scaled l, integer grp)
+halfword filtered_vpackage(halfword p, scaled h, int m, scaled l, integer grp,
+                           int pack_direction)
 {
     halfword q;
     q = p;
-    q = lua_vpack_filter(q, h, m, l, grp);
-    return vpackage(q, h, m, l);
+    q = lua_vpack_filter(q, h, m, l, grp, pack_direction);
+    return vpackage(q, h, m, l, pack_direction);
 }
 
 void finish_vcenter(void)
@@ -1200,7 +1210,7 @@ void finish_vcenter(void)
     halfword p;
     unsave();
     save_ptr--;
-    p = vpack(vlink(cur_list.head_field), saved_value(0), saved_level(0));
+    p = vpack(vlink(cur_list.head_field), saved_value(0), saved_level(0), -1);
     pop_nest();
     p = math_vcenter_group(p);
     tail_append(p);
@@ -1216,15 +1226,15 @@ void package(int c)
     d = box_max_depth;
     unsave();
     save_ptr -= 4;
-    pack_direction = saved_level(2);
     if (cur_list.mode_field == -hmode) {
         cur_box = filtered_hpack(cur_list.head_field,
                                  cur_list.tail_field, saved_value(1),
-                                 saved_level(1), grp);
+                                 saved_level(1), grp, saved_level(2));
         subtype(cur_box) = HLIST_SUBTYPE_HBOX;
     } else {
         cur_box = filtered_vpackage(vlink(cur_list.head_field),
-                                    saved_value(1), saved_level(1), d, grp);
+                                    saved_value(1), saved_level(1), d, grp,
+                                    saved_level(2));
         if (c == vtop_code) {
             /* Readjust the height and depth of |cur_box|,  for \.{\\vtop} */
             /* The height of a `\.{\\vtop}' box is inherited from the first item on its list,
@@ -1634,15 +1644,15 @@ halfword vsplit(halfword n, scaled h)
     p = list_ptr(v);
     list_ptr(v) = null;
     flush_node(v);
-    pack_direction = vdir;
     if (q == null)
         box(n) = null;          /* the |eq_level| of the box stays the same */
     else
         box(n) =
             filtered_vpackage(q, 0, additional, dimen_par(max_depth_code),
-                              split_keep_group);
+                              split_keep_group, vdir);
     return filtered_vpackage(p, h, exactly,
-                             dimen_par(split_max_depth_code), split_off_group);
+                             dimen_par(split_max_depth_code), split_off_group,
+                             vdir);
 }
 
 /*
@@ -1657,6 +1667,7 @@ void begin_box(integer box_context)
     halfword q;                 /* run through the current list */
     halfword k;                 /* 0 or |vmode| or |hmode| */
     integer n;                  /* a box number */
+    integer spec_direction = -1;
     switch (cur_chr) {
     case box_code:
         scan_register_num();
@@ -1734,14 +1745,14 @@ void begin_box(integer box_context)
         }
         if (k == hmode) {
             if ((box_context < box_flag) && (abs(cur_list.mode_field) == vmode))
-                scan_spec(adjusted_hbox_group, true);
+                scan_full_spec(adjusted_hbox_group, spec_direction);
             else
-                scan_spec(hbox_group, true);
+                scan_full_spec(hbox_group, spec_direction);
         } else {
             if (k == vmode) {
-                scan_spec(vbox_group, true);
+                scan_full_spec(vbox_group, spec_direction);
             } else {
-                scan_spec(vtop_group, true);
+                scan_full_spec(vtop_group, spec_direction);
                 k = vmode;
             }
             normal_paragraph();
