@@ -23,7 +23,7 @@
 
 
 static const char _svn_version[] =
-    "$Id: postlinebreak.c 3057 2009-10-08 11:47:45Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/postlinebreak.c $";
+    "$Id: postlinebreak.c 3212 2009-12-02 11:07:20Z taco $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/postlinebreak.c $";
 
 /* So far we have gotten a little way into the |line_break| routine, having
 covered its important |try_break| subroutine. Now let's consider the
@@ -151,6 +151,9 @@ void ext_post_line_break(boolean d,
         for (q = dir_ptr; q != null; q = vlink(q)) {
             halfword tmp = new_dir(dir_dir(q));
             halfword nxt = vlink(temp_head);
+            delete_attribute_ref(node_attr(tmp));
+            node_attr(tmp) = node_attr(temp_head);
+            add_node_attr_ref (node_attr(tmp));
             couple_nodes(temp_head, tmp);
             try_couple_nodes(tmp, nxt); /* \break\par */
         }
@@ -280,6 +283,9 @@ void ext_post_line_break(boolean d,
             e = vlink(r);
             for (p = dir_ptr; p != null; p = vlink(p)) {
                 halfword s = new_dir(dir_dir(p) - 64);
+                delete_attribute_ref(node_attr(s));
+                node_attr(s) = node_attr(r);
+                add_node_attr_ref (node_attr(s));
                 couple_nodes(r, s);
                 try_couple_nodes(s, e);
                 r = s;
@@ -314,11 +320,14 @@ void ext_post_line_break(boolean d,
                 p = alink(q);   /* get |vlink(p) = q| */
                 assert(vlink(p) == q);
                 ptmp = p;
-                p = find_protchar_right(vlink(temp_head), p);
             }
+            p = find_protchar_right(vlink(temp_head), p);
             w = char_pw(p, right_side);
             if (w != 0) {       /* we have found a marginal kern, append it after |ptmp| */
                 k = new_margin_kern(-w, last_rightmost_char, right_side);
+                delete_attribute_ref(node_attr(k));
+                node_attr(k) = node_attr(p);
+                add_node_attr_ref (node_attr(k));
                 vlink(k) = vlink(ptmp);
                 vlink(ptmp) = k;
                 if (ptmp == q)
@@ -331,6 +340,9 @@ void ext_post_line_break(boolean d,
             /* @<Put the \(r)\.{\\rightskip} glue after node |q|@>; */
             halfword r = new_param_glue(right_skip_code);
             vlink(r) = vlink(q);
+            delete_attribute_ref(node_attr(r));
+            node_attr(r) = node_attr(q);
+            add_node_attr_ref (node_attr(r));
             vlink(q) = r;
             q = r;
         }
@@ -370,12 +382,18 @@ void ext_post_line_break(boolean d,
             w = char_pw(p, left_side);
             if (w != 0) {
                 k = new_margin_kern(-w, last_leftmost_char, left_side);
+                delete_attribute_ref(node_attr(k));
+                node_attr(k) = node_attr(q);
+                add_node_attr_ref (node_attr(k));
                 vlink(k) = q;
                 q = k;
             }
         };
         if (left_skip != zero_glue) {
             r = new_param_glue(left_skip_code);
+            delete_attribute_ref(node_attr(r));
+            node_attr(r) = node_attr(q);
+            add_node_attr_ref (node_attr(r));
             vlink(r) = q;
             q = r;
         }
@@ -397,12 +415,11 @@ void ext_post_line_break(boolean d,
             cur_width = varmem[(par_shape_ptr + 2 * cur_line + 1)].cint;
         }
         adjust_tail = adjust_head;
-        pack_direction = paragraph_dir;
         pre_adjust_tail = pre_adjust_head;
         if (pdf_adjust_spacing > 0) {
-            just_box = hpack(q, cur_width, cal_expand_ratio);
+            just_box = hpack(q, cur_width, cal_expand_ratio, paragraph_dir);
         } else {
-            just_box = hpack(q, cur_width, exactly);
+            just_box = hpack(q, cur_width, exactly, paragraph_dir);
         }
         shift_amount(just_box) = cur_indent;
         subtype(just_box) = HLIST_SUBTYPE_LINE;
