@@ -21,8 +21,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: texmath.c 3080 2009-10-25 22:55:06Z oneiros $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/texmath.c $";
+    "$Id: texmath.c 3217 2009-12-03 16:52:01Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/texmath.c $";
 
 #define mode          cur_list.mode_field
 #define head          cur_list.head_field
@@ -610,7 +610,7 @@ distinguished from a missing field, because these are not equivalent
 
 void print_subsidiary_data(pointer p, ASCII_code c)
 {                               /* display a noad field */
-    if (cur_length >= depth_threshold) {
+    if ((int)cur_length >= depth_threshold) {
         if (p != null)
             tprint(" []");
     } else {
@@ -786,7 +786,6 @@ void new_save_level_math(group_code c)
     eq_word_define(int_base + body_direction_code, math_direction);
     eq_word_define(int_base + par_direction_code, math_direction);
     eq_word_define(int_base + text_direction_code, math_direction);
-    eq_word_define(int_base + level_local_dir_code, cur_level);
 }
 
 void push_math(group_code c, int mstyle)
@@ -1956,9 +1955,7 @@ static boolean math_and_text_reversed_p(void)
     while (i < save_ptr) {
         if (save_type(i) == restore_old_value &&
             save_value(i) == int_base + par_direction_code) {
-            if (dir_opposite
-                (dir_secondary[math_direction],
-                 dir_secondary[save_value(i - 1)]))
+            if (textdir_opposite(math_direction, save_value(i - 1)))
                 return true;
         }
         i++;
@@ -1998,7 +1995,7 @@ static void finish_displayed_math(boolean l, pointer a, pointer p)
 
     adjust_tail = adjust_head;
     pre_adjust_tail = pre_adjust_head;
-    eq_box = hpack(p, 0, additional);
+    eq_box = hpack(p, 0, additional, -1);
     p = list_ptr(eq_box);
     t = adjust_tail;
     adjust_tail = null;
@@ -2024,13 +2021,13 @@ static void finish_displayed_math(boolean l, pointer a, pointer p)
                 || (total_shrink[filll] != 0))) {
             list_ptr(eq_box) = null;
             flush_node(eq_box);
-            eq_box = hpack(p, line_w - eqno_w2, exactly);
+            eq_box = hpack(p, line_w - eqno_w2, exactly, -1);
         } else {
             eqno_w = 0;
             if (eq_w > line_w) {
                 list_ptr(eq_box) = null;
                 flush_node(eq_box);
-                eq_box = hpack(p, line_w, exactly);
+                eq_box = hpack(p, line_w, exactly, -1);
             }
         }
         eq_w = width(eq_box);
@@ -2091,7 +2088,7 @@ static void finish_displayed_math(boolean l, pointer a, pointer p)
             vlink(eq_box) = r;
             vlink(r) = a;
         }
-        eq_box = hpack(eq_box, 0, additional);
+        eq_box = hpack(eq_box, 0, additional, -1);
     }
     if (swap_dir) {
         /*    d = line_w - d; */
@@ -2155,7 +2152,7 @@ void after_math(void)
             check_display_math_end();
         }
         run_mlist_to_hlist(p, text_style, false);
-        a = hpack(vlink(temp_head), 0, additional);
+        a = hpack(vlink(temp_head), 0, additional, -1);
         unsave_math();
         decr(save_ptr);         /* now |cur_group=math_shift_group| */
         assert(saved_type(0) == saved_eqno);
