@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: texnodes.h 3057 2009-10-08 11:47:45Z taco $ */
+/* $Id: texnodes.h 3217 2009-12-03 16:52:01Z taco $ */
 
 #include <stdarg.h>
 
@@ -221,7 +221,7 @@ typedef enum {
 #  define y_displace(a)   vlink((a)+4)
 #  define is_char_node(a) (a!=null && type(a)==glyph_node)
 
-#  define char_lang(a)     ((const int)(((unsigned)lang_data(a) & 0x7FFF0000)>>16))
+#  define char_lang(a)     ((const int)(signed short)(((signed int)((unsigned)lang_data(a)&0x7FFF0000)<<1)>>17))
 #  define char_lhmin(a)    ((const int)(((unsigned)lang_data(a) & 0x0000FF00)>>8))
 #  define char_rhmin(a)    ((const int)(((unsigned)lang_data(a) & 0x000000FF)))
 #  define char_uchyph(a)   ((const int)(((unsigned)lang_data(a) & 0x80000000)>>31))
@@ -578,10 +578,10 @@ destination |pdf_ann_left| and |pdf_ann_top| are used for some types of destinat
 #  define pdf_literal_mode(a)  type((a)+2)
 #  define pdf_literal_type(a)  subtype((a)+2)
 
-#  define pdf_ximage_idx(a)    vinfo((a) + 5) /* image index in array */        /* replaces next? */
-#  define pdf_ximage_objnum(a) vinfo((a) + 5)   /* this will be removed soon */
-#  define pdf_ximage_ref(a)    vinfo((a) + 5)
-#  define pdf_xform_objnum(a)  vinfo((a) + 5)
+#  define pdf_ximage_index(a)     vinfo((a) + 5)        /* /Im* number and image index in array */
+#  define pdf_ximage_transform(a) vlink((a) + 5)
+#  define pdf_xform_objnum(a)     vinfo((a) + 5)
+#  define pdf_xform_transform(a)  vlink((a) + 5)
 
 #  define pdf_annot_data(a)       vinfo((a) + 6)
 #  define pdf_link_attr(a)        vinfo((a) + 6)
@@ -667,7 +667,6 @@ extern void set_attribute(halfword n, int c, int w);
 extern int has_attribute(halfword n, int c, int w);
 
 extern halfword new_span_node(halfword n, int c, scaled w);
-extern halfword string_to_pseudo(integer l, integer pool_ptr, integer nl);
 
 extern void print_short_node_contents(halfword n);
 extern void show_node_list(integer i);
