@@ -17,12 +17,12 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
+static const char _svn_version[] =
+    "$Id: textoken.c 3248 2009-12-11 13:39:41Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.47.0/source/texk/web2c/luatexdir/tex/textoken.c $";
 
 #include <ptexlib.h>
 
-static const char _svn_version[] =
-    "$Id: textoken.c 3005 2009-08-16 20:48:36Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/textoken.c $";
-
 #define pausing int_par(pausing_code)
 #define cat_code_table int_par(cat_code_table_code)
 #define tracing_nesting int_par(tracing_nesting_code)
@@ -41,7 +41,6 @@ static const char _svn_version[] =
       a=get_cat_code(cat_code_table,cur_chr);                           \
   } while (0)
 
-
 /* 
 The \TeX\ system does nearly all of its own memory allocation, so that it
 can readily be transported into environments that do not have automatic
@@ -318,6 +317,8 @@ void show_token_list(integer p, integer q, integer l)
     match_chr = '#';
     n = '0';
     tally = 0;
+    if (l<0)
+        l = 0x3FFFFFFF;
     while ((p != null) && (tally < l)) {
         if (p == q) {
             /* Do magic computation */
@@ -427,17 +428,6 @@ void delete_token_ref(halfword p)
 }
 
 
-/* string compare */
-
-boolean str_eq_cstr(str_number r, char *s, size_t l)
-{
-    if (l != (size_t) str_length(r))
-        return false;
-    return (strncmp((const char *) (str_pool + str_start_macro(r)), s, l) == 0);
-}
-
-
-
 int get_char_cat_code(int cur_chr)
 {
     int a;
@@ -527,7 +517,8 @@ boolean scan_keyword(char *s)
         }
         flush_list(token_link(backup_head));
     }
-    return true;
+   cur_cs = save_cur_cs;
+   return true;
 }
 
 /* We can not return |undefined_control_sequence| under some conditions
@@ -563,10 +554,10 @@ halfword active_to_cs(int curchr, int force)
 
 char *cs_to_string(halfword p)
 {                               /* prints a control sequence */
-    char *s;
+    char *s, *sh;
     int k = 0;
     static char ret[256] = { 0 };
-    if (p == null_cs) {
+    if (p == 0 || p == null_cs) {
         ret[k++] = '\\';
         s = "csname";
         while (*s) {
@@ -581,7 +572,8 @@ char *cs_to_string(halfword p)
 
     } else {
         str_number txt = cs_text(p);
-        s = makecstring(txt);
+        sh = makecstring(txt);
+        s = sh;
         if (is_active_cs(txt)) {
             s = s + 3;
             while (*s) {
@@ -595,6 +587,7 @@ char *cs_to_string(halfword p)
             }
             ret[k] = 0;
         }
+        free(sh);
     }
     return (char *) ret;
 }
@@ -797,13 +790,16 @@ void check_outer_validity(void)
                 NULL
             };
             char **errhlp = (char **) errhlp_no;
+            char *ss;
             if (cur_cs != 0) {
                 errhlp = errhlp_cs;
                 cur_cs = 0;
             }
+            ss = cmd_chr_to_string(if_test_cmd, cur_if);
             snprintf(errmsg, 255,
                      "Incomplete %s; all text was ignored after line %d",
-                     cmd_chr_to_string(if_test_cmd, cur_if), (int) skip_line);
+                     ss, (int) skip_line);
+            free(ss);
             /* @.Incomplete \\if...@> */
             cur_tok = cs_token_flag + frozen_fi;
             /* back up one inserted token and call |error| */
@@ -1544,16 +1540,16 @@ at the value |p| that is returned. (If |p=temp_token_head|, the list is empty.)
 symbols that |lua| considers special while scanning a literal string
 */
 
-halfword lua_str_toks(pool_pointer b)
+static halfword lua_str_toks(lstring b)
 {                               /* changes the string |str_pool[b..pool_ptr]| to a token list */
     halfword p;                 /* tail of the token list */
     halfword q;                 /* new node being added to the token list via |store_new_token| */
     halfword t;                 /* token being appended */
-    pool_pointer k;             /* index into |str_pool| */
+    unsigned char *k;             /* index into string */
     p = temp_token_head;
     set_token_link(p, null);
-    k = b;
-    while (k < pool_ptr) {
+    k = (unsigned char *)b.s;
+    while (k < (unsigned char *)b.s+b.l) {
         t = pool_to_unichar(k);
         k += utf8_size(t);
         if (t == ' ') {
@@ -1570,7 +1566,6 @@ halfword lua_str_toks(pool_pointer b)
         }
         fast_store_new_token(t);
     }
-    pool_ptr = b;
     return p;
 }
 
@@ -1580,16 +1575,17 @@ which has similar input/output characteristics.
 */
 
 
-halfword str_toks(pool_pointer b)
+halfword str_toks(lstring s)
 {                               /* changes the string |str_pool[b..pool_ptr]| to a token list */
     halfword p;                 /* tail of the token list */
     halfword q;                 /* new node being added to the token list via |store_new_token| */
     halfword t;                 /* token being appended */
-    pool_pointer k;             /* index into |str_pool| */
+    unsigned char *k, *l;       /* index into string */
     p = temp_token_head;
     set_token_link(p, null);
-    k = b;
-    while (k < pool_ptr) {
+    k = s.s;
+    l = k + s.l;
+    while (k<l) {
         t = pool_to_unichar(k);
         k += utf8_size(t);
         if (t == ' ')
@@ -1598,7 +1594,6 @@ halfword str_toks(pool_pointer b)
             t = other_token + t;
         fast_store_new_token(t);
     }
-    pool_ptr = b;
     return p;
 }
 
@@ -1661,7 +1656,7 @@ static boolean print_convert_string(halfword c, integer i)
         print(job_name);
         break;
     case font_name_code:
-        append_string(font_name(i));
+        append_string((unsigned char *)font_name(i),strlen(font_name(i)));
         if (font_size(i) != font_dsize(i)) {
             tprint(" at ");
             print_scaled(font_size(i));
@@ -1738,7 +1733,7 @@ any pending string in its output. In order to save such a pending string,
 we have to create a temporary string that is destroyed immediately after.
 */
 
-#define save_cur_string() if (str_start_macro(str_ptr)<pool_ptr)  u=make_string()
+#define save_cur_string() if (cur_length>0)  u=make_string()
 #define restore_cur_string() if (u!=0) { decr(str_ptr); u=0; }
 
 void conv_toks(void)
@@ -1749,13 +1744,13 @@ void conv_toks(void)
     halfword save_def_ref;      /* |def_ref| upon entry, important if inside `\.{\\message}' */
     halfword save_warning_index;
     boolean bool;               /* temp boolean */
-    pool_pointer b;             /* base of temporary string */
     str_number s;               /* first temp string */
     integer sn;                 /* lua chunk name */
     str_number u = 0;           /* third temp string, will become non-nil if a string is already being built */
     integer i = 0;              /* first temp integer */
     integer j = 0;              /* second temp integer */
     int c = cur_chr;            /* desired type of conversion */
+    str_number str;
     /* Scan the argument for command |c| */
     switch (c) {
     case number_code:
@@ -1806,7 +1801,7 @@ void conv_toks(void)
         break;
     case pdf_xform_name_code:
         scan_int();
-        pdf_check_obj(static_pdf, obj_type_xform, cur_val);
+        check_obj_exists(static_pdf, obj_type_xform, cur_val);
         break;
     case pdf_creation_date_code:
         ins_list(string_to_toks(getcreationdate(static_pdf)));
@@ -1853,25 +1848,27 @@ void conv_toks(void)
     case normal_deviate_code:
         break;
     case lua_escape_string_code:
-        /*  check if a string is already being built */
-        save_cur_string();
-        save_scanner_status = scanner_status;
-        save_def_ref = def_ref;
-        save_warning_index = warning_index;
-        scan_pdf_ext_toks();
-        bool = in_lua_escape;
-        in_lua_escape = true;
-        s = tokens_to_string(def_ref);
-        in_lua_escape = bool;
-        delete_token_ref(def_ref);
-        def_ref = save_def_ref;
-        warning_index = save_warning_index;
-        scanner_status = save_scanner_status;
-        (void) lua_str_toks(str_start_macro(s));
-        ins_list(token_link(temp_token_head));
-        flush_str(s);
-        restore_cur_string();
-        return;
+        {
+            lstring str;
+	    int l = 0;
+            save_scanner_status = scanner_status;
+            save_def_ref = def_ref;
+            save_warning_index = warning_index;
+            scan_pdf_ext_toks();
+            bool = in_lua_escape;
+            in_lua_escape = true;
+            str.s = (unsigned char *)tokenlist_to_cstring(def_ref, false, &l);
+	    str.l = (unsigned)l;
+            in_lua_escape = bool;
+            delete_token_ref(def_ref);
+            def_ref = save_def_ref;
+            warning_index = save_warning_index;
+            scanner_status = save_scanner_status;
+            (void) lua_str_toks(str);
+            ins_list(token_link(temp_token_head));
+            free(str.s);
+            return;
+        }
         break;
     case math_style_code:
         break;
@@ -1912,7 +1909,7 @@ void conv_toks(void)
         break;
     case pdf_ximage_bbox_code:
         scan_int();
-        pdf_check_obj(static_pdf, obj_type_ximage, cur_val);
+        check_obj_exists(static_pdf, obj_type_ximage, cur_val);
         i = obj_data_ptr(static_pdf, cur_val);
         scan_int();
         j = cur_val;
@@ -1931,7 +1928,6 @@ void conv_toks(void)
 
     old_setting = selector;
     selector = new_string;
-    b = pool_ptr;
 
     /* Print the result of command |c| */
     if (!print_convert_string(c, cur_val)) {
@@ -2020,7 +2016,9 @@ void conv_toks(void)
     }
 
     selector = old_setting;
-    (void) str_toks(b);
+    str = make_string();
+    (void) str_toks(str_lstring(str));
+    flush_str(str);
     ins_list(token_link(temp_token_head));
 }
 
@@ -2199,7 +2197,7 @@ str_number tokens_to_string(halfword p)
                   "tokens_to_string() called while selector = new_string");
     old_setting = selector;
     selector = new_string;
-    show_token_list(token_link(p), null, pool_size - pool_ptr);
+    show_token_list(token_link(p), null, -1);
     selector = old_setting;
     return make_string();
 }
@@ -2247,7 +2245,7 @@ str_number tokens_to_string(halfword p)
   }
 
 #define is_cat_letter(a)                                                \
-  (get_char_cat_code(pool_to_unichar(str_start_macro(a))) == 11)
+    (get_char_cat_code(pool_to_unichar(str_string((a)))) == 11)
 
 /* the actual token conversion in this function is now functionally 
    equivalent to |show_token_list|, except that it always prints the
@@ -2260,7 +2258,7 @@ char *tokenlist_to_cstring(int pp, int inhibit_par, int *siz)
     register integer p, c, m;
     integer q;
     integer infop;
-    char *s;
+    char *s, *sh;
     int e;
     char *ret;
     int match_chr = '#';
@@ -2302,7 +2300,8 @@ char *tokenlist_to_cstring(int pp, int inhibit_par, int *siz)
                     Print_esc("NONEXISTENT.");
                 } else {
                     str_number txt = cs_text(q);
-                    s = makecstring(txt);
+                    sh = makecstring(txt);
+                    s = sh;
                     if (is_active_cs(txt)) {
                         s = s + 3;
                         while (*s) {
@@ -2319,6 +2318,7 @@ char *tokenlist_to_cstring(int pp, int inhibit_par, int *siz)
                             Print_char(' ');
                         }
                     }
+                    free(sh);
                 }
             }
         } else {
@@ -2381,3 +2381,19 @@ char *tokenlist_to_cstring(int pp, int inhibit_par, int *siz)
         *siz = i;
     return ret;
 }
+
+lstring *tokenlist_to_lstring(int pp, int inhibit_par)
+{
+    lstring *ret = xmalloc(sizeof(lstring));
+    ret->s = (unsigned char *)tokenlist_to_cstring(pp, inhibit_par, (int *) &(ret->l));
+    return ret;
+}
+
+void free_lstring(lstring * ls)
+{
+    if (ls == NULL)
+        return;
+    if (ls->s != NULL)
+        free(ls->s);
+    free(ls);
+}
