@@ -113,7 +113,7 @@ Tue Jul 1 15:23:00 UTC 2008
 #define synctextagfield synctex_tag_field
 #define namefield name_field
 
-#define gettexstring(a) xstrdup(makecstring(a))
+#define gettexstring(a) makecstring(a)
 /*EXTERN char *gettexstring(int n);*/
 
 /*  the macros defined below do the same job than their almost eponym
